@@ -1,12 +1,15 @@
-from static_frame.core.util import EMPTY_TUPLE
+from __future__ import annotations
+
+import typing as tp
+
 
 class Assign:
     '''
     Common base class for SeriesAssign and FrameAssign classes.
     '''
-    __slots__ = EMPTY_TUPLE
+    __slots__ = ()
 
-    INTERFACE = (
+    INTERFACE: tp.Tuple[str, ...] = (
         '__call__',
         'apply',
         )
