@@ -1,8 +1,14 @@
+from __future__ import annotations
+
 import typing as tp
+from itertools import chain
+from itertools import zip_longest
 
 import numpy as np
 
 from static_frame.core.container import ContainerBase
+from static_frame.core.container_util import index_from_optional_constructor
+from static_frame.core.container_util import iter_component_signature_bytes
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
 from static_frame.core.display import DisplayHeader
@@ -11,43 +17,51 @@ from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import ErrorInitBus
 from static_frame.core.exception import ErrorInitIndexNonUnique
 from static_frame.core.frame import Frame
+from static_frame.core.index import Index
+from static_frame.core.index_auto import IndexAutoFactoryType
 from static_frame.core.index_auto import RelabelInput
 from static_frame.core.index_base import IndexBase
-from static_frame.core.node_iter import IterNodeNoArg
 from static_frame.core.node_iter import IterNodeApplyType
+from static_frame.core.node_iter import IterNodeNoArg
 from static_frame.core.node_iter import IterNodeType
 from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.node_selector import InterfaceSelectTrio
 from static_frame.core.series import Series
 from static_frame.core.store import Store
-from static_frame.core.store import StoreConfigMap
-from static_frame.core.store import StoreConfigMapInitializer
 from static_frame.core.store_client_mixin import StoreClientMixin
+from static_frame.core.store_config import StoreConfigMap
+from static_frame.core.store_config import StoreConfigMapInitializer
 from static_frame.core.store_hdf5 import StoreHDF5
 from static_frame.core.store_sqlite import StoreSQLite
 from static_frame.core.store_xlsx import StoreXLSX
 from static_frame.core.store_zip import StoreZipCSV
+from static_frame.core.store_zip import StoreZipNPY
+from static_frame.core.store_zip import StoreZipNPZ
 from static_frame.core.store_zip import StoreZipParquet
 from static_frame.core.store_zip import StoreZipPickle
 from static_frame.core.store_zip import StoreZipTSV
-from static_frame.core.store_zip import StoreZipNPZ
+from static_frame.core.style_config import StyleConfig
 from static_frame.core.util import DEFAULT_SORT_KIND
 from static_frame.core.util import DTYPE_BOOL
 from static_frame.core.util import DTYPE_FLOAT_DEFAULT
 from static_frame.core.util import DTYPE_OBJECT
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import IndexInitializer
 from static_frame.core.util import INT_TYPES
-from static_frame.core.util import NameType
-from static_frame.core.util import NULL_SLICE
-from static_frame.core.util import PathSpecifier
-from static_frame.core.util import BoolOrBools
 from static_frame.core.util import NAME_DEFAULT
+from static_frame.core.util import NULL_SLICE
+from static_frame.core.util import ZIP_LONGEST_DEFAULT
+from static_frame.core.util import BoolOrBools
+from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import IndexConstructor
-from static_frame.core.style_config import StyleConfig
-# from static_frame.core.index_auto import IndexAutoFactory
-from static_frame.core.index_auto import IndexAutoFactoryType
+from static_frame.core.util import IndexConstructors
+from static_frame.core.util import IndexInitializer
+from static_frame.core.util import IntegerLocType
+from static_frame.core.util import NameType
+from static_frame.core.util import PathSpecifier
+from static_frame.core.util import TSortKinds
 
+if tp.TYPE_CHECKING:
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 #-------------------------------------------------------------------------------
 class FrameDeferredMeta(type):
@@ -82,7 +96,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         '_max_persist',
         )
 
-    _values_mutable: np.ndarray
+    _values_mutable: NDArrayAny
     _index: IndexBase
     _store: tp.Optional[Store]
     _config: StoreConfigMap
@@ -91,44 +105,6 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
     STATIC = False
     _NDIM: int = 1
 
-    @staticmethod
-    def _deferred_series(
-            labels: tp.Iterable[tp.Hashable],
-            *,
-            index_constructor: IndexConstructor = None,
-            ) -> Series:
-        '''
-        Return an object ``Series`` of ``FrameDeferred`` objects, based on the passed in ``labels``.
-        '''
-        # NOTE: need to accept an  IndexConstructor to support reanimating Index subtypes, IH
-        return Series.from_element(FrameDeferred,
-                index=labels,
-                dtype=DTYPE_OBJECT,
-                index_constructor=index_constructor,
-                )
-
-    @classmethod
-    def from_frames(cls,
-            frames: tp.Iterable[Frame],
-            *,
-            index_constructor: IndexConstructor = None,
-            config: StoreConfigMapInitializer = None,
-            name: NameType = None,
-            ) -> 'Bus':
-        '''Return a :obj:`Bus` from an iterable of :obj:`Frame`; labels will be drawn from :obj:`Frame.name`.
-        '''
-        try:
-            series = Series.from_items(
-                        ((f.name, f) for f in frames),
-                        dtype=DTYPE_OBJECT,
-                        name=name,
-                        index_constructor=index_constructor,
-                        )
-        except ErrorInitIndexNonUnique:
-            raise ErrorInitIndexNonUnique("Frames do not have unique names.") from None
-
-        return cls(series, config=config, own_data=True)
-
     @classmethod
     def from_items(cls,
             pairs: tp.Iterable[tp.Tuple[tp.Hashable, Frame]],
@@ -142,50 +118,58 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         Returns:
             :obj:`Bus`
         '''
-        series = Series.from_items(pairs,
-                dtype=DTYPE_OBJECT,
-                name=name,
+        frames = []
+        index = []
+        for i, f in pairs: # might be a generator
+            index.append(i)
+            frames.append(f)
+
+        return cls(frames,
+                index=index,
                 index_constructor=index_constructor,
+                name=name,
+                config=config,
                 )
-        return cls(series, config=config, own_data=True)
+
+    @classmethod
+    def from_frames(cls,
+            frames: tp.Iterable[Frame],
+            *,
+            index_constructor: IndexConstructor = None,
+            config: StoreConfigMapInitializer = None,
+            name: NameType = None,
+            ) -> 'Bus':
+        '''Return a :obj:`Bus` from an iterable of :obj:`Frame`; labels will be drawn from :obj:`Frame.name`.
+        '''
+        try:
+            return cls.from_items(((f.name, f) for f in frames),
+                    index_constructor=index_constructor,
+                    config=config,
+                    name=name,
+                    )
+        except ErrorInitIndexNonUnique:
+            raise ErrorInitIndexNonUnique('Frames do not have unique names.') from None
 
     @classmethod
     def from_dict(cls,
-            mapping: tp.Dict[tp.Hashable, tp.Any],
+            mapping: tp.Dict[tp.Hashable, Frame],
             *,
-            config: StoreConfigMapInitializer = None,
             name: NameType = None,
             index_constructor: tp.Optional[tp.Callable[..., IndexBase]] = None
             ) -> 'Bus':
-        '''Bus construction from a dictionary, where the first pair value is the index and the second is the value.
+        '''Bus construction from a mapping of labels and :obj:`Frame`.
 
         Args:
             mapping: a dictionary or similar mapping interface.
-            dtype: dtype or valid dtype specifier.
 
         Returns:
             :obj:`Bus`
         '''
-        series = Series.from_dict(mapping,
-                dtype=DTYPE_OBJECT,
-                name=name,
+        return cls(frames=mapping.values(),
+                index=mapping.keys(),
                 index_constructor=index_constructor,
+                name=name,
                 )
-        return cls(series, config=config, own_data=True)
-
-    @classmethod
-    def from_concat(cls,
-            containers: tp.Iterable['Bus'],
-            *,
-            index: tp.Optional[tp.Union[IndexInitializer, IndexAutoFactoryType]] = None,
-            name: NameType = NAME_DEFAULT,
-            ) -> 'Bus':
-        '''
-        Concatenate multiple :obj:`Bus` into a new :obj:`Bus`. All :obj:`Bus` will load all :obj:`Frame` into memory if any are deferred.
-        '''
-        # will extract .values, .index from Bus, which will correct load from Store as needed
-        series = Series.from_concat(containers, index=index, name=name)
-        return cls(series, own_data=True)
 
     @classmethod
     def from_series(cls,
@@ -200,13 +184,31 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         Create a :obj:`Bus` from a :obj:`Series` of :obj:`Frame`.
         '''
         # NOTE: this interface is for 0.9 after the default Bus no longer accepts a Series
-        return cls(series,
+        return cls(series.values,
+                index=series.index,
                 store=store,
                 config=config,
                 max_persist=max_persist,
                 own_data=own_data,
+                own_index=True,
+                name=series.name,
                 )
 
+    @classmethod
+    def from_concat(cls,
+            containers: tp.Iterable['Bus'],
+            *,
+            index: tp.Optional[tp.Union[IndexInitializer, IndexAutoFactoryType]] = None,
+            name: NameType = NAME_DEFAULT,
+            ) -> 'Bus':
+        '''
+        Concatenate multiple :obj:`Bus` into a new :obj:`Bus`. All :obj:`Bus` will load all :obj:`Frame` into memory if any are deferred.
+        '''
+        # will extract .values, .index from Bus, which will correct load from Store as needed
+        # NOTE: useful to use Series here as it handles aligned names, IndexAutoFactory, etc.
+        series = Series.from_concat(containers, index=index, name=name)
+        return cls.from_series(series, own_data=True)
+
     #---------------------------------------------------------------------------
     # constructors by data format
     @classmethod
@@ -217,10 +219,9 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             max_persist: tp.Optional[int] = None,
             index_constructor: IndexConstructor = None,
             ) -> 'Bus':
-        return cls(cls._deferred_series(
-                        store.labels(config=config),
-                        index_constructor=index_constructor,
-                        ),
+        return cls(None, # will generate FrameDeferred array
+                index=store.labels(config=config),
+                index_constructor=index_constructor,
                 store=store,
                 config=config,
                 max_persist=max_persist,
@@ -300,7 +301,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             index_constructor: IndexConstructor = None,
             ) -> 'Bus':
         '''
-        Given a file path to zipped parquet :obj:`Bus` store, return a :obj:`Bus` instance.
+        Given a file path to zipped NPZ :obj:`Bus` store, return a :obj:`Bus` instance.
 
         {args}
         '''
@@ -311,6 +312,27 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 index_constructor=index_constructor,
                 )
 
+    @classmethod
+    @doc_inject(selector='bus_constructor')
+    def from_zip_npy(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            max_persist: tp.Optional[int] = None,
+            index_constructor: IndexConstructor = None,
+            ) -> 'Bus':
+        '''
+        Given a file path to zipped NPY :obj:`Bus` store, return a :obj:`Bus` instance.
+
+        {args}
+        '''
+        store = StoreZipNPY(fp)
+        return cls._from_store(store,
+                config=config,
+                max_persist=max_persist,
+                index_constructor=index_constructor,
+                )
+
     @classmethod
     @doc_inject(selector='bus_constructor')
     def from_zip_parquet(cls,
@@ -397,51 +419,93 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 )
 
     #---------------------------------------------------------------------------
-    @doc_inject(selector='bus_init')
     def __init__(self,
-            series: Series,
+            frames: NDArrayAny | tp.Iterable[Frame | tp.Type[FrameDeferred]] | None,
             *,
+            index: IndexInitializer,
+            index_constructor: IndexConstructor = None,
+            name: NameType = NAME_DEFAULT,
             store: tp.Optional[Store] = None,
             config: StoreConfigMapInitializer = None,
             max_persist: tp.Optional[int] = None,
+            own_index: bool = False,
             own_data: bool = False,
             ):
+
         '''
         Default Bus constructor.
 
         {args}
         '''
-        if series.dtype != DTYPE_OBJECT:
-            raise ErrorInitBus(
-                    f'Series passed to initializer must have dtype object, not {series.dtype}')
-
         if max_persist is not None:
             # use an (ordered) dictionary to give use an ordered set, simply pointing to None for all keys
-            self._last_accessed: tp.Dict[str, None] = {}
+            self._last_accessed: tp.Dict[tp.Hashable, None] = {}
 
-        # do a one time iteration of series
-        def gen() -> tp.Iterator[bool]:
-            for label, value in series.items():
-                if isinstance(value, Frame):
+        if own_index:
+            self._index = index #type: ignore
+        else:
+            self._index = index_from_optional_constructor(index,
+                    default_constructor=Index,
+                    explicit_constructor=index_constructor
+                    )
+        count = len(self._index)
+        frames_array: NDArrayAny
+        self._loaded: NDArrayAny
+        load_array: bool | np.bool_
+        self._loaded_all: bool | np.bool_
+
+        if frames is None:
+            if store is None:
+                raise ErrorInitBus('Cannot initialize a :obj:`Bus` with neither `frames` nor `store`.')
+            self._values_mutable = np.full(count, FrameDeferred, dtype=DTYPE_OBJECT)
+            self._loaded = np.full(count, False, dtype=DTYPE_BOOL)
+            self._loaded_all = False
+        else:
+            if frames.__class__ is np.ndarray:
+                if frames.dtype != DTYPE_OBJECT: #type: ignore
+                    raise ErrorInitBus(
+                            f'Series passed to initializer must have dtype object, not {frames.dtype}') #type: ignore
+                frames_array = frames # type: ignore
+                load_array = False
+            else:
+                if own_data:
+                    raise ErrorInitBus('Cannot use `own_data` when not supplying an array.')
+                frames_array = np.empty(count, dtype=DTYPE_OBJECT)
+                load_array = True
+
+            self._loaded = np.empty(count, dtype=DTYPE_BOOL)
+            # do a one time iteration of series
+
+            for i, (label, value) in enumerate(zip_longest(
+                    index,
+                    frames,
+                    fillvalue=ZIP_LONGEST_DEFAULT,
+                    )):
+                if label is ZIP_LONGEST_DEFAULT or value is ZIP_LONGEST_DEFAULT:
+                    raise ErrorInitBus('frames and index are not of equal length')
+
+                if load_array:
+                    frames_array[i] = value
+
+                if value is FrameDeferred:
+                    self._loaded[i] = False
+                elif isinstance(value, Frame): # permit FrameGO?
                     if max_persist is not None:
                         self._last_accessed[label] = None
-                    yield True
-                elif value is FrameDeferred:
-                    yield False
+                    self._loaded[i] = True
                 else:
                     raise ErrorInitBus(f'supplied {value.__class__} is not a Frame or FrameDeferred.')
 
-        self._loaded = np.fromiter(gen(), dtype=DTYPE_BOOL, count=len(series))
-        self._loaded_all = self._loaded.all()
+            self._loaded_all = self._loaded.all()
 
-        if own_data:
-            self._values_mutable = series.values
+            if own_data or load_array:
+                self._values_mutable = frames_array
+            else:
+                self._values_mutable = frames_array.copy()
             self._values_mutable.flags.writeable = True
-        else:
-            self._values_mutable = series.values.copy()
 
-        self._index = series._index
-        self._name = series._name
+        # self._index = index
+        self._name = None if name is NAME_DEFAULT else name
         self._store = store
 
         # Not handling cases of max_persist being greater than the length of the Series (might floor to length)
@@ -453,14 +517,15 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         self._config = StoreConfigMap.from_initializer(config)
 
     #---------------------------------------------------------------------------
-    def _derive(self,
+    def _derive_from_series(self,
             series: Series,
             *,
             own_data: bool = False,
             ) -> 'Bus':
         '''Utility for creating a derived Bus, propagating the associated ``Store`` and configuration. This can be used if the passed `series` is a subset or re-ordering of self._series; however, if the index has been transformed, this method should not be used, as, if there is a Store, the labels are no longer found in that Store.
         '''
-        return self.__class__(series,
+        # NOTE: there may be a more efficient path than using a Series
+        return self.__class__.from_series(series,
                 store=self._store,
                 config=self._config,
                 max_persist=self._max_persist,
@@ -475,7 +540,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         Returns:
             :obj:`Index`
         '''
-        return reversed(self._index) #type: ignore
+        return reversed(self._index)
 
     # def __copy__(self) -> 'Bus':
     #     '''
@@ -490,7 +555,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
     #---------------------------------------------------------------------------
     # name interface
 
-    @property #type: ignore
+    @property
     @doc_inject()
     def name(self) -> NameType:
         '''{}'''
@@ -500,8 +565,16 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         '''
         Return a new :obj:`Bus` with an updated name attribute.
         '''
-        series = self._to_series_state().rename(name)
-        return self._derive(series, own_data=True)
+        # NOTE: do not want to use .values as this will force loading all Frames; use _values_mutable and let a copy be made by constructor
+        return self.__class__(self._values_mutable,
+                index=self._index,
+                name=name,
+                store=self._store,
+                config=self._config,
+                max_persist=self._max_persist,
+                own_index=True,
+                own_data=False,
+                )
 
     #---------------------------------------------------------------------------
     # interfaces
@@ -579,11 +652,14 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 own_index=own_index,
                 check_equals=check_equals,
                 )
-        return self.__class__(series, config=self._config)
+        # NOTE: do not propagate store after reindex
+        return self.__class__.from_series(series, config=self._config)
 
     @doc_inject(selector='relabel', class_name='Bus')
     def relabel(self,
-            index: tp.Optional[RelabelInput]
+            index: tp.Optional[RelabelInput],
+            *,
+            index_constructor: IndexConstructor = None,
             ) -> 'Bus':
         '''
         {doc}
@@ -591,9 +667,10 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         Args:
             index: {relabel_input}
         '''
-        series = self.to_series().relabel(index)
-        return self.__class__(series, config=self._config)
-
+        # NOTE: can be done without going trhough a series
+        series = self.to_series().relabel(index, index_constructor=index_constructor)
+        # NOTE: do not propagate store after relabel
+        return self.__class__.from_series(series, config=self._config)
 
     @doc_inject(selector='relabel_flat', class_name='Bus')
     def relabel_flat(self) -> 'Bus':
@@ -601,7 +678,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         {doc}
         '''
         series = self.to_series().relabel_flat()
-        return self.__class__(series, config=self._config)
+        return self.__class__.from_series(series, config=self._config)
 
     @doc_inject(selector='relabel_level_add', class_name='Bus')
     def relabel_level_add(self,
@@ -614,7 +691,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             level: {level}
         '''
         series = self.to_series().relabel_level_add(level)
-        return self.__class__(series, config=self._config)
+        return self.__class__.from_series(series, config=self._config)
 
 
     @doc_inject(selector='relabel_level_drop', class_name='Bus')
@@ -628,16 +705,21 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             count: {count}
         '''
         series = self.to_series().relabel_level_drop(count)
-        return self.__class__(series, config=self._config)
+        return self.__class__.from_series(series, config=self._config)
 
     def rehierarch(self,
-            depth_map: tp.Sequence[int]
+            depth_map: tp.Sequence[int],
+            *,
+            index_constructors: IndexConstructors = None,
             ) -> 'Bus':
         '''
         Return a new :obj:`Bus` with new a hierarchy based on the supplied ``depth_map``.
         '''
-        series = self.to_series().rehierarch(depth_map)
-        return self.__class__(series, config=self._config)
+        series = self.to_series().rehierarch(
+                depth_map,
+                index_constructors=index_constructors,
+                )
+        return self.__class__.from_series(series, config=self._config)
 
 
     #---------------------------------------------------------------------------
@@ -675,7 +757,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                     yield frame
         else: # max persist is 1
             for label in labels:
-                yield store.read(label, config=config[labels])
+                yield store.read(label, config=config[label])
 
 
     def _update_series_cache_iloc(self, key: GetItemKeyType) -> None:
@@ -687,11 +769,13 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         '''
         max_persist_active = self._max_persist is not None
 
-        load = False if self._loaded_all else not self._loaded[key].all()
+        load = False if self._loaded_all else not self._loaded[key].all() # type: ignore
         if not load and not max_persist_active:
             return
 
         index = self._index
+        label: tp.Hashable
+
         if not load and max_persist_active: # must update LRU position
             labels = (index.iloc[key],) if isinstance(key, INT_TYPES) else index.iloc[key].values
             for label in labels: # update LRU position
@@ -704,26 +788,26 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             loaded_count = self._loaded.sum()
 
         array = self._values_mutable
-        target_values = array[key]
-        target_labels = self._index.iloc[key]
+        target_values: NDArrayAny | Frame = array[key] # type: ignore
+        target_labels: IndexBase | tp.Hashable = self._index.iloc[key]
         # targets = self._series.iloc[key] # key is iloc key
 
         store_reader: FrameIterType
         targets_items: BusItemsType
 
-        if not isinstance(target_values, np.ndarray):
-            targets_items = ((target_labels, target_values),) # present element as items
+        if not target_values.__class__ is np.ndarray:
+            targets_items = ((target_labels, target_values),) # type: ignore # present element as items
             store_reader = (self._store.read(target_labels,
                     config=self._config[target_labels]) for _ in range(1))
         else: # more than one Frame
             store_reader = self._store_reader(
                     store=self._store,
                     config=self._config,
-                    labels=(label for label, f in zip(target_labels, target_values)
+                    labels=(label for label, f in zip(target_labels, target_values) # type: ignore
                             if f is FrameDeferred),
                     max_persist=self._max_persist,
                     )
-            targets_items = zip(target_labels, target_values)
+            targets_items = zip(target_labels, target_values) # type: ignore
 
         # Iterate over items that have been selected; there must be at least 1 FrameDeffered among this selection
         for label, frame in targets_items:
@@ -750,7 +834,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 array[idx_remove] = FrameDeferred
                 loaded_count -= 1
 
-        self._loaded_all = self._loaded.all()
+        self._loaded_all: bool = self._loaded.all() # type: ignore
 
     def unpersist(self) -> None:
         '''Replace loaded :obj:`Frame` with :obj:`FrameDeferred`.
@@ -786,19 +870,21 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         self._update_series_cache_iloc(key=key)
 
         # iterable selection should be handled by NP
-        values = self._values_mutable[key]
+        values: tp.Any = self._values_mutable[key] # type: ignore
 
         # NOTE: Bus only stores Frame and FrameDeferred, can rely on check with values
         if not values.__class__ is np.ndarray: # if we have a single element
             return values #type: ignore
 
-        # values will be copied and made immutable
-        series = Series(
-                values,
+        return self.__class__(values,
                 index=self._index.iloc[key],
                 name=self._name,
+                store=self._store,
+                config=self._config,
+                max_persist=self._max_persist,
+                own_index=True,
+                own_data=False, # force immutable copy
                 )
-        return self._derive(series, own_data=True)
 
     def _extract_loc(self, key: GetItemKeyType) -> 'Bus':
         iloc_key = self._index._loc_to_iloc(key)
@@ -816,9 +902,9 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
     #---------------------------------------------------------------------------
     # utilities for alternate extraction: drop
 
-    def _drop_iloc(self, key: GetItemKeyType) -> 'Bus':
+    def _drop_iloc(self, key: IntegerLocType) -> 'Bus':
         series = self._to_series_state()._drop_iloc(key)
-        return self._derive(series, own_data=True)
+        return self._derive_from_series(series, own_data=True)
 
     def _drop_loc(self, key: GetItemKeyType) -> 'Bus':
         return self._drop_iloc(self._index._loc_to_iloc(key))
@@ -886,7 +972,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
     _items_store = items
 
     @property
-    def values(self) -> np.ndarray:
+    def values(self) -> NDArrayAny:
         '''A 1D object array of all :obj:`Frame` contained in the :obj:`Bus`. The returned ``np.ndarray`` will have ``Frame``; this will never return an array with ``FrameDeferred``, but ``max_persist`` will be observed in reading from the Store.
         '''
         # NOTE: when self._values_mutable is fully loaded, it could become immutable and avoid a copy
@@ -980,7 +1066,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 frames=(f.dtypes for f in self._values_mutable if f is not FrameDeferred),
                 fill_value=None,
                 ).reindex(index=self._index, fill_value=None)
-        return tp.cast(Frame, f)
+        return f
 
     @property
     def shapes(self) -> Series:
@@ -1020,14 +1106,14 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                         else missing for f in self._values_mutable)
                 yield Series(values, index=self._index, dtype=dtype, name=attr)
 
-        return tp.cast(Frame, Frame.from_concat(gen(), axis=1))
+        return Frame.from_concat(gen(), axis=1)
 
 
     #---------------------------------------------------------------------------
     # common attributes from the numpy array
 
     @property
-    def dtype(self) -> np.dtype:
+    def dtype(self) -> DtypeAny:
         '''
         Return the dtype of the underlying NumPy array.
 
@@ -1064,7 +1150,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         Returns:
             :obj:`int`
         '''
-        return self._values_mutable.size #type: ignore
+        return self._values_mutable.size
 
     #---------------------------------------------------------------------------
 
@@ -1220,8 +1306,8 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
     def sort_index(self,
             *,
             ascending: BoolOrBools = True,
-            kind: str = DEFAULT_SORT_KIND,
-            key: tp.Optional[tp.Callable[[IndexBase], tp.Union[np.ndarray, IndexBase]]] = None,
+            kind: TSortKinds = DEFAULT_SORT_KIND,
+            key: tp.Optional[tp.Callable[[IndexBase], tp.Union[NDArrayAny, IndexBase]]] = None,
             ) -> 'Bus':
         '''
         Return a new Bus ordered by the sorted Index.
@@ -1240,14 +1326,14 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 kind=kind,
                 key=key,
                 )
-        return self._derive(series, own_data=True)
+        return self._derive_from_series(series, own_data=True)
 
     @doc_inject(selector='sort')
     def sort_values(self,
             *,
             ascending: bool = True,
-            kind: str = DEFAULT_SORT_KIND,
-            key: tp.Callable[['Series'], tp.Union[np.ndarray, 'Series']],
+            kind: TSortKinds = DEFAULT_SORT_KIND,
+            key: tp.Callable[['Series'], tp.Union[NDArrayAny, 'Series']],
             ) -> 'Bus':
         '''
         Return a new Bus ordered by the sorted values. Note that as a Bus contains Frames, a `key` argument must be provided to extract a sortable value, and this key function will process a :obj:`Series` of :obj:`Frame`.
@@ -1261,19 +1347,13 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         Returns:
             :obj:`Bus`
         '''
-        values = self.values # this will handle max_persist, but will deliver an array with all Frame loaded
-        cfs = Series(values,
-                index=self._index,
-                own_index=True,
-                name=self._name,
-                )
+        cfs = self.to_series() # force loading all Frame
         series = cfs.sort_values(
                 ascending=ascending,
                 kind=kind,
                 key=key,
                 )
-        return self._derive(series, own_data=True)
-
+        return self._derive_from_series(series, own_data=True)
 
     def roll(self,
             shift: int,
@@ -1290,7 +1370,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             :obj:`Bus`
         '''
         series = self._to_series_state().roll(shift=shift, include_index=include_index)
-        return self._derive(series, own_data=True)
+        return self._derive_from_series(series, own_data=True)
 
     def shift(self,
             shift: int,
@@ -1307,7 +1387,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             :obj:`Bus`
         '''
         series = self._to_series_state().shift(shift=shift, fill_value=fill_value)
-        return self._derive(series, own_data=True)
+        return self._derive_from_series(series, own_data=True)
 
     #---------------------------------------------------------------------------
     # exporter
@@ -1329,3 +1409,32 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 own_index=True,
                 name=self._name,
                 )
+
+    def _to_signature_bytes(self,
+            include_name: bool = True,
+            include_class: bool = True,
+            encoding: str = 'utf-8',
+            ) -> bytes:
+
+        v = (f._to_signature_bytes(
+                include_name=include_name,
+                include_class=include_class,
+                encoding=encoding,
+                ) for f in self._axis_element())
+
+        return b''.join(chain(
+                iter_component_signature_bytes(self,
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),
+                (self._index._to_signature_bytes(
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),),
+                v))
+
+
+
+
+
+
