@@ -1,28 +1,41 @@
+from __future__ import annotations
+
 import typing as tp
 from functools import partial
-from collections import deque
+
 import numpy as np
+import typing_extensions as tpe
 
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
 from static_frame.core.display_config import DisplayConfig
 from static_frame.core.display_config import DisplayFormats
 from static_frame.core.doc_str import doc_inject
+from static_frame.core.exception import ErrorNotTruthy
 from static_frame.core.interface_meta import InterfaceMeta
+from static_frame.core.memory_measure import MemoryDisplay
+from static_frame.core.node_fill_value import InterfaceBatchFillValue
+from static_frame.core.node_hashlib import InterfaceHashlib
+from static_frame.core.node_transpose import InterfaceBatchTranspose
+from static_frame.core.style_config import StyleConfig
 from static_frame.core.util import DTYPE_FLOAT_DEFAULT
 from static_frame.core.util import DTYPES_BOOL
 from static_frame.core.util import DTYPES_INEXACT
-from static_frame.core.util import EMPTY_TUPLE
+from static_frame.core.util import INT64_MAX
+from static_frame.core.util import OPERATORS
+from static_frame.core.util import UFUNC_TO_REVERSE_OPERATOR
+from static_frame.core.util import DtypeSpecifier
+from static_frame.core.util import NameType
 from static_frame.core.util import UFunc
 from static_frame.core.util import ufunc_all
 from static_frame.core.util import ufunc_any
 from static_frame.core.util import ufunc_nanall
 from static_frame.core.util import ufunc_nanany
-from static_frame.core.util import OPERATORS
-from static_frame.core.style_config import StyleConfig
 
 if tp.TYPE_CHECKING:
-    from static_frame.core.frame import Frame #pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.frame import Frame  # pylint: disable=W0611 #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 T = tp.TypeVar('T')
 
@@ -31,7 +44,7 @@ class ContainerBase(metaclass=InterfaceMeta):
     '''
     Root of all containers. The core containers, like Series, Frame, and Index, inherit from ContainerOperand. The higher-order containers, like Bus, Quilt, Batch, and Yarn, inherit from ContainerBase.
     '''
-    __slots__ = EMPTY_TUPLE
+    __slots__ = ()
 
     #---------------------------------------------------------------------------
     # class attrs
@@ -41,13 +54,42 @@ class ContainerBase(metaclass=InterfaceMeta):
     #---------------------------------------------------------------------------
     # common display functions
 
-    @property #type: ignore
+    @property
     @doc_inject()
     def interface(self) -> 'Frame':
         '''{}'''
         from static_frame.core.interface import InterfaceSummary
         return InterfaceSummary.to_frame(self.__class__)
 
+    # def __sizeof__(self) -> int:
+        # NOTE: implementing this to use memory_total is difficult, as we cannot pass in self without an infinite loop; trying to leave out self but keep its components returns a slightly different result as we miss the "native" (shallow) __sizeof__ components (and possible GC components as well).
+        # return memory_total(self, format=MeasureFormat.REFERENCED)
+
+    @property
+    def name(self) -> NameType:
+        return None
+
+    def _memory_label_component_pairs(self,
+            ) -> tp.Iterable[tp.Tuple[str, tp.Any]]:
+        return ()
+
+    @property
+    def memory(self) -> MemoryDisplay:
+        '''Return a :obj:`MemoryDisplay`, providing the size in memory of this object. For compound containers, component sizes will also be provided. Size can be interpreted through six combinations of three configurations:
+
+        L: Local: memory ignoring referenced array data provided via views.
+        LM: Local Materialized: memory where arrays that are locally owned report their byte payload
+        LMD: Local Materialized Data: locally owned memory of arrays byte payloads, excluding all other components
+
+        R: Referenced: memory including referenced array data provided via views
+        RM: Referenced Materialized: memory where arrays that are locally owned or referenced report their byte payload
+        RMD: Referenced Materialized Data: localy owned and referenced array byte payloads, excluding all other components
+        '''
+        label_component_pairs = self._memory_label_component_pairs()
+        return MemoryDisplay.from_any(self,
+                label_component_pairs=label_component_pairs,
+                )
+
     def display(self,
             config: tp.Optional[DisplayConfig] = None,
             *,
@@ -94,6 +136,7 @@ class ContainerBase(metaclass=InterfaceMeta):
                 ))
         return self.display(config=DisplayConfig(**args))
 
+
     #---------------------------------------------------------------------------
     def equals(self,
             other: tp.Any,
@@ -109,66 +152,105 @@ class ContainerBase(metaclass=InterfaceMeta):
     #---------------------------------------------------------------------------
     def __bool__(self) -> bool:
         '''
-        Raises ValueError to prohibit ambiguous use of truethy evaluation.
+        Raises ValueError to prohibit ambiguous use of truthy evaluation.
         '''
-        raise ValueError('The truth value of a container is ambiguous. For a truthy indicator of non-empty status, use the `size` attribute.')
+        raise ErrorNotTruthy()
+
+    def __lt__(self, other: tp.Any) -> tp.Any:
+        return NotImplemented #pragma: no cover
+
+    def __le__(self, other: tp.Any) -> tp.Any:
+        return NotImplemented #pragma: no cover
+
+    def __eq__(self, other: tp.Any) -> tp.Any:
+        return NotImplemented #pragma: no cover
+
+    def __ne__(self, other: tp.Any) -> tp.Any:
+        return NotImplemented #pragma: no cover
+
+    def __gt__(self, other: tp.Any) -> tp.Any:
+        return NotImplemented #pragma: no cover
+
+    def __ge__(self, other: tp.Any) -> tp.Any:
+        return NotImplemented #pragma: no cover
 
     #---------------------------------------------------------------------------
+
+    def _to_signature_bytes(self,
+            include_name: bool = True,
+            include_class: bool = True,
+            encoding: str = 'utf-8',
+            ) -> bytes:
+        raise NotImplementedError() #pragma: no cover
+
+    @property
+    def via_hashlib(self) -> InterfaceHashlib:
+        '''
+        Interface for deriving cryptographic hashes from this container.
+        '''
+        return InterfaceHashlib(
+                to_bytes=self._to_signature_bytes,
+                include_name=True,
+                include_class=True,
+                encoding='utf-8',
+                )
+
     def to_visidata(self) -> None:
         '''Open an interactive VisiData session.
         '''
-        from static_frame.core.display_visidata import view_sf #pragma: no cover
-        view_sf(self) #type: ignore [no-untyped-call] #pragma: no cover
+        from static_frame.core.display_visidata import view_sf  # pragma: no cover
+        view_sf(self) # type: ignore  #pragma: no cover
 
 
-class ContainerOperand(ContainerBase):
-    '''Base class of all containers that support opperators.'''
 
-    __slots__ = EMPTY_TUPLE
+
+class ContainerOperandSequence(ContainerBase):
+    '''Base class of all sequence-like containers that support operators but tend to decay to NumPy array, not specialized container subclasses. IndexBase inherits from this class.'''
+
+    __slots__ = ()
 
     interface: 'Frame' # property that returns a Frame
-    values: np.ndarray
+    # values: NDArrayAny
 
-    def _ufunc_unary_operator(self: T, operator: UFunc) -> T:
-        raise NotImplementedError() #pragma: no cover
-
-    def _ufunc_binary_operator(self: T, *,
+    # NOTE: the return type here is intentionally broad as it will get specialized in derived classes
+    def _ufunc_binary_operator(self, *,
             operator: UFunc,
             other: tp.Any,
             fill_value: object = np.nan,
-            ) -> T:
+            ) -> tp.Any:
+        raise NotImplementedError() #pragma: no cover
+
+    @property
+    def values(self) -> NDArrayAny:
         raise NotImplementedError() #pragma: no cover
 
-    #---------------------------------------------------------------------------
-    def __pos__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(OPERATORS['__pos__'])
-
-    def __neg__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(OPERATORS['__neg__'])
-
-    def __abs__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(OPERATORS['__abs__'])
-
-    def __invert__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(OPERATORS['__invert__'])
-
     #---------------------------------------------------------------------------
     def __add__(self, other: tp.Any) -> tp.Any:
+        if other.__class__ is InterfaceBatchFillValue or other.__class__ is InterfaceBatchTranspose:
+            return NotImplemented
         return self._ufunc_binary_operator(operator=OPERATORS['__add__'], other=other)
 
     def __sub__(self, other: tp.Any) -> tp.Any:
+        if other.__class__ is InterfaceBatchFillValue or other.__class__ is InterfaceBatchTranspose:
+            return NotImplemented
         return self._ufunc_binary_operator(operator=OPERATORS['__sub__'], other=other)
 
     def __mul__(self, other: tp.Any) -> tp.Any:
+        if other.__class__ is InterfaceBatchFillValue or other.__class__ is InterfaceBatchTranspose:
+            return NotImplemented
         return self._ufunc_binary_operator(operator=OPERATORS['__mul__'], other=other)
 
     def __matmul__(self, other: tp.Any) -> tp.Any:
         return self._ufunc_binary_operator(operator=OPERATORS['__matmul__'], other=other)
 
     def __truediv__(self, other: tp.Any) -> tp.Any:
+        if other.__class__ is InterfaceBatchFillValue or other.__class__ is InterfaceBatchTranspose:
+            return NotImplemented
         return self._ufunc_binary_operator(operator=OPERATORS['__truediv__'], other=other)
 
     def __floordiv__(self, other: tp.Any) -> tp.Any:
+        if other.__class__ is InterfaceBatchFillValue or other.__class__ is InterfaceBatchTranspose:
+            return NotImplemented
         return self._ufunc_binary_operator(operator=OPERATORS['__floordiv__'], other=other)
 
     def __mod__(self, other: tp.Any) -> tp.Any:
@@ -232,18 +314,46 @@ class ContainerOperand(ContainerBase):
         return self._ufunc_binary_operator(operator=OPERATORS['__rfloordiv__'], other=other)
 
     # --------------------------------------------------------------------------
-    # ufunc axis skipna methods: applied along an axis, reducing dimensionality.
-    # NOTE: as argmin and argmax have iloc/loc interetaions, they are implemented on derived containers
+    def __array__(self, dtype: DtypeSpecifier = None) -> NDArrayAny:
+        '''
+        Support the __array__ interface, returning an array of values.
+        '''
+        if dtype is None:
+            return self.values
+        array: NDArrayAny = self.values.astype(dtype)
+        return array
 
+    def __array_ufunc__(self,
+            ufunc: UFunc,
+            method: str,
+            *args: tp.Any,
+            **kwargs: tp.Any,
+            ) -> tp.Any:
+        '''Support for NumPy elements or arrays on the left hand of binary operators.
+        '''
+        if len(args) == 2 and args[1] is self and method == '__call__':
+            # self is right-hand side of binary operator with NumPy object
+            return self._ufunc_binary_operator(
+                    operator=UFUNC_TO_REVERSE_OPERATOR[ufunc],
+                    other=args[0],
+                    )
+        return NotImplemented  #pragma: no cover
+
+    # NOTE: this method will support aribitrary np functions; we choosen not to support these as not all functions make sense for SF containers
+    # def __array_function__(self, func, types, args, kwargs):
+    #     raise NotImplementedError(f'no support for {func}')
+
+    # --------------------------------------------------------------------------
+    # ufunc axis skipna methods: applied along an axis, reducing dimensionality.
     def _ufunc_axis_skipna(self, *,
             axis: int,
             skipna: bool,
             ufunc: UFunc,
             ufunc_skipna: UFunc,
             composable: bool,
-            dtypes: tp.Tuple[np.dtype, ...],
+            dtypes: tp.Tuple[DtypeAny, ...],
             size_one_unity: bool
-            ) -> np.ndarray:
+            ) -> tp.Any: # usually a Series
         '''
         Args:
             dtypes: iterable of valid dtypes that can be returned; first is default of not match
@@ -253,12 +363,11 @@ class ContainerOperand(ContainerBase):
         # not sure if these make sense on TypeBlocks, as they reduce dimensionality
         raise NotImplementedError() #pragma: no cover
 
-
     @doc_inject(selector='ufunc_skipna')
     def all(self,
             axis: int = 0,
             skipna: bool = True,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Logical ``and`` over values along the specified axis.
 
@@ -278,7 +387,7 @@ class ContainerOperand(ContainerBase):
     def any(self,
             axis: int = 0,
             skipna: bool = True,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Logical ``or`` over values along the specified axis.
 
@@ -298,7 +407,7 @@ class ContainerOperand(ContainerBase):
     def sum(self,
             axis: int = 0,
             skipna: bool = True,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Sum values along the specified axis.
 
@@ -310,7 +419,7 @@ class ContainerOperand(ContainerBase):
                 ufunc=np.sum,
                 ufunc_skipna=np.nansum,
                 composable=False,
-                dtypes=EMPTY_TUPLE, # float or int, row type will match except Boolean
+                dtypes=(), # float or int, row type will match except Boolean
                 size_one_unity=True
                 )
 
@@ -318,7 +427,7 @@ class ContainerOperand(ContainerBase):
     def min(self,
             axis: int = 0,
             skipna: bool = True,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Return the minimum along the specified axis.
 
@@ -330,7 +439,7 @@ class ContainerOperand(ContainerBase):
                 ufunc=np.min,
                 ufunc_skipna=np.nanmin,
                 composable=True,
-                dtypes=EMPTY_TUPLE,
+                dtypes=(),
                 size_one_unity=True
                 )
 
@@ -349,7 +458,7 @@ class ContainerOperand(ContainerBase):
                 ufunc=np.max,
                 ufunc_skipna=np.nanmax,
                 composable=True,
-                dtypes=EMPTY_TUPLE,
+                dtypes=(),
                 size_one_unity=True
                 )
 
@@ -357,7 +466,7 @@ class ContainerOperand(ContainerBase):
     def mean(self,
             axis: int = 0,
             skipna: bool = True,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Return the mean along the specified axis.
 
@@ -377,7 +486,7 @@ class ContainerOperand(ContainerBase):
     def median(self,
             axis: int = 0,
             skipna: bool = True,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Return the median along the specified axis.
 
@@ -398,7 +507,7 @@ class ContainerOperand(ContainerBase):
             axis: int = 0,
             skipna: bool = True,
             ddof: int = 0,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Return the standard deviaton along the specified axis.
 
@@ -407,8 +516,8 @@ class ContainerOperand(ContainerBase):
         return self._ufunc_axis_skipna(
                 axis=axis,
                 skipna=skipna,
-                ufunc=partial(np.std, ddof=ddof),
-                ufunc_skipna=partial(np.nanstd, ddof=ddof),
+                ufunc=partial(np.std, ddof=ddof), #type: ignore
+                ufunc_skipna=partial(np.nanstd, ddof=ddof), #type: ignore
                 composable=False,
                 dtypes=(DTYPE_FLOAT_DEFAULT,), # Ufuncs only return real result.
                 size_one_unity=False
@@ -419,7 +528,7 @@ class ContainerOperand(ContainerBase):
             axis: int = 0,
             skipna: bool = True,
             ddof: int = 0,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Return the variance along the specified axis.
 
@@ -428,8 +537,8 @@ class ContainerOperand(ContainerBase):
         return self._ufunc_axis_skipna(
                 axis=axis,
                 skipna=skipna,
-                ufunc=partial(np.var, ddof=ddof),
-                ufunc_skipna=partial(np.nanvar, ddof=ddof),
+                ufunc=partial(np.var, ddof=ddof), #type: ignore
+                ufunc_skipna=partial(np.nanvar, ddof=ddof), #type: ignore
                 composable=False,
                 dtypes=(DTYPE_FLOAT_DEFAULT,), # Ufuncs only return real result.
                 size_one_unity=False
@@ -439,7 +548,7 @@ class ContainerOperand(ContainerBase):
     def prod(self,
             axis: int = 0,
             skipna: bool = True,
-            out: tp.Optional[np.ndarray] = None,
+            out: tp.Optional[NDArrayAny] = None,
             ) -> tp.Any:
         '''Return the product along the specified axis.
 
@@ -451,10 +560,59 @@ class ContainerOperand(ContainerBase):
                 ufunc=np.prod,
                 ufunc_skipna=np.nanprod,
                 composable=False, # Block compbinations with overflow and NaNs require this.
-                dtypes=EMPTY_TUPLE,
+                dtypes=(),
                 size_one_unity=True
                 )
 
+    #---------------------------------------------------------------------------
+    def _repr_html_(self) -> str:
+        '''
+        Provide HTML representation for Jupyter Notebooks.
+        '''
+        # NOTE: We observe that Jupyter will window big content into scrollable component, so do not limit output and introduce ellipsis.
+
+        config = DisplayActive.get(
+                display_format=DisplayFormats.HTML_TABLE,
+                type_show=False,
+                display_columns=INT64_MAX,
+                display_rows=INT64_MAX,
+                )
+        # modify the active display to be for HTML
+        return repr(self.display(config))
+
+
+
+
+class ContainerOperand(ContainerOperandSequence):
+    '''Base class of all mapping-like containers that support operators. Series, TypeBlocks, and Frame inherit from this class. These containers preserve the type in unary and binary operations.'''
+
+    __slots__ = ()
+
+    #---------------------------------------------------------------------------
+    def __pos__(self) -> tpe.Self:
+        return self._ufunc_unary_operator(OPERATORS['__pos__'])
+
+    def __neg__(self) -> tpe.Self:
+        return self._ufunc_unary_operator(OPERATORS['__neg__'])
+
+    def __abs__(self) -> tpe.Self:
+        return self._ufunc_unary_operator(OPERATORS['__abs__'])
+
+    def __invert__(self) -> tpe.Self:
+        return self._ufunc_unary_operator(OPERATORS['__invert__'])
+
+    #---------------------------------------------------------------------------
+
+    def _ufunc_unary_operator(self: T, operator: UFunc) -> T:
+        raise NotImplementedError() #pragma: no cover
+
+    def _ufunc_binary_operator(self: T, *,
+            operator: UFunc,
+            other: tp.Any,
+            fill_value: object = np.nan,
+            ) -> T:
+        raise NotImplementedError() #pragma: no cover
+
     # ufunc shape skipna methods -----------------------------------------------
 
     def _ufunc_shape_skipna(self, *,
@@ -463,9 +621,9 @@ class ContainerOperand(ContainerBase):
             ufunc: UFunc,
             ufunc_skipna: UFunc,
             composable: bool,
-            dtypes: tp.Tuple[np.dtype, ...],
+            dtypes: tp.Tuple[DtypeAny, ...],
             size_one_unity: bool
-            ) -> np.ndarray:
+            ) -> tp.Any:
         # not sure if these make sense on TypeBlocks, as they reduce dimensionality
         raise NotImplementedError() #pragma: no cover
 
@@ -484,7 +642,7 @@ class ContainerOperand(ContainerBase):
                 ufunc=np.cumsum,
                 ufunc_skipna=np.nancumsum,
                 composable=False,
-                dtypes=EMPTY_TUPLE,
+                dtypes=(),
                 size_one_unity=True
                 )
 
@@ -503,39 +661,6 @@ class ContainerOperand(ContainerBase):
                 ufunc=np.cumprod,
                 ufunc_skipna=np.nancumprod,
                 composable=False,
-                dtypes=EMPTY_TUPLE,
+                dtypes=(),
                 size_one_unity=True
                 )
-
-    #---------------------------------------------------------------------------
-    def _repr_html_(self) -> str:
-        '''
-        Provide HTML representation for Jupyter Notebooks.
-        '''
-        # NOTE: We observe that Jupyter will window big content into scrollable component, so do not limit output and introduce ellipsis.
-
-        config = DisplayActive.get(
-                display_format=DisplayFormats.HTML_TABLE,
-                type_show=False,
-                display_columns=np.inf,
-                display_rows=np.inf,
-                )
-        # modify the active display to be for HTML
-        return repr(self.display(config))
-
-
-#-------------------------------------------------------------------------------
-# TODO: replace usage with ContainerMap; use this in a test to validate
-
-def container_opperand_map() -> tp.Dict[str, tp.Type[ContainerOperand]]:
-    '''Return a mapping of ContainerOperand types, from name to cls. Note that other modules must be loaded before this returns usable results.
-    '''
-    def collector() -> tp.Iterator[tp.Type[ContainerOperand]]:
-        targets = deque((ContainerOperand,))
-        while targets:
-            target = targets.popleft()
-            for part in target.__subclasses__():
-                targets.append(part)
-                yield part
-
-    return {cls.__name__: cls for cls in collector()}
