@@ -1,30 +1,35 @@
-import sys
-import typing as tp
-import os
+from __future__ import annotations
+
 import html
 import inspect
+import os
 import platform
 import re
-from functools import partial
+import sys
+import typing as tp
 from collections import namedtuple
+from functools import partial
 
 import numpy as np
 
 from static_frame.core.display_color import HexColor
-from static_frame.core.util import _gen_skip_middle
-from static_frame.core.util import CallableToIterType
+from static_frame.core.display_config import _DISPLAY_FORMAT_HTML
+from static_frame.core.display_config import _DISPLAY_FORMAT_MAP
+from static_frame.core.display_config import _DISPLAY_FORMAT_TERMINAL
+from static_frame.core.display_config import DisplayConfig
+from static_frame.core.style_config import StyleConfig
 from static_frame.core.util import COMPLEX_TYPES
 from static_frame.core.util import DTYPE_INT_KINDS
 from static_frame.core.util import DTYPE_STR_KINDS
 from static_frame.core.util import FLOAT_TYPES
-from static_frame.core.display_config import DisplayConfig
-from static_frame.core.display_config import _DISPLAY_FORMAT_HTML
-from static_frame.core.display_config import _DISPLAY_FORMAT_MAP
-from static_frame.core.display_config import _DISPLAY_FORMAT_TERMINAL
-from static_frame.core.style_config import StyleConfig
+from static_frame.core.util import CallableToIterType
+from static_frame.core.util import gen_skip_middle
 
 if tp.TYPE_CHECKING:
-    from static_frame.core.index_base import IndexBase # pylint: disable=unused-import #pragma: no cover
+    from static_frame.core.index_base import IndexBase  # pylint: disable=unused-import #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
+    THeaderSpecifier = tp.Union[DtypeAny, tp.Type[tp.Any], str, 'DisplayHeader', None] # pylint: disable=W0611 #pragma: no cover
 
 
 _module = sys.modules[__name__]
@@ -41,7 +46,7 @@ class DisplayTypeCategory:
     CONFIG_ATTR = 'type_color_default'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool: #pylint: disable=W0613
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool: #pylint: disable=W0613
         return True
 
 
@@ -49,56 +54,56 @@ class DisplayTypeInt(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_int'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind in DTYPE_INT_KINDS
 
 class DisplayTypeFloat(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_float'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind == 'f'
 
 class DisplayTypeComplex(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_complex'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind == 'c'
 
 class DisplayTypeBool(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_bool'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind == 'b'
 
 class DisplayTypeObject(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_object'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind == 'O'
 
 class DisplayTypeStr(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_str'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind in DTYPE_STR_KINDS
 
 class DisplayTypeDateTime(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_datetime'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind == 'M'
 
 class DisplayTypeTimeDelta(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_timedelta'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         return isinstance(t, np.dtype) and t.kind == 'm'
 
 
@@ -106,7 +111,7 @@ class DisplayTypeIndex(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_index'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         from static_frame.core.index_base import IndexBase
         if not inspect.isclass(t):
             return False
@@ -116,7 +121,7 @@ class DisplayTypeSeries(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_series'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         from static_frame import Series
         if not inspect.isclass(t):
             return False
@@ -126,7 +131,7 @@ class DisplayTypeFrame(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_frame'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         from static_frame import Frame
         if not inspect.isclass(t):
             return False
@@ -136,7 +141,7 @@ class DisplayTypeBus(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_bus'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         from static_frame import Bus
         if not inspect.isclass(t):
             return False
@@ -146,7 +151,7 @@ class DisplayTypeQuilt(DisplayTypeCategory):
     CONFIG_ATTR = 'type_color_quilt'
 
     @staticmethod
-    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+    def in_category(t: tp.Union[type, DtypeAny]) -> bool:
         from static_frame import Quilt
         if not inspect.isclass(t):
             return False
@@ -171,10 +176,12 @@ class DisplayTypeCategoryFactory:
             DisplayTypeQuilt,
             )
 
-    _TYPE_TO_CATEGORY_CACHE: tp.Dict[tp.Union[type, np.dtype], tp.Type[DisplayTypeCategory]] = {}
+    _TYPE_TO_CATEGORY_CACHE: tp.Dict[tp.Union[type, DtypeAny], tp.Type[DisplayTypeCategory]] = {}
 
     @classmethod
-    def to_category(cls, dtype: tp.Optional[tp.Union[type, np.dtype]]) -> tp.Type[DisplayTypeCategory]:
+    def to_category(cls,
+            dtype: tp.Union[type, DtypeAny],
+            ) -> tp.Type[DisplayTypeCategory]:
         if dtype not in cls._TYPE_TO_CATEGORY_CACHE:
             category = None
             for dtc in cls._DISPLAY_TYPE_CATEGORIES:
@@ -217,7 +224,7 @@ def terminal_ansi(stream: tp.TextIO = sys.stdout) -> bool:
 _module._display_active = DisplayConfig()  # type: ignore
 
 class DisplayActive:
-    '''Utility interface for setting module-level display configuration.
+    '''Utility interface for setting and storing default display configurations.
     '''
     FILE_NAME = '.static_frame.conf'
 
@@ -227,7 +234,7 @@ class DisplayActive:
 
     @staticmethod
     def get(**kwargs: tp.Union[bool, int, str]) -> DisplayConfig:
-        config: DisplayConfig = _module._display_active  # type: ignore
+        config: DisplayConfig = _module._display_active
         if not kwargs:
             return config
         args = config.to_dict()
@@ -280,10 +287,9 @@ class DisplayHeader:
         Provide string representation before additon of outer delimiters.
         '''
         if self.name is not None:
-            return '{}: {}'.format(self.cls.__name__, self.name)
+            return f'{self.cls.__name__}: {self.name}'
         return self.cls.__name__
 
-
 HeaderInitializer = tp.Optional[tp.Union[str, DisplayHeader]]
 
 # An NT that stores a formatted strings ``format_str`` as well as a ``raw`` string
@@ -316,12 +322,13 @@ class Display:
 
     @staticmethod
     def type_attributes(
-            type_input: tp.Union[np.dtype, type, DisplayHeader],
+            type_input: THeaderSpecifier,
             config: DisplayConfig
             ) -> tp.Tuple[str, tp.Type[DisplayTypeCategory]]:
         '''
         Return the `type_input` as a string, applying delimters to either numpy dtypes or Python classes.
         '''
+        type_ref: THeaderSpecifier
         if isinstance(type_input, np.dtype):
             type_str = str(type_input)
             type_ref = type_input
@@ -367,7 +374,7 @@ class Display:
 
     @classmethod
     def to_cell(cls,
-            value: object, # dtype, HeaderInitializer, or a type
+            value: tp.Any,
             config: DisplayConfig,
             is_dtype: bool = False) -> DisplayCell:
         '''
@@ -420,9 +427,9 @@ class Display:
 
     @classmethod
     def from_values(cls,
-            values: np.ndarray,
+            values: NDArrayAny | tp.Sequence[tp.Any],
             *,
-            header: object,
+            header: THeaderSpecifier,
             config: tp.Optional[DisplayConfig] = None,
             outermost: bool = False,
             index_depth: int = 0,
@@ -445,10 +452,10 @@ class Display:
             else:
                 rows.append([cls.CELL_EMPTY])
 
-        if values.__class__ is np.ndarray and values.ndim == 2:
+        if values.__class__ is np.ndarray and values.ndim == 2: # type: ignore
             # NOTE: this is generally only used by TypeBlocks
             # get rows from numpy string formatting
-            np_rows = np.array_str(values).split('\n')
+            np_rows = np.array_str(values).split('\n') # type: ignore
             last_idx = len(np_rows) - 1
             for idx, row in enumerate(np_rows):
                 # trim brackets
@@ -456,11 +463,11 @@ class Display:
                 row = row[2: len(row) - end_slice_len].strip()
                 rows.append([cls.to_cell(row, config=config)])
         else:
+            value_gen: tp.Callable[[], tp.Iterator[tp.Any]]
             count_max = config.display_rows
-            # print('comparing values to count_max', len(values), count_max)
             if len(values) > config.display_rows:
                 data_half_count = Display.truncate_half_count(count_max)
-                value_gen = partial(_gen_skip_middle,
+                value_gen = partial(gen_skip_middle,
                         forward_iter=values.__iter__,
                         forward_count=data_half_count,
                         reverse_iter=partial(reversed, values),
@@ -478,7 +485,7 @@ class Display:
 
         # add the types to the last row
         if values.__class__ is np.ndarray and config.type_show:
-            rows.append([cls.to_cell(values.dtype, config=config, is_dtype=True)])
+            rows.append([cls.to_cell(values.dtype, config=config, is_dtype=True)]) # type: ignore
 
         return cls(rows,
                 config=config,
@@ -491,8 +498,8 @@ class Display:
     @classmethod
     def from_params(cls,
             *,
-            index: "IndexBase",
-            columns: "IndexBase",
+            index: 'IndexBase',
+            columns: 'IndexBase',
             header: DisplayHeader,
             column_forward_iter: CallableToIterType,
             column_reverse_iter: CallableToIterType,
@@ -533,7 +540,7 @@ class Display:
             # columns as they will look after application of truncation and insertion of ellipsis
             data_half_count = cls.truncate_half_count(config.display_columns)
 
-            column_gen = partial(_gen_skip_middle,
+            column_gen = partial(gen_skip_middle,
                     forward_iter=column_forward_iter,
                     forward_count=data_half_count,
                     reverse_iter=column_reverse_iter,
@@ -725,7 +732,6 @@ class Display:
                         cell_formatted = cell_format_str.format(cell_raw)
                         cell_fill_width = pad_width - len(cell.raw) # this includes margin
 
-                    # print(col_idx, row_idx, cell, max_width, pad_width, cell_fill_width)
                     if config.cell_align_left:
                         # must manually add space as color chars make ljust not work
                         msg = cell_formatted + ' ' * cell_fill_width
@@ -798,7 +804,7 @@ class Display:
         # use mostly for debugging; strings in rows already have space / padding
         return dfc.LINE_SEP.join(''.join(r) for r in rows)
 
-    def to_rows(self) -> tp.Iterable[str]:
+    def to_rows(self) -> tp.Sequence[str]:
         '''
         Alternate output method for observing rows as strings within a list. Useful for testing.
         '''
@@ -833,7 +839,7 @@ class Display:
             self._rows[row_idx].extend(row)
 
     def extend_iterable(self,
-            iterable: tp.Sequence[tp.Any],
+            iterable: tp.Sequence[tp.Any] | NDArrayAny,
             header: HeaderInitializer
             ) -> None:
         '''
@@ -849,7 +855,7 @@ class Display:
 
         if len(iterable) > count_max:
             data_half_count = self.truncate_half_count(count_max)
-            value_gen: tp.Callable[[], tp.Iterator[tp.Any]] = partial(_gen_skip_middle,
+            value_gen: tp.Callable[[], tp.Iterator[tp.Any]] = partial(gen_skip_middle,
                     forward_iter=iterable.__iter__,
                     forward_count=data_half_count,
                     reverse_iter=partial(reversed, iterable),
