@@ -1,7 +1,7 @@
+from __future__ import annotations
 
 import typing as tp
 
-
 #-------------------------------------------------------------------------------
 # https://www.w3.org/TR/css-color-3/#svg-color
 
@@ -290,7 +290,7 @@ class HexColor:
         Given a hex color and text, return a string formatted for ANSI colors
         '''
         ansi = cls._to_ansi(hex_color)
-        return '\033[38;5;{ansi:d}m{text:s}\033[0m'.format(
+        return '\033[38;5;{ansi:d}m{text:s}\033[0m'.format( # pylint: disable=C0209
                 ansi=ansi,
                 text=text)
 
@@ -316,7 +316,7 @@ class HexColor:
         Given a hex color and text, return a string formatted for ANSI colors
         '''
         color = cls.get_html(hex_color)
-        return '<span style="color: {color}">{text}</span>'.format(
+        return '<span style="color: {color}">{text}</span>'.format( # pylint: disable=C0209
                 color=color,
                 text=text)
 
