@@ -1,14 +1,14 @@
-from enum import Enum
-import typing as tp
-import re
+from __future__ import annotations
+
 import json
+import re
+import typing as tp
+from enum import Enum
 
-import numpy as np
-
-from static_frame.core.interface_meta import InterfaceMeta
 from static_frame.core import display_html_datatables
+from static_frame.core.interface_meta import InterfaceMeta
 from static_frame.core.style_config import StyleConfig
-
+from static_frame.core.util import INT64_MAX
 
 ColorConstructor = tp.Union[int, str]
 
@@ -172,7 +172,7 @@ class DisplayFormatHTMLPre(DisplayFormat):
             ) -> str:
 
         style = 'style="white-space: pre; font-family: monospace"'
-        id_str = 'id="{}" '.format(identifier) if identifier else ''
+        id_str = 'id="{identifier}" ' if identifier else ''
         return f'<div {id_str}{style}>{msg}</div>'
 
 
@@ -364,7 +364,7 @@ class DisplayConfig(metaclass=InterfaceMeta):
 
     @classmethod
     def from_file(cls, fp: str) -> 'DisplayConfig':
-        with open(fp) as f:
+        with open(fp, encoding='utf-8') as f:
             return cls.from_json(f.read())
 
     @classmethod
@@ -466,12 +466,12 @@ class DisplayConfig(metaclass=InterfaceMeta):
     def write(self, fp: str) -> None:
         '''Write a JSON file.
         '''
-        with open(fp, 'w') as f:
+        with open(fp, 'w', encoding='utf-8') as f:
             f.write(self.to_json() + '\n')
 
     def __repr__(self) -> str:
         return '<' + self.__class__.__name__ + ' ' + ' '.join(
-                '{k}={v}'.format(k=k, v=getattr(self, k))
+                f'{k}={getattr(self, k)}'
                 for k in self.__slots__) + '>'
 
     def to_dict(self, **kwargs: object) -> tp.Dict[str, tp.Any]:
@@ -524,18 +524,18 @@ class DisplayConfigs:
             )
 
     UNBOUND = DisplayConfig(
-            display_columns=np.inf,
-            display_rows=np.inf,
-            cell_max_width=np.inf,
-            cell_max_width_leftmost=np.inf,
+            display_columns=INT64_MAX,
+            display_rows=INT64_MAX,
+            cell_max_width=INT64_MAX,
+            cell_max_width_leftmost=INT64_MAX,
             )
     UNBOUND_COLUMNS = DisplayConfig(
-            display_columns=np.inf,
-            cell_max_width=np.inf,
-            cell_max_width_leftmost=np.inf,
+            display_columns=INT64_MAX,
+            cell_max_width=INT64_MAX,
+            cell_max_width_leftmost=INT64_MAX,
             )
     UNBOUND_ROWS = DisplayConfig(
-            display_rows=np.inf,
-            cell_max_width=np.inf,
-            cell_max_width_leftmost=np.inf,
+            display_rows=INT64_MAX,
+            cell_max_width=INT64_MAX,
+            cell_max_width_leftmost=INT64_MAX,
             )
