@@ -1,12 +1,10 @@
 '''
 Storage for common doc strings and templates shared in non-related classes and methods.
 '''
-
+from __future__ import annotations
 
 import typing as tp
 
-from static_frame.core.util import AnyCallable
-
 #-------------------------------------------------------------------------------
 # common strings
 #NOTE: for kwargs, it is sometimes useful to only define the string, not the variable name, as in some contexts different variable names are use same conceptual entity.
@@ -24,7 +22,7 @@ AXIS = '''axis: Integer specifying axis, where 0 is rows and 1 is columns. Axis
 
 CHUNKSIZE = 'chunksize: Units of work per executor, as passed to the Thread- or ProcessPoolExecutor.'
 
-COLUMNS_CONSTRUCTOR = "columns_constructor: Optional class or constructor function to create the :obj:`Index` applied to the columns."
+COLUMNS_CONSTRUCTOR = 'columns_constructor: Optional class or constructor function to create the :obj:`Index` applied to the columns.'
 
 CONSOLIDATE_BLOCKS = 'consolidate_blocks: Optionally consolidate adjacent same-typed columns into contiguous arrays.'
 
@@ -32,11 +30,11 @@ DEEPCOPY_FROM_BUS = 'deepcopy_from_bus: Boolean to determine if containers are d
 
 DTYPE_SPECIFIER = '''dtype: A value suitable for specyfying a NumPy dtype, such as a Python type (float), NumPy array protocol strings ('f8'), or a dtype instance.'''
 
-DTYPES = "dtypes: Optionally provide an iterable of dtypes, equal in length to the length of each row, or a mapping by column name. If a dtype is given as None, NumPy's default type determination will be used."
+DTYPES = "dtypes: Optionally provide an iterable of dtypes, equal in length to the length of each row, or a mapping by column name (where overspecied labels is not an error). If a dtype is given as None, element-wise type determination will be used."
 
 FP = 'fp: A string file path or :obj:`Path` instance.'
 
-INDEX_CONSTRUCTOR = "index_constructor: Optional class or constructor function to create the :obj:`Index` applied to the rows."
+INDEX_CONSTRUCTOR = 'index_constructor: Optional class or constructor function to create the :obj:`Index` applied to the rows.'
 
 MAX_PERSIST = 'max_persist: When loading :obj:`Frame` from a :obj:`Store`, optionally define the maximum number of :obj:`Frame` to remain in the :obj:`Bus`, regardless of the size of the :obj:`Bus`. If more than ``max_persist`` number of :obj:`Frame` are loaded, least-recently loaded :obj:`Frame` will be replaced by ``FrameDeferred``. A ``max_persist`` of 1, for example, permits reading one :obj:`Frame` at a time without ever holding in memory more than 1 :obj:`Frame`.'
 
@@ -97,7 +95,7 @@ class DOC_TEMPLATE:
     '''
 
     label_widths_at_depth = '''
-    A generator of pairs, where each pair is the label and the count of that label found at the depth specified by  ``depth_level``.
+    A generator of pairs, where each pair is the label and the contiguous count of that label found at the depth specified by  ``depth_level``.
 
     Args:
         depth_level: a depth level, starting from zero.
@@ -224,7 +222,7 @@ class DOC_TEMPLATE:
             compare_name="compare_name: Include equality of the container's name (and all composed containers) in the comparison.",
             compare_dtype="compare_dtype: Include equality of the container's dtype (and all composed containers) in the comparison.",
             compare_class="compare_class: Include equality of the container's class (and all composed containers) in the comparison.",
-            skipna="skipna: If True, comparisons between missing values are equal.",
+            skipna='skipna: If True, comparisons between missing values are equal.',
             )
 
     from_any = dict(
@@ -278,19 +276,19 @@ class DOC_TEMPLATE:
             )
 
     insert = dict(
-            key_before="key: Label before which the new container will be inserted.",
-            key_after="key: Label after which the new container will be inserted.",
-            container="container: Container to be inserted.",
+            key_before='key: Label before which the new container will be inserted.',
+            key_after='key: Label after which the new container will be inserted.',
+            container='container: Container to be inserted.',
             fill_value='fill_value: A value to be used to fill space after reindexing the new container.'
             )
 
     join = dict(
-            left_depth_level="left_depth_level: Specify one or more left index depths to include in the join predicate.",
-            left_columns="left_columns: Specify one or more left columns to include in the join predicate.",
-            right_depth_level="right_depth_level: Specify one or more right index depths to include in the join predicate.",
-            right_columns="right_columns: Specify one or more right columns to include in the join predicate.",
-            left_template="left_template: Provide a format string for naming left columns in the joined result.",
-            right_template="right_template: Provide a format string for naming right columns in the joined result.",
+            left_depth_level='left_depth_level: Specify one or more left index depths to include in the join predicate.',
+            left_columns='left_columns: Specify one or more left columns to include in the join predicate.',
+            right_depth_level='right_depth_level: Specify one or more right index depths to include in the join predicate.',
+            right_columns='right_columns: Specify one or more right columns to include in the join predicate.',
+            left_template='left_template: Provide a format string for naming left columns in the joined result.',
+            right_template='right_template: Provide a format string for naming right columns in the joined result.',
             fill_value='fill_value: A value to be used to fill space created in the join.',
             composite_index='composite_index: If True, an index of tuples will be returned, formed from the left index label and the right index label; if False, an index of matching labels, if unique, will be returned.',
             composite_index_fill_value='composite_index_fill_value: Value to be used when forming a composite index when a label is missing.'
@@ -402,7 +400,7 @@ class DOC_TEMPLATE:
             )
     sort = dict(
             ascending='ascending: If True, sort in ascending order; if False, sort in descending order.',
-            ascendings='''ascending: Boolean, or iterable of Booleans; if ``True``, the lowest ranks correspond to the lowest values; if an iterable, apply per column or row. The default is ``True``.''',
+            ascendings='''ascendings: Boolean, or iterable of Booleans; if ``True``, the lowest ranks correspond to the lowest values; if an iterable, apply per column or row. The default is ``True``.''',
             kind='kind: Name of the sort algorithm as passed to NumPy.',
             key='key: A function that is used to pre-process the selected columns or rows and derive new values to sort by.'
             )
@@ -412,6 +410,7 @@ class DOC_TEMPLATE:
         Args:
             {FP}
             {STORE_CONFIG_MAP}
+            compression: Provide a zip compression setting using values from the Python ``zipfile`` module; ``zipfile.ZIP_DEFLATED`` is standard zlib compression; ``zipfile.ZIP_STORED`` disables compression and may give better performance at the cost of larger file sizes.
             '''
             )
 
@@ -444,18 +443,48 @@ class DOC_TEMPLATE:
             '''
             )
 
-# NOTE: F here should replace AnyCallable below
+    www = dict(
+            doc='''Given a URL, return a loadable component, either as an in-memory store (a StringIO or BytesIO) or a disk-based store (either to a provided file path or or managed temporary file that will be deleted after being read).
+            ''',
+            url="url: A URL string or a `Request` object from Python's standard library `urllib`.",
+            in_memory="in_memory: Define if data should be written to disk (reducing RAM usage) or held in memory until loaded.",
+            buffer_size="buffer_size: When `in_memory` is False, the number of bytes to be written per network read",
+            fp="fp: Optionally specify a file path to write to; cannot be specified if `in_memory` is True. When reading from archives (zip, gzip), only the extracted component will be written to this file path.",
+            component="component: When reading from a ZIP, specify the name of the component to load; if not specified and only one component exists, that component will be used.",
+            encoding="encoding: Specify the encoding of the resource to be delivered to the caller. If encoding is `None`, the resource is treated as binary."
+            )
+    json = dict(
+            json_index = 'A JSON object keyed by index labels, where values are rows represented by an object mapping of column labels to values.',
+            json_columns = 'A JSON object keyed by column labels, where values are columns represented by an object mapping of index labels to values.',
+            json_split = 'A JSON object with a key for "columns", "index", and "data"; each are arrays of values.',
+            json_records = 'A JSON array of row objects, where column labels are repeated for each row, and no index labels are included.',
+            json_values = 'A JSON array of arrays of row data; no index or columns labels are included.',
+            indent = 'indent: If indent is a non-negative integer or string, then JSON array elements and object members will be pretty-printed with that indent level.',
+            dtypes=DTYPES,
+            name=NAME,
+            consolidate_blocks=CONSOLIDATE_BLOCKS
+    )
+    unique_enumerated = dict(
+            doc = 'Given a collection of values, return two NumPy arrays: the first provides index positions for lookup of observed values; the second provides those observed, unique, values. Sometimes called "factorize".',
+            args = '''
+        Args:
+            retain_order: Boolean. If True, observed order is retained. If False, order may be sorted or observed, depending on the dtype and values. Retaining order may decrease performance.
+            func: A callable that takes an element and returns a Boolean if that element should be treated as missing, excluded from the unique value collection, and given a -1 `indexer` position. The NumPy function `np.isnan` or the StaticFrame `sf.isna_element` are examples of suitable functions.
+        '''
+    )
+
+# https://mypy.readthedocs.io/en/stable/generics.html#declaring-decorators
 F = tp.TypeVar('F', bound=tp.Callable[..., tp.Any])
 
 def doc_inject(*,
         selector: tp.Optional[str] = None,
         **kwargs: object
-        ) -> tp.Callable[[AnyCallable], AnyCallable]:
+        ) -> tp.Callable[[F], F]:
     '''
     Args:
         selector: optionally specify name of doc template dictionary to use; if not provided, the name of the function will be used.
     '''
-    def decorator(f: AnyCallable) -> AnyCallable:
+    def decorator(f: F) -> F:
 
         assert f.__doc__ is not None, f'{f} must have a docstring!'
 
