@@ -1,41 +1,51 @@
+from __future__ import annotations
+
 import typing as tp
+from functools import partial
+from itertools import chain
 
 import numpy as np
-from arraykit import resolve_dtype
 
-from static_frame.core.container import ContainerOperand
+from static_frame.core.container import ContainerOperandSequence
+from static_frame.core.container_util import IMTOAdapter
+from static_frame.core.container_util import imto_adapter_factory
+from static_frame.core.container_util import index_many_to_one
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
 from static_frame.core.display_config import DisplayConfig
 from static_frame.core.display_config import DisplayFormats
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import ErrorInitIndex
+from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.node_re import InterfaceRe
+from static_frame.core.node_str import InterfaceString
+from static_frame.core.style_config import STYLE_CONFIG_DEFAULT
+from static_frame.core.style_config import StyleConfig
+from static_frame.core.style_config import style_config_css_factory
+from static_frame.core.util import OPERATORS
 from static_frame.core.util import DepthLevelSpecifier
 from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import IndexConstructor
+from static_frame.core.util import IntegerLocType
+from static_frame.core.util import KeyTransformType
+from static_frame.core.util import ManyToOneType
 from static_frame.core.util import NameType
 from static_frame.core.util import PathSpecifierOrFileLike
 from static_frame.core.util import UFunc
 from static_frame.core.util import write_optional_file
-from static_frame.core.util import iterable_to_array_1d
-from static_frame.core.util import dtype_from_element
-from static_frame.core.util import EMPTY_TUPLE
-
-from static_frame.core.style_config import StyleConfig
-from static_frame.core.style_config import style_config_css_factory
-from static_frame.core.style_config import STYLE_CONFIG_DEFAULT
-from static_frame.core.node_re import InterfaceRe
-from static_frame.core.node_dt import InterfaceDatetime
-from static_frame.core.node_str import InterfaceString
 
 if tp.TYPE_CHECKING:
-    import pandas #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.series import Series #pylint: disable=W0611,C0412 #pragma: no cover
-    from static_frame.core.index_hierarchy import IndexHierarchy #pylint: disable=W0611,C0412 #pragma: no cover
-    from static_frame.core.index_auto import RelabelInput #pylint: disable=W0611,C0412 #pragma: no cover
+    import pandas  # pylint: disable=W0611 #pragma: no cover
+
+    from static_frame.core.index_auto import RelabelInput  # pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.index_hierarchy import IndexHierarchy  # pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.series import Series  # pylint: disable=W0611,C0412 #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 I = tp.TypeVar('I', bound='IndexBase')
 
-class IndexBase(ContainerOperand):
+class IndexBase(ContainerOperandSequence):
     '''
     All indices are derived from ``IndexBase``, including ``Index`` and ``IndexHierarchy``.
     '''
@@ -46,53 +56,67 @@ class IndexBase(ContainerOperand):
 
     _recache: bool
     _name: NameType
-    values: np.ndarray
-    positions: np.ndarray
     depth: int
+    _NDIM: int
 
     loc: tp.Any
     iloc: tp.Any # this does not work: InterfaceGetItem[I]
-    dtype: np.dtype
 
-    __pos__: tp.Callable[['IndexBase'], np.ndarray]
-    __neg__: tp.Callable[['IndexBase'], np.ndarray]
-    __abs__: tp.Callable[['IndexBase'], np.ndarray]
-    __invert__: tp.Callable[['IndexBase'], np.ndarray]
-    __add__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __sub__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __mul__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __matmul__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __truediv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __floordiv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __mod__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    # __divmod__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __pow__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __lshift__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __rshift__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __and__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __xor__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __or__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __lt__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __le__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __eq__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __ne__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __gt__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __ge__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __radd__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __rsub__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __rmul__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __rtruediv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __rfloordiv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
+    #---------------------------------------------------------------------------
+    def _ufunc_unary_operator(self, operator: UFunc) -> NDArrayAny:
+        raise NotImplementedError() #pragma: no cover
+
+    @property
+    def positions(self) -> NDArrayAny:
+        raise NotImplementedError() #pragma: no cover
+
+    #---------------------------------------------------------------------------
+    def __pos__(self) -> NDArrayAny:
+        return self._ufunc_unary_operator(OPERATORS['__pos__'])
+
+    def __neg__(self) -> NDArrayAny:
+        return self._ufunc_unary_operator(OPERATORS['__neg__'])
+
+    def __abs__(self) -> NDArrayAny:
+        return self._ufunc_unary_operator(OPERATORS['__abs__'])
+
+    def __invert__(self) -> NDArrayAny:
+        return self._ufunc_unary_operator(OPERATORS['__invert__'])
+
+    __add__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __sub__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __mul__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __matmul__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __truediv__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __floordiv__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __mod__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    # __divmod__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __pow__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __lshift__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __rshift__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __and__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __xor__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __or__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __lt__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __le__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __eq__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __ne__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __gt__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __ge__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __radd__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __rsub__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __rmul__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __rtruediv__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
+    __rfloordiv__: tp.Callable[['IndexBase', tp.Any], NDArrayAny]
     # __len__: tp.Callable[['IndexBase'], int]
 
     _IMMUTABLE_CONSTRUCTOR: tp.Callable[..., 'IndexBase']
     _MUTABLE_CONSTRUCTOR: tp.Callable[..., 'IndexBase']
 
-    _UFUNC_UNION: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray]
-    _UFUNC_INTERSECTION: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray]
-    _UFUNC_DIFFERENCE: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray]
-
-    label_widths_at_depth: tp.Callable[[I, int], tp.Iterator[tp.Tuple[tp.Hashable, int]]]
+    def label_widths_at_depth(self,
+            depth_level: DepthLevelSpecifier = 0
+            ) -> tp.Iterator[tp.Tuple[tp.Hashable, int]]:
+        raise NotImplementedError() #pragma: no cover
 
     #---------------------------------------------------------------------------
     # base class interface, mostly for mypy
@@ -103,7 +127,7 @@ class IndexBase(ContainerOperand):
     @classmethod
     def from_pandas(cls,
             value: 'pandas.Index',
-            ) -> 'IndexBase':
+            ) -> IndexBase:
         '''
         Given a Pandas index, return the appropriate IndexBase derived class.
         '''
@@ -113,35 +137,18 @@ class IndexBase(ContainerOperand):
 
         from static_frame import Index
         from static_frame import IndexGO
-        from static_frame import IndexHierarchy
-        from static_frame import IndexHierarchyGO
         from static_frame import IndexNanosecond
         from static_frame import IndexNanosecondGO
         from static_frame.core.index_datetime import IndexDatetime
 
-        if isinstance(value, pandas.MultiIndex):
-            # iterating over a hierarchical index will iterate over labels
-            name: tp.Optional[tp.Tuple[tp.Hashable, ...]] = tuple(value.names)
-            # if not assigned Pandas returns None for all components, which will raise issue if trying to unset this index.
-            if all(n is None for n in name): #type: ignore
-                name = None
-            depth = value.nlevels
-
-            if not cls.STATIC:
-                return IndexHierarchyGO.from_labels(value,
-                        name=name,
-                        depth_reference=depth)
-            return IndexHierarchy.from_labels(value,
-                    name=name,
-                    depth_reference=depth)
-        elif isinstance(value, pandas.DatetimeIndex):
+        if isinstance(value, pandas.DatetimeIndex):
             # if IndexDatetime, use cls, else use IndexNanosecond
             if issubclass(cls, IndexDatetime):
                 return cls(value, name=value.name)
-            else:
-                if not cls.STATIC:
-                    return IndexNanosecondGO(value, name=value.name)
-                return IndexNanosecond(value, name=value.name)
+
+            if not cls.STATIC:
+                return IndexNanosecondGO(value, name=value.name)
+            return IndexNanosecond(value, name=value.name)
 
         if not cls.STATIC:
             return IndexGO(value, name=value.name)
@@ -184,16 +191,19 @@ class IndexBase(ContainerOperand):
 
     def values_at_depth(self,
             depth_level: DepthLevelSpecifier = 0
-            ) -> np.ndarray:
+            ) -> NDArrayAny:
         raise NotImplementedError() #pragma: no cover
 
     @property
     def index_types(self) -> 'Series':
         # NOTE: this implementation is here due to pydoc.render_doc call that led to calling this base class method
         from static_frame.core.series import Series
-        return Series(EMPTY_TUPLE) # pragma: no cover
+        return Series(()) # pragma: no cover
 
-    def _extract_iloc(self: I, key: GetItemKeyType) -> tp.Union[I, tp.Hashable]:
+    def _extract_iloc(self, key: IntegerLocType | None) -> tp.Any:
+        raise NotImplementedError() #pragma: no cover
+
+    def _extract_iloc_by_int(self, key: int | np.integer[tp.Any]) -> tp.Hashable:
         raise NotImplementedError() #pragma: no cover
 
     def _update_array_cache(self) -> None:
@@ -211,7 +221,7 @@ class IndexBase(ContainerOperand):
     def _drop_iloc(self: I, key: GetItemKeyType) -> I:
         raise NotImplementedError() #pragma: no cover
 
-    def isin(self, other: tp.Iterable[tp.Any]) -> np.ndarray:
+    def isin(self, other: tp.Iterable[tp.Any]) -> NDArrayAny:
         raise NotImplementedError() #pragma: no cover
 
     def roll(self: I, shift: int) -> I:
@@ -224,10 +234,14 @@ class IndexBase(ContainerOperand):
             count: int = 1,
             *,
             seed: tp.Optional[int] = None,
-            ) -> tp.Tuple[I, np.ndarray]:
+            ) -> tp.Tuple[I, NDArrayAny]:
         raise NotImplementedError() #pragma: no cover
 
-    def level_add(self, level: tp.Hashable) -> 'IndexHierarchy':
+    def level_add(self,
+            level: tp.Hashable,
+            *,
+            index_constructor: IndexConstructor = None,
+            ) -> 'IndexHierarchy':
         raise NotImplementedError() #pragma: no cover
 
     def display(self,
@@ -237,6 +251,59 @@ class IndexBase(ContainerOperand):
             ) -> Display:
         raise NotImplementedError()
 
+    # ufunc shape skipna methods -----------------------------------------------
+
+    def _ufunc_shape_skipna(self, *,
+            axis: int,
+            skipna: bool,
+            ufunc: UFunc,
+            ufunc_skipna: UFunc,
+            composable: bool,
+            dtypes: tp.Tuple[DtypeAny, ...],
+            size_one_unity: bool
+            ) -> NDArrayAny:
+        # not sure if these make sense on TypeBlocks, as they reduce dimensionality
+        raise NotImplementedError() #pragma: no cover
+
+    @doc_inject(selector='ufunc_skipna')
+    def cumsum(self,
+            axis: int = 0,
+            skipna: bool = True,
+            ) -> NDArrayAny:
+        '''Return the cumulative sum over the specified axis.
+
+        {args}
+        '''
+        return self._ufunc_shape_skipna(
+                axis=axis,
+                skipna=skipna,
+                ufunc=np.cumsum,
+                ufunc_skipna=np.nancumsum,
+                composable=False,
+                dtypes=(),
+                size_one_unity=True
+                )
+
+    @doc_inject(selector='ufunc_skipna')
+    def cumprod(self,
+            axis: int = 0,
+            skipna: bool = True,
+            ) -> NDArrayAny:
+        '''Return the cumulative product over the specified axis.
+
+        {args}
+        '''
+        return self._ufunc_shape_skipna(
+                axis=axis,
+                skipna=skipna,
+                ufunc=np.cumprod,
+                ufunc_skipna=np.nancumprod,
+                composable=False,
+                dtypes=(),
+                size_one_unity=True
+                )
+
+
     #---------------------------------------------------------------------------
 
     @doc_inject(selector='sample')
@@ -256,83 +323,27 @@ class IndexBase(ContainerOperand):
 
     #---------------------------------------------------------------------------
 
-    @doc_inject(selector='searchsorted', label_type='iloc (integer)')
-    def iloc_searchsorted(self,
-            values: tp.Any,
-            *,
-            side_left: bool = True,
-            ) -> tp.Union[tp.Hashable, tp.Iterable[tp.Hashable]]:
-        '''
-        {doc}
-
-        Args:
-            {values}
-            {side_left}
-        '''
-        if not isinstance(values, str) and hasattr(values, '__len__'):
-            if not values.__class__ is np.ndarray:
-                values, _ = iterable_to_array_1d(values)
-        return np.searchsorted(self.values, #type: ignore [no-any-return]
-                values,
-                'left' if side_left else 'right',
-                )
-
-    @doc_inject(selector='searchsorted', label_type='loc (label)')
-    def loc_searchsorted(self,
-            values: tp.Any,
-            *,
-            side_left: bool = True,
-            fill_value: tp.Any = np.nan,
-            ) -> tp.Union[tp.Hashable, tp.Iterable[tp.Hashable]]:
-        '''
-        {doc}
-
-        Args:
-            {values}
-            {side_left}
-            {fill_value}
-        '''
-        sel = self.iloc_searchsorted(values, side_left=side_left)
-
-        length = self.__len__()
-        if sel.ndim == 0 and sel == length: # an element:
-            return fill_value #type: ignore [no-any-return]
-
-        mask = sel == length
-        if not mask.any():
-            return self.values[sel] #type: ignore [no-any-return]
-
-        post = np.empty(len(sel),
-                dtype=resolve_dtype(self.dtype,
-                dtype_from_element(fill_value))
-                )
-        sel[mask] = 0 # set out of range values to zero
-        post[:] = self.values[sel]
-        post[mask] = fill_value
-        post.flags.writeable = False
-        return post #type: ignore [no-any-return]
-
-    #---------------------------------------------------------------------------
-
     def _loc_to_iloc(self,
             key: GetItemKeyType,
-            ) -> GetItemKeyType:
+            key_transform: KeyTransformType = None,
+            partial_selection: bool = False,
+            ) -> IntegerLocType:
         raise NotImplementedError() #pragma: no cover
 
     def loc_to_iloc(self,
             key: GetItemKeyType,
-            ) -> GetItemKeyType:
+            ) -> IntegerLocType:
         raise NotImplementedError() #pragma: no cover
 
     def __getitem__(self: I,
-            key: GetItemKeyType
-            ) -> tp.Union[I, tp.Hashable]:
+            key: IntegerLocType
+            ) -> tp.Any:
         raise NotImplementedError() #pragma: no cover
 
     #---------------------------------------------------------------------------
     # name interface
 
-    @property #type: ignore
+    @property
     @doc_inject()
     def name(self) -> NameType:
         '''{}'''
@@ -355,7 +366,8 @@ class IndexBase(ContainerOperand):
                 yield str(name)
             # try to use name only if it is a tuple of the right size
             elif name and self._name_is_names():
-                for n in name: #type: ignore [attr-defined]
+                # name is a tuple of length equal to depth
+                for n in name: # type: ignore
                     yield str(n)
             else:
                 for i in range(depth):
@@ -389,90 +401,68 @@ class IndexBase(ContainerOperand):
     # set operations
 
     def _ufunc_set(self: I,
-            func: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray],
-            other: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]
+            others: tp.Iterable[tp.Union['IndexBase', tp.Iterable[tp.Hashable]]],
+            many_to_one_type: ManyToOneType,
             ) -> I:
-        raise NotImplementedError() #pragma: no cover
+        '''Normalize inputs and call `index_many_to_one`.
+        '''
+
+        if self._recache:
+            self._update_array_cache()
+
+        imtoaf = partial(imto_adapter_factory,
+                depth=self.depth,
+                name=self.name,
+                ndim=self.ndim,
+                )
+
+        indices: tp.Iterable[tp.Union[IndexBase, IMTOAdapter]]
+
+        if hasattr(others, '__len__') and len(others) == 1: # type: ignore
+            # NOTE: having only one `other` is far more common than many others; thus, optimize for that case by not using an iterator
+            indices = (self, imtoaf(others[0])) # type: ignore
+        else:
+            indices = chain((self,), (imtoaf(other) for other in others))
+
+        return index_many_to_one( # type: ignore
+                indices,
+                cls_default=self.__class__,
+                many_to_one_type=many_to_one_type,
+                )
 
     def intersection(self: I, *others: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]) -> I:
         '''
         Perform intersection with one or many Index, container, or NumPy array. Identical comparisons retain order.
         '''
-        # NOTE: must get UFunc off of class to avoid automatic addition of self to signature
-        func = self.__class__._UFUNC_INTERSECTION
-        if len(others) == 1:
-            return self._ufunc_set(func, others[0])
-
-        post = self
-        for other in others:
-            post = post._ufunc_set(func, other)
-        return post
+        return self._ufunc_set(others, ManyToOneType.INTERSECT)
 
     def union(self: I, *others: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]) -> I:
         '''
         Perform union with another Index, container, or NumPy array. Identical comparisons retain order.
         '''
-        func = self.__class__._UFUNC_UNION
-        if len(others) == 1:
-            return self._ufunc_set(func, others[0])
+        return self._ufunc_set(others, ManyToOneType.UNION)
 
-        post = self
-        for other in others:
-            post = post._ufunc_set(func, other)
-        return post
-
-
-    def difference(self: I, other: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]) -> I:
+    def difference(self: I, *others: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]) -> I:
         '''
         Perform difference with another Index, container, or NumPy array. Retains order.
         '''
-        return self._ufunc_set(
-                self.__class__._UFUNC_DIFFERENCE,
-                other)
-
-    #---------------------------------------------------------------------------
-    # metaclass-applied functions
-
-    def _ufunc_shape_skipna(self, *,
-            axis: int,
-            skipna: bool,
-            ufunc: UFunc,
-            ufunc_skipna: UFunc,
-            composable: bool,
-            dtypes: tp.Tuple[np.dtype, ...],
-            size_one_unity: bool
-            ) -> np.ndarray:
-        '''
-        For Index and IndexHierarchy, _ufunc_shape_skipna and _ufunc_axis_skipna are defined the same.
-
-        Returns:
-            immutable NumPy array.
-        '''
-        return self._ufunc_axis_skipna(
-                axis=axis,
-                skipna=skipna,
-                ufunc=ufunc,
-                ufunc_skipna=ufunc_skipna,
-                composable=composable, # shape on axis 1 is never composable
-                dtypes=dtypes,
-                size_one_unity=size_one_unity
-                )
+        return self._ufunc_set(others, ManyToOneType.DIFFERENCE)
 
     #---------------------------------------------------------------------------
     # via interfaces
 
     @property
-    def via_str(self) -> InterfaceString[np.ndarray]:
+    def via_str(self) -> InterfaceString[NDArrayAny]:
         raise NotImplementedError() #pragma: no cover
 
     @property
-    def via_dt(self) -> InterfaceDatetime[np.ndarray]:
+    def via_dt(self) -> InterfaceDatetime[NDArrayAny]:
         raise NotImplementedError() #pragma: no cover
 
     def via_re(self,
             pattern: str,
             flags: int = 0,
-            ) -> InterfaceRe[np.ndarray]:
+            ) -> InterfaceRe[NDArrayAny]:
         raise NotImplementedError() #pragma: no cover
 
     #---------------------------------------------------------------------------
@@ -510,15 +500,21 @@ class IndexBase(ContainerOperand):
                 )
         content = repr(self.display(config))
         # path_filter called internally
-        fp = write_optional_file(content=content, fp=fp)
+        fp_post: tp.Optional[str] = write_optional_file(content=content, fp=fp)
 
-        if fp and show:
-            import webbrowser #pragma: no cover
-            webbrowser.open_new_tab(fp) #pragma: no cover
+        if fp_post is not None and show:
+            import webbrowser  # pragma: no cover
+            webbrowser.open_new_tab(fp_post) #pragma: no cover
 
-        return fp
+        return fp_post
 
     def to_pandas(self) -> 'pandas.Series':
         raise NotImplementedError() #pragma: no cover
 
+    def _to_signature_bytes(self,
+            include_name: bool = True,
+            include_class: bool = True,
+            encoding: str = 'utf-8',
+            ) -> bytes:
+        raise NotImplementedError() #pragma: no cover
 
