@@ -1,15 +1,20 @@
+from __future__ import annotations
+
 import typing as tp
 
-from static_frame.core.util import array2d_to_tuples
+import numpy as np
+
 from static_frame.core.util import DTYPE_BOOL
 from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import PositionsAllocator
+from static_frame.core.util import array2d_to_tuples
 from static_frame.core.util import intersect1d
 from static_frame.core.util import intersect2d
-from static_frame.core.util import PositionsAllocator
 
 if tp.TYPE_CHECKING:
-
-    from static_frame.core.index import Index  # pylint: disable = W0611 #pragma: no cover
+    from static_frame.core.index_base import IndexBase  # pylint: disable = W0611 #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    # DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 
 class IndexCorrespondence:
@@ -33,8 +38,8 @@ class IndexCorrespondence:
 
     @classmethod
     def from_correspondence(cls,
-            src_index: 'Index',
-            dst_index: 'Index') -> 'IndexCorrespondence':
+            src_index: 'IndexBase',
+            dst_index: 'IndexBase') -> 'IndexCorrespondence':
         '''
         Return an IndexCorrespondence instance from the correspondence of two Index or IndexHierarchy objects.
 
@@ -73,12 +78,14 @@ class IndexCorrespondence:
                     )
             if mixed_depth:
                 # when mixed, on the 1D index we have to use loc_to_iloc with tuples
-                common_labels = list(array2d_to_tuples(common_labels))
+                common_labels = list(array2d_to_tuples(common_labels)) # type: ignore
             has_common = len(common_labels) > 0
         else:
             has_common = False
 
         size = len(dst_index.values)
+        iloc_dst: GetItemKeyType
+
         # either a reordering or a subset
         if has_common:
             if len(common_labels) == len(dst_index):
