@@ -1,18 +1,19 @@
-import typing as tp
+from __future__ import annotations
+
 import datetime
+import typing as tp
+from functools import partial
 
 import numpy as np
-
-from automap import AutoMap  # pylint: disable = E0611
-
+from arraymap import AutoMap  # pylint: disable = E0611
 
 from static_frame.core.doc_str import doc_inject
-from static_frame.core.index import _INDEX_GO_SLOTS
-from static_frame.core.index import _INDEX_SLOTS
-from static_frame.core.index import _IndexGOMixin
+from static_frame.core.exception import InvalidDatetime64Initializer
+from static_frame.core.exception import LocInvalid
+from static_frame.core.index import INDEX_GO_LEAF_SLOTS
 from static_frame.core.index import Index
 from static_frame.core.index import IndexGO
-from static_frame.core.util import DateInitializer
+from static_frame.core.index import _IndexGOMixin
 from static_frame.core.util import DT64_DAY
 from static_frame.core.util import DT64_H
 from static_frame.core.util import DT64_M
@@ -22,21 +23,30 @@ from static_frame.core.util import DT64_NS
 from static_frame.core.util import DT64_S
 from static_frame.core.util import DT64_US
 from static_frame.core.util import DT64_YEAR
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import IndexInitializer
-from static_frame.core.util import key_to_datetime_key
+from static_frame.core.util import DTYPE_BOOL
+from static_frame.core.util import NAME_DEFAULT
 from static_frame.core.util import TD64_DAY
 from static_frame.core.util import TD64_MONTH
 from static_frame.core.util import TD64_YEAR
-from static_frame.core.util import to_datetime64
-from static_frame.core.util import to_timedelta64
+from static_frame.core.util import DateInitializer
+from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import IndexInitializer
+from static_frame.core.util import KeyTransformType
+from static_frame.core.util import NameType
+from static_frame.core.util import WarningsSilent
 from static_frame.core.util import YearInitializer
 from static_frame.core.util import YearMonthInitializer
-from static_frame.core.util import NameType
-from static_frame.core.util import NAME_DEFAULT
+from static_frame.core.util import key_to_datetime_key
+from static_frame.core.util import to_datetime64
+from static_frame.core.util import to_timedelta64
 
 if tp.TYPE_CHECKING:
-    import pandas  #pylint: disable = W0611 #pragma: no cover
+    import pandas  # pylint: disable = W0611 #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
+
+
+key_to_datetime_key_year = partial(key_to_datetime_key, dtype=DT64_YEAR)
 
 I = tp.TypeVar('I', bound='IndexDatetime')
 
@@ -49,16 +59,16 @@ class IndexDatetime(Index):
     '''
 
     STATIC = True
-    _DTYPE = None # define in derived class
-    __slots__ = _INDEX_SLOTS
+    _DTYPE: DtypeAny # define in derived class
+    __slots__ = ()
 
-    @doc_inject(selector='index_date_time_init')
+    # @doc_inject(selector='index_date_time_init')
     def __init__(self,
             labels: IndexInitializer,
             *,
-            name: NameType = NAME_DEFAULT,
             loc_is_iloc: bool = False,
-            ):
+            name: NameType = NAME_DEFAULT,
+            ) -> None:
         '''Initializer.
 
         {args}
@@ -84,9 +94,9 @@ class IndexDatetime(Index):
 
     def _ufunc_binary_operator(self, *,
             operator: tp.Callable[..., tp.Any],
-            other: object,
-            fill_value: object = np.nan,
-            ) -> np.ndarray:
+            other: tp.Any,
+            fill_value: tp.Any = np.nan,
+            ) -> NDArrayAny:
 
         if self._recache:
             self._update_array_cache()
@@ -96,26 +106,45 @@ class IndexDatetime(Index):
 
         if isinstance(other, Index):
             other = other.values # operate on labels to labels
+            other_is_array = True
         elif isinstance(other, str):
-            # do not pass dtype, as want to coerce to this parsed type, not the type of sled
+            # do not pass dtype, as want to coerce to this parsed type, not the type of self
             other = to_datetime64(other)
+            other_is_array = False
+        elif other.__class__ is np.ndarray:
+            other_is_array = True
+        else:
+            other_is_array = False
 
+        result: NDArrayAny
         if isinstance(other, np.datetime64):
             # convert labels to other's datetime64 type to enable matching on month, year, etc.
-            array = operator(self._labels.astype(other.dtype), other)
+            result = operator(self._labels.astype(other.dtype), other)
         elif isinstance(other, datetime.timedelta):
-            array = operator(self._labels, to_timedelta64(other))
-        else:
-            # np.timedelta64 should work fine here
-            array = operator(self._labels, other)
+            result = operator(self._labels, to_timedelta64(other))
+        else: # np.timedelta64 should work fine here
+            with WarningsSilent():
+                result = operator(self._labels, other)
 
-        array.flags.writeable = False
-        return array
+        # NOTE: similar branching as in container_util.apply_binary_operator
+        # NOTE: all string will have been converted to dt64, or raise ValueError; comparison to same sized iterables (list, tuple) will result in an array when they are the same size
+        if result is False: # will never be True
+            if not other_is_array and hasattr(other, '__len__') and len(other) == len(self): # type: ignore
+                # NOTE: equality comparisons of an array to same sized iterable normally return an array, but with dt64 types they just return False
+                result = np.full(self.shape, result, dtype=DTYPE_BOOL)
+            elif other_is_array and other.size == 1:
+                # elements in arrays of 0 or more dimensions are acceptable; this is what NP does for arithmetic operators when the types are compatible
+                result = np.full(self.shape, result, dtype=DTYPE_BOOL)
+            else:
+                raise ValueError('operands could not be broadcast together')
+                # raise on unaligned shapes as is done for arithmetic operators
+
+        result.flags.writeable = False
+        return result
 
     def _loc_to_iloc(self,  # type: ignore
             key: GetItemKeyType,
-            *,
-            offset: tp.Optional[int] = None,
+            key_transform: KeyTransformType = key_to_datetime_key,
             partial_selection: bool = False,
             ) -> GetItemKeyType:
         '''
@@ -124,8 +153,7 @@ class IndexDatetime(Index):
         # not passing self.dtype to key_to_datetime_key so as to allow translation to a foreign datetime; slice comparison will be handled by map_slice_args
         return Index._loc_to_iloc(self,
                 key=key,
-                offset=offset,
-                key_transform=key_to_datetime_key,
+                key_transform=key_transform,
                 partial_selection=partial_selection,
                 )
 
@@ -145,7 +173,7 @@ class IndexDatetime(Index):
             values: tp.Any,
             *,
             side_left: bool = True,
-            ) -> tp.Union[tp.Hashable, tp.Iterable[tp.Hashable]]:
+            ) -> NDArrayAny:
         '''
         {doc}
 
@@ -154,7 +182,7 @@ class IndexDatetime(Index):
             {side_left}
         '''
         # permit variable forms of date specification
-        return Index.iloc_searchsorted(self, #type: ignore [no-any-return]
+        return Index.iloc_searchsorted(self,
                 key_to_datetime_key(values),
                 side_left=side_left,
                 )
@@ -163,19 +191,19 @@ class IndexDatetime(Index):
 #-------------------------------------------------------------------------------
 class _IndexDatetimeGOMixin(_IndexGOMixin):
 
-    _DTYPE: tp.Optional[np.dtype]
+    _DTYPE: DtypeAny
     _map: tp.Optional[AutoMap]
     __slots__ = () # define in derived class
 
     def append(self, value: tp.Hashable) -> None:
         '''Specialize for fixed-typed indices: convert `value` argument; do not need to resolve_dtype with each addition; self._map is never None
         '''
-        value = to_datetime64(value, self._DTYPE)
+        value = to_datetime64(value, self._DTYPE) # type: ignore
         if self._map is not None:
             try:
                 self._map.add(value)
-            except ValueError:
-                raise KeyError(f'duplicate key append attempted: {value}')
+            except ValueError as e:
+                raise KeyError(f'duplicate key append attempted: {value}') from e
         self._labels_mutable.append(value)
         self._positions_mutable_count += 1 #pylint: disable=E0237
         self._recache = True #pylint: disable=E0237
@@ -186,7 +214,7 @@ class IndexYear(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_YEAR
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
     @classmethod
     def from_date_range(cls: tp.Type[I],
@@ -238,7 +266,7 @@ class IndexYear(IndexDatetime):
         '''
         Get an IndexDate instance over a range of years, where start and end are inclusive.
         '''
-        labels = np.arange(
+        labels: NDArrayAny = np.arange(
                 to_datetime64(start, DT64_YEAR),
                 to_datetime64(stop, DT64_YEAR) + TD64_YEAR,
                 step=np.timedelta64(step, 'Y'),
@@ -246,6 +274,36 @@ class IndexYear(IndexDatetime):
         labels.flags.writeable = False
         return cls(labels, name=name)
 
+    #---------------------------------------------------------------------------
+    # specializations to permit integers as years
+
+    def __contains__(self, value: tp.Any) -> bool:
+        '''Return True if value in the labels. Will only return True for an exact match to the type of dates stored within.
+        '''
+        try:
+            return self._map.__contains__(to_datetime64(value, self._DTYPE)) #type: ignore
+        except InvalidDatetime64Initializer:
+            # if value is a dt64 and not of a the same unit as self._DTYPE, the initializations exception is raised, which means False
+            return False
+
+
+    def _loc_to_iloc(self,  # type: ignore
+            key: GetItemKeyType,
+            key_transform: KeyTransformType = key_to_datetime_key_year,
+            partial_selection: bool = False,
+            ) -> GetItemKeyType:
+        '''
+        Specialized for IndexData indices to convert string data representations into np.datetime64 objects as appropriate.
+        '''
+        try:
+            return Index._loc_to_iloc(self,
+                    key=key,
+                    key_transform=key_transform,
+                    partial_selection=partial_selection,
+                    )
+        except InvalidDatetime64Initializer as e:
+            raise LocInvalid(e.args[0]) from None
+
     #---------------------------------------------------------------------------
     def to_pandas(self) -> None:
         '''Return a Pandas Index.
@@ -253,10 +311,11 @@ class IndexYear(IndexDatetime):
         raise NotImplementedError('Pandas does not support a year type, and it is ambiguous if a date proxy should be the first of the year or the last of the year.')
 
 
+
 class IndexYearGO(_IndexDatetimeGOMixin, IndexYear):
 
     _IMMUTABLE_CONSTRUCTOR = IndexYear
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexYear._MUTABLE_CONSTRUCTOR = IndexYearGO
 
@@ -266,7 +325,7 @@ class IndexYearMonth(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_MONTH
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
     @classmethod
     def from_date_range(cls: tp.Type[I],
@@ -338,7 +397,7 @@ class IndexYearMonth(IndexDatetime):
 class IndexYearMonthGO(_IndexDatetimeGOMixin, IndexYearMonth):
 
     _IMMUTABLE_CONSTRUCTOR = IndexYearMonth
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexYearMonth._MUTABLE_CONSTRUCTOR = IndexYearMonthGO
 
@@ -349,7 +408,7 @@ class IndexDate(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_DAY
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
     @classmethod
     def from_date_range(cls: tp.Type[I],
@@ -362,7 +421,7 @@ class IndexDate(IndexDatetime):
         '''
         Get an IndexDate instance over a range of dates, where start and stop is inclusive.
         '''
-        labels = np.arange(
+        labels: NDArrayAny = np.arange(
                 to_datetime64(start, DT64_DAY),
                 to_datetime64(stop, DT64_DAY) + TD64_DAY,
                 np.timedelta64(step, 'D'))
@@ -410,7 +469,7 @@ class IndexDate(IndexDatetime):
 class IndexDateGO(_IndexDatetimeGOMixin, IndexDate):
 
     _IMMUTABLE_CONSTRUCTOR = IndexDate
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexDate._MUTABLE_CONSTRUCTOR = IndexDateGO
 
@@ -420,12 +479,12 @@ class IndexHour(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_H
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
 class IndexHourGO(_IndexDatetimeGOMixin, IndexHour):
 
     _IMMUTABLE_CONSTRUCTOR = IndexHour
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexHour._MUTABLE_CONSTRUCTOR = IndexHourGO
 
@@ -435,12 +494,12 @@ class IndexMinute(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_M
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
 class IndexMinuteGO(_IndexDatetimeGOMixin, IndexMinute):
 
     _IMMUTABLE_CONSTRUCTOR = IndexMinute
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexMinute._MUTABLE_CONSTRUCTOR = IndexMinuteGO
 
@@ -450,12 +509,12 @@ class IndexSecond(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_S
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
 class IndexSecondGO(_IndexDatetimeGOMixin, IndexSecond):
 
     _IMMUTABLE_CONSTRUCTOR = IndexSecond
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexSecond._MUTABLE_CONSTRUCTOR = IndexSecondGO
 
@@ -465,12 +524,12 @@ class IndexMillisecond(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_MS
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
 class IndexMillisecondGO(_IndexDatetimeGOMixin, IndexMillisecond):
 
     _IMMUTABLE_CONSTRUCTOR = IndexMillisecond
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexMillisecond._MUTABLE_CONSTRUCTOR = IndexMillisecondGO
 
@@ -480,12 +539,12 @@ class IndexMicrosecond(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_US
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
 class IndexMicrosecondGO(_IndexDatetimeGOMixin, IndexMicrosecond):
 
     _IMMUTABLE_CONSTRUCTOR = IndexMicrosecond
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexMicrosecond._MUTABLE_CONSTRUCTOR = IndexMicrosecondGO
 
@@ -495,19 +554,19 @@ class IndexNanosecond(IndexDatetime):
     '''
     STATIC = True
     _DTYPE = DT64_NS
-    __slots__ = _INDEX_SLOTS
+    __slots__ = ()
 
 class IndexNanosecondGO(_IndexDatetimeGOMixin, IndexNanosecond):
 
     _IMMUTABLE_CONSTRUCTOR = IndexNanosecond
-    __slots__ = _INDEX_GO_SLOTS
+    __slots__ = INDEX_GO_LEAF_SLOTS
 
 IndexNanosecond._MUTABLE_CONSTRUCTOR = IndexNanosecondGO
 
 
 
 #-------------------------------------------------------------------------------
-_DTYPE_TO_CLASS = {cls._DTYPE: cls for cls in (
+_DTYPE_TO_CLASS: tp.Dict[DtypeAny, tp.Type[Index]] = {cls._DTYPE: cls for cls in (
         IndexYear,
         IndexYearMonth,
         IndexDate,
@@ -519,12 +578,12 @@ _DTYPE_TO_CLASS = {cls._DTYPE: cls for cls in (
         IndexNanosecond
         )}
 
-def dtype_to_index_cls(static: bool, dtype: np.dtype) -> tp.Type[Index]:
+def dtype_to_index_cls(static: bool, dtype: DtypeAny) -> tp.Type[Index]:
     '''
     Given an the class of the Index from which this is valled, as well as the dtype of the resultant array, return the appropriate Index class.
     '''
 
-    resolved_static = _DTYPE_TO_CLASS.get(dtype)
+    resolved_static: tp.Type[Index] | None = _DTYPE_TO_CLASS.get(dtype)
     if resolved_static is not None:
         if static:
             return resolved_static
