@@ -1,37 +1,37 @@
 '''
 Tools for iterators in Series and Frame. These components are imported by both series.py and frame.py; these components also need to be able to return Series and Frame, and thus use deferred, function-based imports.
 '''
+from __future__ import annotations
 
 import typing as tp
 from enum import Enum
 from functools import partial
-from concurrent.futures import ProcessPoolExecutor
-from concurrent.futures import ThreadPoolExecutor
-# import multiprocessing as mp
-# mp_context = mp.get_context('spawn')
 
 import numpy as np
 from arraykit import name_filter
 
+from static_frame.core.container_util import group_from_container
 from static_frame.core.doc_str import doc_inject
+from static_frame.core.util import KEY_ITERABLE_TYPES
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import DepthLevelSpecifier
 from static_frame.core.util import DtypeSpecifier
-from static_frame.core.util import KEY_ITERABLE_TYPES
+from static_frame.core.util import IndexConstructor
 from static_frame.core.util import Mapping
 from static_frame.core.util import NameType
 from static_frame.core.util import TupleConstructorType
+from static_frame.core.util import get_concurrent_executor
 from static_frame.core.util import iterable_to_array_1d
-from static_frame.core.util import IndexConstructor
-# from static_frame.core.util import array_from_iterator
-
 
 if tp.TYPE_CHECKING:
-    from static_frame.core.frame import Frame # pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.series import Series # pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.quilt import Quilt # pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.bus import Bus # pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.yarn import Yarn # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.bus import Bus  # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.frame import Frame  # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.index import Index  # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.quilt import Quilt  # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.series import Series  # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.yarn import Yarn  # pylint: disable=W0611 #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    # DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 
 FrameOrSeries = tp.TypeVar('FrameOrSeries', 'Frame', 'Series', 'Bus', 'Quilt', 'Yarn')
@@ -54,12 +54,16 @@ class IterNodeApplyType(Enum):
 
     @classmethod
     def is_items(cls, apply_type: 'IterNodeApplyType') -> bool:
-        if apply_type is cls.SERIES_VALUES or apply_type is cls.INDEX_LABELS:
+        '''Return True if the apply_constructor to be used consumes items; otherwise, the apply_constructor consumes values alone.
+        '''
+        if (apply_type is cls.SERIES_VALUES
+                or apply_type is cls.INDEX_LABELS
+                ):
             return False
         return True
 
 
-
+# NOTE: the generic type here is the type returned from calls to apply()
 class IterNodeDelegate(tp.Generic[FrameOrSeries]):
     '''
     Delegate returned from :obj:`static_frame.IterNode`, providing iteration as well as a family of apply methods.
@@ -73,20 +77,11 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             '_apply_type',
             )
 
-    INTERFACE = (
+    INTERFACE: tp.Tuple[str, ...] = (
             'apply',
             'apply_iter',
             'apply_iter_items',
             'apply_pool',
-            'map_all',
-            'map_all_iter',
-            'map_all_iter_items',
-            'map_any',
-            'map_any_iter',
-            'map_any_iter_items',
-            'map_fill',
-            'map_fill_iter',
-            'map_fill_iter_items',
             )
 
     def __init__(self,
@@ -110,13 +105,13 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
 
     def _apply_iter_items_parallel(self,
             func: AnyCallable,
+            *,
             max_workers: tp.Optional[int] = None,
             chunksize: int = 1,
             use_threads: bool = False,
+            mp_context: tp.Optional[str] = None,
             ) -> tp.Iterator[tp.Tuple[tp.Any, tp.Any]]:
 
-        pool_executor = ThreadPoolExecutor if use_threads else ProcessPoolExecutor
-
         if not callable(func): # support array, Series mapping
             func = getattr(func, '__getitem__')
 
@@ -135,19 +130,26 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
                     func_keys.append(k)
                     yield k, v
 
-        with pool_executor(max_workers=max_workers) as executor:
+        pool_executor = get_concurrent_executor(
+                use_threads=use_threads,
+                max_workers=max_workers,
+                mp_context=mp_context,
+                )
+
+        with pool_executor() as executor:
             yield from zip(func_keys,
                     executor.map(func, arg_gen(), chunksize=chunksize)
                     )
 
     def _apply_iter_parallel(self,
             func: AnyCallable,
+            *,
             max_workers: tp.Optional[int] = None,
             chunksize: int = 1,
             use_threads: bool = False,
+            mp_context: tp.Optional[str] = None,
             ) -> tp.Iterator[tp.Any]:
 
-        pool_executor = ThreadPoolExecutor if use_threads else ProcessPoolExecutor
         if not callable(func): # support array, Series mapping
             func = getattr(func, '__getitem__')
 
@@ -155,11 +157,167 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
         arg_gen = (self._func_values if self._yield_type is IterNodeType.VALUES
                 else self._func_items)
 
-        with pool_executor(max_workers=max_workers) as executor:
+        pool_executor = get_concurrent_executor(
+                use_threads=use_threads,
+                max_workers=max_workers,
+                mp_context=mp_context,
+                )
+
+        with pool_executor() as executor:
             yield from executor.map(func, arg_gen(), chunksize=chunksize)
 
     #---------------------------------------------------------------------------
-    # public interface
+    @doc_inject(selector='apply')
+    def apply_iter_items(self,
+            func: AnyCallable,
+            ) -> tp.Iterator[tp.Tuple[tp.Any, tp.Any]]:
+        '''
+        {doc} A generator of resulting key, value pairs.
+
+        Args:
+            {func}
+
+        Yields:
+            Pairs of label, value after function application.
+        '''
+        # depend on yield type, we determine what the passed in function expects to
+        if self._yield_type is IterNodeType.VALUES:
+            yield from ((k, func(v)) for k, v in self._func_items())
+        else:
+            yield from ((k, func(k, v)) for k, v in self._func_items())
+
+    @doc_inject(selector='apply')
+    def apply_iter(self,
+            func: AnyCallable
+            ) -> tp.Iterator[tp.Any]:
+        '''
+        {doc} A generator of resulting values.
+
+        Args:
+            {func}
+
+        Yields:
+            Values after function application.
+        '''
+        if self._yield_type is IterNodeType.VALUES:
+            yield from (func(v) for v in self._func_values())
+        else:
+            yield from (func(k, v) for k, v in self._func_items())
+
+    @doc_inject(selector='apply')
+    def apply(self,
+            func: AnyCallable,
+            *,
+            dtype: DtypeSpecifier = None,
+            name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor] = None,
+            columns_constructor: tp.Optional[IndexConstructor] = None,
+            ) -> FrameOrSeries:
+        '''
+        {doc} Returns a new container.
+
+        Args:
+            {func}
+            {dtype}
+        '''
+        if not callable(func):
+            raise RuntimeError('use map_fill(), map_any(), or map_all() for applying a mapping type')
+
+        if IterNodeApplyType.is_items(self._apply_type):
+            apply_func = self.apply_iter_items
+        else:
+            apply_func = self.apply_iter
+
+        if self._apply_type is IterNodeApplyType.FRAME_ELEMENTS:
+            # can always pass columns_constructor
+            return self._apply_constructor(
+                    apply_func(func),
+                    dtype=dtype,
+                    name=name,
+                    index_constructor=index_constructor,
+                    columns_constructor=columns_constructor,
+                    )
+
+        if columns_constructor is not None:
+            raise RuntimeError('Cannot use `columns_constructor` in this type of apply.')
+        return self._apply_constructor(
+                apply_func(func),
+                dtype=dtype,
+                name=name,
+                index_constructor=index_constructor,
+                )
+
+
+    @doc_inject(selector='apply')
+    def apply_pool(self,
+            func: AnyCallable,
+            *,
+            dtype: DtypeSpecifier = None,
+            name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor]= None,
+            max_workers: tp.Optional[int] = None,
+            chunksize: int = 1,
+            use_threads: bool = False
+            ) -> FrameOrSeries:
+        '''
+        {doc} Employ parallel processing with either the ProcessPoolExecutor or ThreadPoolExecutor.
+
+        Args:
+            {func}
+            *
+            {dtype}
+            {name}
+            {max_workers}
+            {chunksize}
+            {use_threads}
+        '''
+        # only use when we need pairs of values to dynamically create an Index
+        if IterNodeApplyType.is_items(self._apply_type):
+            apply_func = self._apply_iter_items_parallel
+        else:
+            apply_func = self._apply_iter_parallel
+        return self._apply_constructor(
+                apply_func(func,
+                        max_workers=max_workers,
+                        chunksize=chunksize,
+                        use_threads=use_threads,
+                        ),
+                dtype=dtype,
+                name=name,
+                index_constructor=index_constructor,
+                )
+
+    def __iter__(self) -> tp.Union[
+            tp.Iterator[tp.Any],
+            tp.Iterator[tp.Tuple[tp.Any, tp.Any]]
+            ]:
+        '''
+        Return a generator based on the yield type.
+        '''
+        if self._yield_type is IterNodeType.VALUES:
+            yield from self._func_values()
+        else:
+            yield from self._func_items()
+
+
+class IterNodeDelegateMapable(IterNodeDelegate[FrameOrSeries]):
+    '''
+    Delegate returned from :obj:`static_frame.IterNode`, providing iteration as well as a family of apply methods.
+    '''
+
+    __slots__ = ()
+
+    INTERFACE = IterNodeDelegate.INTERFACE + (
+            'map_all',
+            'map_all_iter',
+            'map_all_iter_items',
+            'map_any',
+            'map_any_iter',
+            'map_any_iter_items',
+            'map_fill',
+            'map_fill_iter',
+            'map_fill_iter_items',
+            )
 
     @doc_inject(selector='map_any')
     def map_any_iter_items(self,
@@ -356,138 +514,21 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
                 index_constructor=index_constructor,
                 )
 
-
-    #---------------------------------------------------------------------------
-    @doc_inject(selector='apply')
-    def apply_iter_items(self,
-            func: AnyCallable,
-            ) -> tp.Iterator[tp.Tuple[tp.Any, tp.Any]]:
-        '''
-        {doc} A generator of resulting key, value pairs.
-
-        Args:
-            {func}
-        '''
-        # depend on yield type, we determine what the passed in function expects to
-        if self._yield_type is IterNodeType.VALUES:
-            yield from ((k, func(v)) for k, v in self._func_items())
-        else:
-            yield from ((k, func(k, v)) for k, v in self._func_items())
-
-    @doc_inject(selector='apply')
-    def apply_iter(self,
-            func: AnyCallable
-            ) -> tp.Iterator[tp.Any]:
-        '''
-        {doc} A generator of resulting values.
-
-        Args:
-            {func}
-        '''
-        if self._yield_type is IterNodeType.VALUES:
-            yield from (func(v) for v in self._func_values())
-        else:
-            yield from (func(k, v) for k, v in self._func_items())
-
-    @doc_inject(selector='apply')
-    def apply(self,
-            func: AnyCallable,
-            *,
-            dtype: DtypeSpecifier = None,
-            name: NameType = None,
-            index_constructor: tp.Optional[IndexConstructor]= None,
-            ) -> FrameOrSeries:
-        '''
-        {doc} Returns a new container.
-
-        Args:
-            {func}
-            {dtype}
-        '''
-        if not callable(func):
-            raise RuntimeError('use map_fill(), map_any(), or map_all() for applying a mapping type')
-
-        if IterNodeApplyType.is_items(self._apply_type):
-            apply_func = self.apply_iter_items
-        else:
-            apply_func = self.apply_iter
-
-        return self._apply_constructor(
-                apply_func(func),
-                dtype=dtype,
-                name=name,
-                index_constructor=index_constructor,
-                )
-
-    @doc_inject(selector='apply')
-    def apply_pool(self,
-            func: AnyCallable,
-            *,
-            dtype: DtypeSpecifier = None,
-            name: NameType = None,
-            index_constructor: tp.Optional[IndexConstructor]= None,
-            max_workers: tp.Optional[int] = None,
-            chunksize: int = 1,
-            use_threads: bool = False
-            ) -> FrameOrSeries:
-        '''
-        {doc} Employ parallel processing with either the ProcessPoolExecutor or ThreadPoolExecutor.
-
-        Args:
-            {func}
-            *
-            {dtype}
-            {name}
-            {max_workers}
-            {chunksize}
-            {use_threads}
-        '''
-        # only use when we need pairs of values to dynamically create an Index
-        if IterNodeApplyType.is_items(self._apply_type):
-            apply_func = self._apply_iter_items_parallel
-        else:
-            apply_func = self._apply_iter_parallel
-        return self._apply_constructor(
-                apply_func(func,
-                        max_workers=max_workers,
-                        chunksize=chunksize,
-                        use_threads=use_threads,
-                        ),
-                dtype=dtype,
-                name=name,
-                index_constructor=index_constructor,
-                )
-
-    def __iter__(self) -> tp.Union[
-            tp.Iterator[tp.Any],
-            tp.Iterator[tp.Tuple[tp.Any, tp.Any]]
-            ]:
-        '''
-        Return a generator based on the yield type.
-        '''
-        if self._yield_type is IterNodeType.VALUES:
-            yield from self._func_values()
-        else:
-            yield from self._func_items()
-
-
-
 #-------------------------------------------------------------------------------
 
-_ITER_NODE_SLOTS = (
-        '_container',
-        '_func_values',
-        '_func_items',
-        '_yield_type',
-        '_apply_type'
-        )
-
 class IterNode(tp.Generic[FrameOrSeries]):
     '''Interface to a type of iteration on :obj:`static_frame.Series` and :obj:`static_frame.Frame`.
     '''
     # Stores two version of a generator function: one to yield single values, another to yield items pairs. The latter is needed in all cases, as when we use apply we return a Series, and need to have recourse to an index.
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = (
+        '_container',
+        '_func_values',
+        '_func_items',
+        '_yield_type',
+        '_apply_type',
+        )
+    CLS_DELEGATE = IterNodeDelegate
 
     def __init__(self, *,
             container: FrameOrSeries,
@@ -510,7 +551,7 @@ class IterNode(tp.Generic[FrameOrSeries]):
     #---------------------------------------------------------------------------
     # apply constructors
 
-    def to_series_values(self,
+    def to_series_from_values(self,
             values: tp.Iterator[tp.Any],
             *,
             dtype: DtypeSpecifier,
@@ -532,18 +573,18 @@ class IterNode(tp.Generic[FrameOrSeries]):
             index = index_constructor(index)
 
         # PERF: passing count here permits faster generator realization
-        values, _ = iterable_to_array_1d(
+        array, _ = iterable_to_array_1d(
                 values,
                 count=index.shape[0],
                 dtype=dtype,
                 )
-        return Series(values,
+        return Series(array,
                 name=name,
                 index=index,
                 own_index=own_index,
                 )
 
-    def to_series_items(self,
+    def to_series_from_items(self,
             pairs: tp.Iterable[tp.Tuple[tp.Hashable, tp.Any]],
             *,
             dtype: DtypeSpecifier = None,
@@ -579,7 +620,7 @@ class IterNode(tp.Generic[FrameOrSeries]):
                 index_constructor=index_constructor_final,
                 )
 
-    def to_series_items_group(self,
+    def to_series_from_group_items(self,
             pairs: tp.Iterable[tp.Tuple[tp.Hashable, tp.Any]],
             *,
             dtype: DtypeSpecifier = None,
@@ -587,8 +628,8 @@ class IterNode(tp.Generic[FrameOrSeries]):
             index_constructor: tp.Optional[IndexConstructor]= None,
             name_index: NameType = None,
             ) -> 'Series':
-        from static_frame.core.series import Series
         from static_frame.core.index import Index
+        from static_frame.core.series import Series
 
         # NOTE: when used on labels, this key is given; when used on labels (indices) depth_level is given; only take the key if it is a hashable (a string or a tuple, not a slice, list, or array)
 
@@ -603,38 +644,48 @@ class IterNode(tp.Generic[FrameOrSeries]):
                 index_constructor=index_constructor
                 )
 
-    def to_frame_elements(self,
+    def to_frame_from_elements(self,
             items: tp.Iterable[tp.Tuple[
                     tp.Tuple[tp.Hashable, tp.Hashable], tp.Any]],
             *,
             dtype: DtypeSpecifier = None,
             name: NameType = None,
             index_constructor: tp.Optional[IndexConstructor]= None,
+            columns_constructor: tp.Optional[IndexConstructor]= None,
             axis: int = 0,
             ) -> 'Frame':
+        # NOTE: this is only called from `Frame` to produce a new `Frame`
+
         from static_frame.core.frame import Frame
 
-        index_constructor = (self._container._index.from_labels
-                if index_constructor is None else index_constructor)
+        if index_constructor is not None:
+            index = index_constructor(self._container._index)
+        else:
+            index = self._container._index
+
+        assert isinstance(self._container, Frame) # mypy
+
+        if columns_constructor is not None:
+            columns = columns_constructor(self._container._columns)
+        else:
+            columns = self._container._columns
 
-        assert isinstance(self._container, Frame)
         return self._container.__class__.from_element_items(
                 items,
-                index=self._container._index,
-                columns=self._container._columns,
+                index=index,
+                columns=columns,
                 axis=axis,
                 own_index=True,
-                index_constructor=index_constructor,
-                columns_constructor=self._container._columns.from_labels,
+                own_columns=True,
                 name=name,
                 )
 
-    def to_index_labels(self,
+    def to_index_from_labels(self,
             values: tp.Iterator[tp.Hashable], #pylint: disable=function-redefined
             dtype: DtypeSpecifier = None,
             name: NameType = None,
             index_constructor: tp.Optional[IndexConstructor]= None,
-            ) -> np.ndarray:
+            ) -> NDArrayAny:
         # NOTE: name argument is for common interface
         if index_constructor is not None:
             raise RuntimeError('index_constructor not supported with this interface')
@@ -644,9 +695,9 @@ class IterNode(tp.Generic[FrameOrSeries]):
         return array
 
     #---------------------------------------------------------------------------
-    def get_delegate(self,
-            **kwargs: object
-            ) -> IterNodeDelegate[FrameOrSeries]:
+    def _get_delegate_kwargs(self,
+            **kwargs: object,
+            ) -> tp.Dict[str, tp.Any]:
         '''
         In usage as an iteator, the args passed here are expected to be argument for the core iterators, i.e., axis arguments.
 
@@ -664,38 +715,38 @@ class IterNode(tp.Generic[FrameOrSeries]):
         apply_constructor: tp.Callable[..., tp.Union['Frame', 'Series']]
 
         if self._apply_type is IterNodeApplyType.SERIES_VALUES:
-            apply_constructor = partial(self.to_series_values, axis=axis)
+            apply_constructor = partial(self.to_series_from_values, axis=axis)
 
         elif self._apply_type is IterNodeApplyType.SERIES_ITEMS:
-            apply_constructor = partial(self.to_series_items, axis=axis)
+            apply_constructor = partial(self.to_series_from_items, axis=axis)
 
         elif self._apply_type is IterNodeApplyType.SERIES_ITEMS_GROUP_VALUES:
             try:
                 name_index = name_filter(kwargs.get('key', None))
             except TypeError:
                 name_index = None
-            apply_constructor = partial(self.to_series_items_group,
+            apply_constructor = partial(self.to_series_from_group_items,
                     name_index=name_index,
                     )
 
         elif self._apply_type is IterNodeApplyType.SERIES_ITEMS_GROUP_LABELS:
             # will always have `depth_level` in kwargs, and for Frame an axis; could attempt to get name from the index if it has a name
             name_index = None
-            apply_constructor = partial(self.to_series_items_group,
+            apply_constructor = partial(self.to_series_from_group_items,
                     name_index=name_index,
                     )
 
         elif self._apply_type is IterNodeApplyType.FRAME_ELEMENTS:
             assert isinstance(self._container, Frame) # for typing
-            apply_constructor = partial(self.to_frame_elements, axis=axis)
+            apply_constructor = partial(self.to_frame_from_elements, axis=axis)
 
         elif self._apply_type is IterNodeApplyType.INDEX_LABELS:
-            apply_constructor = self.to_index_labels
+            apply_constructor = self.to_index_from_labels # type: ignore
 
         else:
             raise NotImplementedError(self._apply_type) #pragma: no cover
 
-        return IterNodeDelegate(
+        return dict(
                 func_values=func_values,
                 func_items=func_items,
                 yield_type=self._yield_type,
@@ -703,40 +754,70 @@ class IterNode(tp.Generic[FrameOrSeries]):
                 apply_type=self._apply_type,
                 )
 
+    def get_delegate(self,
+            **kwargs: object,
+            ) -> IterNodeDelegate[FrameOrSeries]:
+        return IterNodeDelegate(**self._get_delegate_kwargs(**kwargs))
+
+    def get_delegate_mapable(self,
+            **kwargs: object,
+            ) -> IterNodeDelegateMapable[FrameOrSeries]:
+        return IterNodeDelegateMapable(**self._get_delegate_kwargs(**kwargs))
 
 #-------------------------------------------------------------------------------
 # specialize IterNode based on arguments given to __call__
 
 class IterNodeNoArg(IterNode[FrameOrSeries]):
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
+    CLS_DELEGATE = IterNodeDelegate
 
     def __call__(self,
             ) -> IterNodeDelegate[FrameOrSeries]:
         return IterNode.get_delegate(self)
 
 
-class IterNodeAxis(IterNode[FrameOrSeries]):
+class IterNodeNoArgMapable(IterNode[FrameOrSeries]):
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
+    CLS_DELEGATE = IterNodeDelegateMapable
+
+    def __call__(self,
+            ) -> IterNodeDelegateMapable[FrameOrSeries]:
+        return IterNode.get_delegate_mapable(self)
+
+class IterNodeAxisElement(IterNode[FrameOrSeries]):
+
+    __slots__ = ()
+    CLS_DELEGATE = IterNodeDelegateMapable
 
     def __call__(self,
             *,
             axis: int = 0
-            ) -> IterNodeDelegate[FrameOrSeries]:
-        return IterNode.get_delegate(self, axis=axis)
+            ) -> IterNodeDelegateMapable[FrameOrSeries]:
+        return IterNode.get_delegate_mapable(self, axis=axis)
 
+class IterNodeAxis(IterNode[FrameOrSeries]):
+
+    __slots__ = ()
+
+    def __call__(self,
+            *,
+            axis: int = 0
+            ) -> IterNodeDelegateMapable[FrameOrSeries]:
+        return IterNode.get_delegate_mapable(self, axis=axis)
 
 class IterNodeConstructorAxis(IterNode[FrameOrSeries]):
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
+    CLS_DELEGATE = IterNodeDelegateMapable
 
     def __call__(self,
             *,
             axis: int = 0,
             constructor: tp.Optional[TupleConstructorType] = None,
-            ) -> IterNodeDelegate[FrameOrSeries]:
-        return IterNode.get_delegate(self,
+            ) -> IterNodeDelegateMapable[FrameOrSeries]:
+        return IterNode.get_delegate_mapable(self,
                 axis=axis,
                 constructor=constructor,
                 )
@@ -746,7 +827,7 @@ class IterNodeGroup(IterNode[FrameOrSeries]):
     Iterator on 1D groupings where no args are required (but axis is retained for compatibility)
     '''
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
 
     def __call__(self,
             *,
@@ -759,7 +840,7 @@ class IterNodeGroupAxis(IterNode[FrameOrSeries]):
     Iterator on 2D groupings where key and axis are required.
     '''
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
 
     def __call__(self,
             key: KEY_ITERABLE_TYPES, # type: ignore
@@ -770,19 +851,49 @@ class IterNodeGroupAxis(IterNode[FrameOrSeries]):
         return IterNode.get_delegate(self, key=key, axis=axis, drop=drop)
 
 
+class IterNodeGroupOther(IterNode[FrameOrSeries]):
+    '''
+    Iterator on 1D groupings where group values are provided.
+    '''
+
+    __slots__ = ()
+
+    def __call__(self,
+            other: tp.Union[NDArrayAny, 'Index', 'Series', tp.Iterable[tp.Any]],
+            *,
+            fill_value: tp.Any = np.nan,
+            axis: int = 0
+            ) -> IterNodeDelegate[FrameOrSeries]:
+
+        index_ref = (self._container._index if axis == 0
+                else self._container._columns) # type: ignore
+
+        group_source = group_from_container(
+                index=index_ref,
+                group_source=other,
+                fill_value=fill_value,
+                axis=axis,
+                )
+
+        # kwargs are partialed into finc_values, func_items
+        return IterNode.get_delegate(self,
+                axis=axis,
+                group_source=group_source,
+                )
+
 class IterNodeDepthLevel(IterNode[FrameOrSeries]):
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
 
     def __call__(self,
             depth_level: tp.Optional[DepthLevelSpecifier] = None
-            ) -> IterNodeDelegate[FrameOrSeries]:
-        return IterNode.get_delegate(self, depth_level=depth_level)
+            ) -> IterNodeDelegateMapable[FrameOrSeries]:
+        return IterNode.get_delegate_mapable(self, depth_level=depth_level)
 
 
 class IterNodeDepthLevelAxis(IterNode[FrameOrSeries]):
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
 
     def __call__(self,
             depth_level: DepthLevelSpecifier = 0,
@@ -794,7 +905,7 @@ class IterNodeDepthLevelAxis(IterNode[FrameOrSeries]):
 
 class IterNodeWindow(IterNode[FrameOrSeries]):
 
-    __slots__ = _ITER_NODE_SLOTS
+    __slots__ = ()
 
     def __call__(self, *,
             size: int,
