@@ -1,37 +1,39 @@
+from __future__ import annotations
+
 import typing as tp
-from functools import partial
 from collections import defaultdict
-from itertools import repeat
-from itertools import product
+from functools import partial
 from itertools import chain
+from itertools import product
+from itertools import repeat
 
 import numpy as np
 from arraykit import resolve_dtype
 from arraykit import resolve_dtype_iter
 
-from static_frame.core.index_base import IndexBase
+from static_frame.core.container_util import index_from_optional_constructor
 from static_frame.core.index import Index
+from static_frame.core.index_base import IndexBase
 from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.type_blocks import TypeBlocks
+from static_frame.core.util import DEFAULT_FAST_SORT_KIND
+from static_frame.core.util import AnyCallable
 from static_frame.core.util import DepthLevelSpecifier
 from static_frame.core.util import IndexConstructor
+from static_frame.core.util import NameType
+from static_frame.core.util import TSortKinds
 from static_frame.core.util import UFunc
+from static_frame.core.util import dtype_from_element
+from static_frame.core.util import iterable_to_array_1d
 from static_frame.core.util import ufunc_dtype_to_dtype
 from static_frame.core.util import ufunc_unique
 from static_frame.core.util import ufunc_unique1d
-from static_frame.core.util import EMPTY_TUPLE
-from static_frame.core.util import DEFAULT_FAST_SORT_KIND
-from static_frame.core.util import dtype_from_element
-from static_frame.core.util import NameType
-from static_frame.core.util import iterable_to_array_1d
-
-
-from static_frame.core.container_util import index_from_optional_constructor
-from static_frame.core.type_blocks import TypeBlocks
 
 if tp.TYPE_CHECKING:
-    from static_frame.core.frame import Frame #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.series import Series #pylint: disable=W0611 #pragma: no cover
-
+    from static_frame.core.frame import Frame  # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.series import Series  # pylint: disable=W0611 #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 
 #-------------------------------------------------------------------------------
@@ -41,7 +43,7 @@ def extrapolate_column_fields(
         data_fields: tp.Sequence[tp.Hashable],
         func_fields: tp.Iterable[tp.Hashable],
         ) -> tp.Iterable[tp.Hashable]:
-    '''"Determine columns to be reatined from gruop and data fields.
+    '''Determine columns to be reatined from gruop and data fields.
     Used in Frame.pivot.
 
     Args:
@@ -83,12 +85,12 @@ def pivot_records_dtypes(
         data_fields: tp.Iterable[tp.Hashable],
         func_single: tp.Optional[UFunc],
         func_map: tp.Sequence[tp.Tuple[tp.Hashable, UFunc]]
-        ) -> tp.Iterator[tp.Optional[np.dtype]]:
+        ) -> tp.Iterator[tp.Optional[DtypeAny]]:
     '''
     Iterator of ordered dtypes, providing multiple dtypes per field when func_map is provided.
     '''
     for field in data_fields:
-        dtype = dtype_map[field]
+        dtype: DtypeAny = dtype_map[field] # type: ignore
         if func_single:
             yield ufunc_dtype_to_dtype(func_single, dtype)
         else: # we assume
@@ -100,28 +102,28 @@ def pivot_records_items_to_frame(
         blocks: TypeBlocks,
         group_fields_iloc: tp.Iterable[tp.Hashable],
         group_depth: int,
-        data_fields_iloc: tp.Iterable[tp.Hashable],
+        data_fields_iloc: tp.Iterable[int],
         func_single: tp.Optional[UFunc],
         func_map: tp.Sequence[tp.Tuple[tp.Hashable, UFunc]],
         func_no: bool,
-        kind: str,
+        kind: TSortKinds,
         columns_constructor: IndexConstructor,
-        columns: tp.List[tp.Hashable],
+        columns: tp.Sequence[tp.Hashable],
         index_constructor: IndexConstructor,
-        dtypes: tp.Tuple[tp.Optional[np.dtype]],
+        dtypes: tp.Tuple[tp.Optional[DtypeAny], ...],
         frame_cls: tp.Type['Frame'],
         ) -> 'Frame':
     '''
     Given a Frame and pivot parameters, perform the group by ont he group_fields and within each group,
     '''
     group_key = group_fields_iloc if group_depth > 1 else group_fields_iloc[0] #type: ignore
-    record_size = len(data_fields_iloc) * (1 if (func_single or func_no) else len(func_map))
+    record_size = len(data_fields_iloc) * (1 if (func_single or func_no) else len(func_map)) # type: ignore
 
-    index_labels = []
+    index_labels: tp.List[tp.Hashable] = []
     arrays: tp.List[tp.List[tp.Any]] = [list() for _ in range(record_size)]
 
     for label, _, part in blocks.group(axis=0, key=group_key, kind=kind):
-        index_labels.append(label)
+        index_labels.append(label) # type: ignore
         if func_no:
             if len(part) != 1:
                 raise RuntimeError('pivot requires aggregation of values; provide a `func` argument.')
@@ -138,7 +140,7 @@ def pivot_records_items_to_frame(
                     arrays[i].append(func(values))
                     i += 1
 
-    def gen() -> tp.Iterator[np.ndarray]:
+    def gen() -> tp.Iterator[NDArrayAny]:
         for b, dtype in zip(arrays, dtypes):
             if dtype is None:
                 array, _ = iterable_to_array_1d(b)
@@ -161,23 +163,23 @@ def pivot_records_items_to_blocks(*,
         blocks: TypeBlocks,
         group_fields_iloc: tp.Iterable[tp.Hashable],
         group_depth: int,
-        data_fields_iloc: tp.Iterable[tp.Hashable],
+        data_fields_iloc: tp.Iterable[int],
         func_single: tp.Optional[UFunc],
         func_map: tp.Sequence[tp.Tuple[tp.Hashable, UFunc]],
         func_no: bool,
         fill_value: tp.Any,
-        fill_value_dtype: np.dtype,
+        fill_value_dtype: DtypeAny,
         index_outer: 'IndexBase',
-        dtypes: tp.Tuple[tp.Optional[np.dtype]],
-        kind: str,
-        ) -> tp.List[np.ndarray]:
+        dtypes: tp.Tuple[tp.Optional[DtypeAny], ...],
+        kind: TSortKinds,
+        ) -> tp.List[NDArrayAny]:
     '''
     Given a Frame and pivot parameters, perform the group by ont he group_fields and within each group,
     '''
     # NOTE: this delivers results by label, row for use in a Frame.from_records_items constructor
 
     group_key = group_fields_iloc if group_depth > 1 else group_fields_iloc[0] #type: ignore
-    arrays: tp.List[tp.Union[tp.List[tp.Any], np.ndarray]] = []
+    arrays: tp.List[tp.Union[tp.List[tp.Any], NDArrayAny]] = []
     for dtype in dtypes:
         if dtype is None:
             # we can use fill_value here, as either it will be completely replaced (and not effect dtype evaluation) or be needed (and already there)
@@ -185,12 +187,13 @@ def pivot_records_items_to_blocks(*,
         else:
             arrays.append(np.empty(len(index_outer), dtype=dtype))
 
-    # try to use the dtype specifieid; fill values at end of necessary
-    iloc_found: tp.Set[int] = set()
+    # try to use the dtype specified; fill values at end if necessary
+    # collect all possible ilocs, and remove as observerd; if any remain, we have fill targets
+    iloc_not_found: tp.Set[int] = set(range(len(index_outer)))
     # each group forms a row, each label a value in the index
     for label, _, part in blocks.group(axis=0, key=group_key, kind=kind):
         iloc: int = index_outer._loc_to_iloc(label) #type: ignore
-        iloc_found.add(iloc)
+        iloc_not_found.remove(iloc)
         if func_no:
             if len(part) != 1:
                 raise RuntimeError('pivot requires aggregation of values; provide a `func` argument.')
@@ -208,10 +211,10 @@ def pivot_records_items_to_blocks(*,
                     arrays[arrays_key][iloc] = func(values)
                     arrays_key += 1
 
-    if len(iloc_found) != len(index_outer):
-        # we did not fill all arrrays and have values that need to be filled
+    if iloc_not_found:
+        # we did not fill all arrays and have values that need to be filled
         # order does not matter
-        fill_targets = list(set(range(len(index_outer))) - iloc_found)
+        fill_targets = list(iloc_not_found)
         # mutate in place then make immutable
         for arrays_key in range(len(arrays)): #pylint: disable=C0200
             array = arrays[arrays_key]
@@ -231,8 +234,8 @@ def pivot_records_items_to_blocks(*,
             if not array.__class__ is np.ndarray: # a list
                 array, _ = iterable_to_array_1d(array, count=len(index_outer))
                 arrays[arrays_key] = array # re-assign new array
-            array.flags.writeable = False
-    return arrays
+            array.flags.writeable = False # type: ignore
+    return arrays # type: ignore # we have converted all sequences to arrays at this point
 
 
 
@@ -240,14 +243,14 @@ def pivot_items_to_block(*,
         blocks: TypeBlocks,
         group_fields_iloc: tp.Iterable[tp.Hashable],
         group_depth: int,
-        data_field_iloc: tp.Hashable,
+        data_field_iloc: int,
         func_single: tp.Optional[UFunc],
-        dtype: tp.Optional[np.dtype],
+        dtype: tp.Optional[DtypeAny],
         fill_value: tp.Any,
-        fill_value_dtype: np.dtype,
+        fill_value_dtype: DtypeAny,
         index_outer: 'IndexBase',
-        kind: str,
-        ) -> np.ndarray:
+        kind: TSortKinds,
+        ) -> NDArrayAny:
     '''
     Specialized generator of pairs for when we have only one data_field and one function.
     '''
@@ -313,14 +316,14 @@ def pivot_items_to_frame(*,
         blocks: TypeBlocks,
         group_fields_iloc: tp.Iterable[tp.Hashable],
         group_depth: int,
-        data_field_iloc: tp.Hashable,
-        func_single: tp.Optional[UFunc],
+        data_field_iloc: int,
+        func_single: tp.Optional[AnyCallable],
         frame_cls: tp.Type['Frame'],
         name: NameType,
-        dtype: np.dtype,
+        dtype: DtypeAny | None,
         index_constructor: IndexConstructor,
         columns_constructor: IndexConstructor,
-        kind: str,
+        kind: TSortKinds,
         ) -> 'Frame':
     '''
     Specialized generator of pairs for when we have only one data_field and one function.
@@ -331,7 +334,7 @@ def pivot_items_to_frame(*,
     group_key = group_fields_iloc if group_depth > 1 else group_fields_iloc[0] #type: ignore
 
     if func_single:
-        labels = []
+        labels: tp.List[tp.Hashable] = []
         values = []
         for label, _, v in blocks.group_extract(
                 axis=0,
@@ -339,7 +342,7 @@ def pivot_items_to_frame(*,
                 extract=data_field_iloc,
                 kind=kind,
                 ):
-            labels.append(label)
+            labels.append(label) # type: ignore
             values.append(func_single(v))
 
         if dtype is None:
@@ -380,14 +383,14 @@ def pivot_core(
         func_map: tp.Sequence[tp.Tuple[tp.Hashable, UFunc]],
         fill_value: object = np.nan,
         index_constructor: IndexConstructor = None,
-        kind: str = DEFAULT_FAST_SORT_KIND,
+        kind: TSortKinds = DEFAULT_FAST_SORT_KIND,
         ) -> 'Frame':
     '''Core implementation of Frame.pivot(). The Frame has already been reduced to just relevant columns, and all fields groups are normalized as lists of hashables.
     '''
-    from static_frame.core.series import Series
     from static_frame.core.frame import Frame
+    from static_frame.core.series import Series
 
-    func_no = func_single is None and func_map is EMPTY_TUPLE
+    func_no = func_single is None and func_map == ()
 
     data_fields_len = len(data_fields)
     index_depth = len(index_fields)
@@ -412,15 +415,18 @@ def pivot_core(
         columns_name = columns_name[0] # type: ignore
         columns_constructor = partial(frame._COLUMNS_CONSTRUCTOR, name=columns_name)
     else:
-        columns_constructor = partial(frame._COLUMNS_HIERARCHY_CONSTRUCTOR.from_labels,
+        columns_constructor = partial(frame._COLUMNS_HIERARCHY_CONSTRUCTOR.from_labels, # type: ignore
                 depth_reference=columns_depth,
                 name=columns_name)
 
+    dtype_single: DtypeAny | None
     dtype_map = frame.dtypes # returns a Series
+
+    dtypes_per_data_fields: tp.Tuple[DtypeAny | None, ...]
     if func_no:
         dtypes_per_data_fields = tuple(dtype_map[field] for field in data_fields)
         if data_fields_len == 1:
-            dtype_single = dtype_map[data_fields[0]]
+            dtype_single = dtype_map[data_fields[0]] # type: ignore
     else:
         dtypes_per_data_fields = tuple(pivot_records_dtypes(
                 dtype_map=dtype_map,
@@ -429,7 +435,7 @@ def pivot_core(
                 func_map=func_map,
                 ))
         if func_single and data_fields_len == 1:
-            dtype_single = ufunc_dtype_to_dtype(func_single, dtype_map[data_fields[0]])
+            dtype_single = ufunc_dtype_to_dtype(func_single, dtype_map[data_fields[0]]) # type: ignore
 
     fill_value_dtype = dtype_from_element(fill_value)
 
@@ -479,7 +485,7 @@ def pivot_core(
                     )
         columns_final = (f.columns.rename(columns_name) if columns_depth == 1
                 else columns_constructor(f.columns))
-        return f.relabel(columns=columns_final) #type: ignore
+        return f.relabel(columns=columns_final)
 
     #---------------------------------------------------------------------------
     # Second major branch: we are grouping by index and columns fields. This is done with an outer and inner gruop by. The index is calculated ahead of time.
@@ -509,7 +515,7 @@ def pivot_core(
         # derive the column fields represented by this group
         sub_columns = extrapolate_column_fields(
                 columns_fields,
-                group if not retuple_group_label else (group,),
+                group if not retuple_group_label else (group,), # type: ignore
                 data_fields,
                 func_fields,
                 )
@@ -517,7 +523,7 @@ def pivot_core(
 
         sub_frame: Frame
         # if sub_columns length is 1, that means that we only need to extract one column out of the sub blocks
-        if len(sub_columns) == 1:
+        if len(sub_columns) == 1: # type: ignore
             sub_blocks.append(pivot_items_to_block(blocks=sub,
                             group_fields_iloc=index_fields_iloc,
                             group_depth=index_depth,
@@ -569,7 +575,7 @@ def pivot_outer_index(
     if index_depth == 1:
         index_values = ufunc_unique1d(
                 frame._blocks._extract_array_column(
-                        frame._columns._loc_to_iloc(index_loc)),
+                        frame._columns._loc_to_iloc(index_loc)), # type: ignore
                 )
         index_values.flags.writeable = False
         name = index_fields[0]
@@ -590,7 +596,7 @@ def pivot_outer_index(
         index_inner = index_from_optional_constructor( # type: ignore
                 index_values,
                 default_constructor=partial(
-                        IndexHierarchy.from_labels,
+                        IndexHierarchy.from_values_per_depth,
                         name=name,
                         ),
                 explicit_constructor=None if index_constructor is None else partial(index_constructor, name=name),
@@ -603,16 +609,16 @@ def pivot_outer_index(
 class PivotIndexMap(tp.NamedTuple):
     targets_unique: tp.Iterable[tp.Hashable]
     target_depth: int
-    target_select: np.ndarray
+    target_select: NDArrayAny
     group_to_target_map: tp.Dict[tp.Optional[tp.Hashable], tp.Dict[tp.Any, int]]
     group_depth: int
-    group_select: np.ndarray
-    group_to_dtype: tp.Dict[tp.Optional[tp.Hashable], np.dtype]
+    group_select: NDArrayAny
+    group_to_dtype: tp.Dict[tp.Hashable | None, DtypeAny]
 
 def pivot_index_map(*,
         index_src: IndexBase,
         depth_level: DepthLevelSpecifier,
-        dtypes_src: tp.Optional[tp.Sequence[np.dtype]],
+        dtypes_src: NDArrayAny | None, # array of dtypes
         ) -> PivotIndexMap:
     '''
     Args:
@@ -636,9 +642,10 @@ def pivot_index_map(*,
 
     group_depth = len(group_arrays)
     target_depth = len(target_arrays)
-    group_to_dtype: tp.Dict[tp.Optional[tp.Hashable], np.dtype] = {}
+    group_to_dtype: tp.Dict[tp.Optional[tp.Hashable], DtypeAny] = {}
     targets_unique: tp.Iterable[tp.Hashable]
 
+    group_to_target_map: tp.Dict[tp.Any, tp.Dict[tp.Any, int]]
     if group_depth == 0:
         # targets must be a tuple
         group_to_target_map = {
@@ -649,9 +656,9 @@ def pivot_index_map(*,
             group_to_dtype[None] = resolve_dtype_iter(dtypes_src)
     else:
         group_to_target_map = defaultdict(dict)
-        targets_unique = dict() # Store targets in order observed
+        targets_unique = {} # Store targets in order observed
 
-        for axis_idx, (group, target, dtype) in enumerate(zip(
+        for axis_idx, (group, target, dtype) in enumerate(zip( # type: ignore
                 zip(*group_arrays), # get tuples of len 1 to depth
                 zip(*target_arrays),
                 (dtypes_src if dtypes_src is not None else repeat(None)),
@@ -672,7 +679,7 @@ def pivot_index_map(*,
             targets_unique=targets_unique,
             target_depth=target_depth,
             target_select=target_select,
-            group_to_target_map=group_to_target_map, #type: ignore
+            group_to_target_map=group_to_target_map,
             group_depth=group_depth,
             group_select=group_select,
             group_to_dtype=group_to_dtype
@@ -688,11 +695,10 @@ class PivotDeriveConstructors(tp.NamedTuple):
 def pivot_derive_constructors(*,
         contract_src: IndexBase,
         expand_src: IndexBase,
-        group_select: np.ndarray, # Boolean
+        group_select: NDArrayAny, # Boolean
         group_depth: int,
-        target_select: np.ndarray,
-        # target_depth: int,
-        group_to_target_map: tp.Dict[tp.Hashable, tp.Tuple[tp.Hashable]],
+        target_select: NDArrayAny,
+        group_to_target_map: tp.Dict[tp.Optional[tp.Hashable], tp.Dict[tp.Any, int]],
         expand_is_columns: bool,
         frame_cls: tp.Type['Frame'],
         ) -> PivotDeriveConstructors:
@@ -736,7 +742,7 @@ def pivot_derive_constructors(*,
     if expand_src.depth == 1:
         expand_types = [expand_src.__class__]
     else:
-        expand_types = list(expand_src._levels.index_types()) #type: ignore
+        expand_types = expand_src.index_types.values.tolist()
 
     if contract_src.depth == 1:
         expand_types.append(contract_src.__class__)
