@@ -1,11 +1,14 @@
-import typing as tp
+from __future__ import annotations
+
 import importlib
 import platform as platform_mod
 import sys
+import typing as tp
 
-from static_frame.core.series import Series
-from static_frame.core.display import Display
 import static_frame
+from static_frame.core.display import Display
+from static_frame.core.series import Series
+
 
 class Platform:
 
@@ -37,9 +40,9 @@ class Platform:
                     continue #pragma: no cover
 
                 if hasattr(mod, '__version__'):
-                    yield package, mod.__version__ #type: ignore
+                    yield package, mod.__version__
                 elif hasattr(mod, 'version'): # msgpack
-                    yield package, mod.version #type: ignore
+                    yield package, mod.version
                 else:
                     yield package, None
 
@@ -47,6 +50,6 @@ class Platform:
 
     @classmethod
     def display(cls) -> Display:
-        return cls.to_series().display_wide() #type: ignore [no-any-return]
+        return cls.to_series().display_wide()
 
 
