@@ -1,57 +1,69 @@
+from __future__ import annotations
+
 import typing as tp
+from functools import partial
+from itertools import chain
 from itertools import repeat
 from itertools import zip_longest
-from functools import partial
 
 import numpy as np
 
+from static_frame.core.axis_map import bus_to_hierarchy
+from static_frame.core.axis_map import get_extractor
 from static_frame.core.bus import Bus
 from static_frame.core.container import ContainerBase
 from static_frame.core.container_util import axis_window_items
+from static_frame.core.container_util import iter_component_signature_bytes
 from static_frame.core.display import Display
+from static_frame.core.display import DisplayActive
 from static_frame.core.display import DisplayHeader
 from static_frame.core.display_config import DisplayConfig
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import AxisInvalid
-from static_frame.core.exception import ErrorInitQuilt
 from static_frame.core.exception import ErrorInitIndexNonUnique
+from static_frame.core.exception import ErrorInitQuilt
 from static_frame.core.exception import NotImplementedAxis
 from static_frame.core.frame import Frame
 from static_frame.core.hloc import HLoc
+from static_frame.core.index_auto import IndexAutoConstructorFactory
 from static_frame.core.index_base import IndexBase
+from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.index_hierarchy import TreeNodeT
+from static_frame.core.node_iter import IterNodeApplyType
 from static_frame.core.node_iter import IterNodeAxis
 from static_frame.core.node_iter import IterNodeConstructorAxis
 from static_frame.core.node_iter import IterNodeType
 from static_frame.core.node_iter import IterNodeWindow
-from static_frame.core.node_iter import IterNodeApplyType
-from static_frame.core.node_selector import InterfaceGetItem
+from static_frame.core.node_selector import InterfaceGetItemCompound
 from static_frame.core.series import Series
 from static_frame.core.store import Store
-from static_frame.core.store import StoreConfigMapInitializer
 from static_frame.core.store_client_mixin import StoreClientMixin
+from static_frame.core.store_config import StoreConfigMapInitializer
 from static_frame.core.store_hdf5 import StoreHDF5
 from static_frame.core.store_sqlite import StoreSQLite
 from static_frame.core.store_xlsx import StoreXLSX
 from static_frame.core.store_zip import StoreZipCSV
+from static_frame.core.store_zip import StoreZipNPY
+from static_frame.core.store_zip import StoreZipNPZ
 from static_frame.core.store_zip import StoreZipParquet
 from static_frame.core.store_zip import StoreZipPickle
 from static_frame.core.store_zip import StoreZipTSV
-from static_frame.core.store_zip import StoreZipNPZ
+from static_frame.core.style_config import StyleConfig
+from static_frame.core.util import INT_TYPES
+from static_frame.core.util import NULL_SLICE
 from static_frame.core.util import AnyCallable
-from static_frame.core.util import get_tuple_constructor
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import GetItemKeyTypeCompound
-from static_frame.core.util import INT_TYPES
 from static_frame.core.util import NameType
-from static_frame.core.util import NULL_SLICE
 from static_frame.core.util import PathSpecifier
 from static_frame.core.util import concat_resolved
-from static_frame.core.style_config import StyleConfig
-from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.util import get_tuple_constructor
 from static_frame.core.yarn import Yarn
 
-from static_frame.core.axis_map import bus_to_hierarchy
-from static_frame.core.axis_map import get_extractor
+if tp.TYPE_CHECKING:
+    from static_frame.core.index import Index  # pylint: disable=W0611 #pragma: no cover
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 class Quilt(ContainerBase, StoreClientMixin):
     '''
@@ -108,25 +120,26 @@ class Quilt(ContainerBase, StoreClientMixin):
             ends = range(starts[1], vector_len, chunksize)
 
         if label_extractor is None:
-            label_extractor = lambda x: x.iloc[0] #type: ignore
+            label_extractor = lambda x: x.iloc[0]
 
-        axis_map_components: tp.Dict[tp.Hashable, IndexBase] = {}
+        axis_map_components: TreeNodeT = {}
         opposite = None
 
         def values() -> tp.Iterator[Frame]:
             nonlocal opposite
 
             for start, end in zip_longest(starts, ends, fillvalue=vector_len):
+                # NOTE: index / columns cannot be IndexHierarchy
                 if axis == 0: # along rows
                     f = frame.iloc[start:end]
-                    label = label_extractor(f.index) #type: ignore
-                    axis_map_components[label] = f.index
+                    label = label_extractor(f.index)
+                    axis_map_components[label] = f.index # type: ignore
                     if opposite is None:
-                        opposite = f.columns
+                        opposite = f.columns # type: ignore
                 elif axis == 1: # along columns
-                    f = frame.iloc[:, start:end]
-                    label = label_extractor(f.columns) #type: ignore
-                    axis_map_components[label] = f.columns
+                    f = frame.iloc[NULL_SLICE, start:end]
+                    label = label_extractor(f.columns) # type: ignore
+                    axis_map_components[label] = f.columns # type: ignore
                     if opposite is None:
                         opposite = f.index
                 else:
@@ -136,7 +149,8 @@ class Quilt(ContainerBase, StoreClientMixin):
         name = name if name else frame.name
         bus = Bus.from_frames(values(), config=config, name=name)
 
-        axis_hierarchy = IndexHierarchy.from_tree(axis_map_components)
+        axis_hierarchy = IndexHierarchy.from_tree(axis_map_components,
+                index_constructors=IndexAutoConstructorFactory)
 
         return cls(bus,
                 axis=axis,
@@ -257,7 +271,7 @@ class Quilt(ContainerBase, StoreClientMixin):
             max_persist: tp.Optional[int] = None,
             ) -> 'Quilt':
         '''
-        Given a file path to zipped parquet :obj:`Quilt` store, return a :obj:`Quilt` instance.
+        Given a file path to zipped NPZ :obj:`Quilt` store, return a :obj:`Quilt` instance.
 
         {args}
         '''
@@ -270,6 +284,31 @@ class Quilt(ContainerBase, StoreClientMixin):
                 max_persist=max_persist,
                 )
 
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_zip_npy(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to zipped NPY :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreZipNPY(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
+
     @classmethod
     @doc_inject(selector='quilt_constructor')
     def from_zip_parquet(cls,
@@ -413,7 +452,7 @@ class Quilt(ContainerBase, StoreClientMixin):
                 )
 
     #---------------------------------------------------------------------------
-    @doc_inject(selector='quilt_init')
+    # @doc_inject(selector='quilt_init')
     def __init__(self,
             bus: tp.Union[Bus, Yarn],
             *,
@@ -445,7 +484,7 @@ class Quilt(ContainerBase, StoreClientMixin):
     def _error_update_axis_labels(axis: int) -> ErrorInitQuilt:
         axis_label = 'index' if axis == 0 else 'column'
         axis_labels = 'indices' if axis == 0 else 'columns'
-        err_msg = f"Duplicate {axis_label} labels across frames. Either ensure all {axis_labels} are unique for all frames, or set retain_labels=True to obtain an IndexHierarchy"
+        err_msg = f'Duplicate {axis_label} labels across frames. Either ensure all {axis_labels} are unique for all frames, or set retain_labels=True to obtain an IndexHierarchy'
         return ErrorInitQuilt(err_msg)
 
     def _update_axis_labels(self) -> None:
@@ -484,11 +523,11 @@ class Quilt(ContainerBase, StoreClientMixin):
     #---------------------------------------------------------------------------
     # name interface
 
-    @property #type: ignore
+    @property
     @doc_inject()
     def name(self) -> NameType:
         '''{}'''
-        return self._bus.name #type: ignore
+        return self._bus.name
 
     def rename(self, name: NameType) -> 'Quilt':
         '''
@@ -521,24 +560,23 @@ class Quilt(ContainerBase, StoreClientMixin):
         if self._assign_axis:
             self._update_axis_labels()
 
-        drop_column_dtype = False
+        config = config or DisplayActive.get()
 
+        drop_column_dtype = False
         if self._axis == 0:
             if not self._retain_labels:
-                index = self.index.rename("Concatenated")
+                index = self.index.rename('Concatenated')
             else:
-                index = self._bus.index.rename("Frames")
-            columns = self.columns.rename("Aligned")
+                index = self._bus.index.rename('Frames')
+            columns = self.columns.rename('Aligned')
         else:
-            index = self.index.rename("Aligned")
+            index = self.index.rename('Aligned')
             if not self._retain_labels:
-                columns = self.columns.rename("Concatenated")
+                columns = self.columns.rename('Concatenated')
             else:
-                columns = self._bus.index.rename("Frames")
+                columns = self._bus.index.rename('Frames')
                 drop_column_dtype = True
 
-        config = config or DisplayConfig()
-
         def placeholder_gen() -> tp.Iterator[tp.Iterable[tp.Any]]:
             assert config is not None
             yield from repeat(tuple(repeat(config.cell_placeholder, times=len(index))), times=len(columns))
@@ -571,9 +609,9 @@ class Quilt(ContainerBase, StoreClientMixin):
     #---------------------------------------------------------------------------
     # accessors
 
-    @property #type: ignore
+    @property
     @doc_inject(selector='values_2d', class_name='Quilt')
-    def values(self) -> np.ndarray:
+    def values(self) -> NDArrayAny:
         '''
         {}
         '''
@@ -597,6 +635,12 @@ class Quilt(ContainerBase, StoreClientMixin):
             self._update_axis_labels()
         return self._columns
 
+    @property
+    def bus(self) -> tp.Union[Bus, Yarn]:
+        '''The ``Bus`` instance assigned to this ``Quilt``.
+        '''
+        return self._bus
+
     #---------------------------------------------------------------------------
 
     @property
@@ -711,7 +755,7 @@ class Quilt(ContainerBase, StoreClientMixin):
     #---------------------------------------------------------------------------
     # axis iterators
 
-    def _axis_array(self, axis: int) -> tp.Iterator[np.ndarray]:
+    def _axis_array(self, axis: int) -> tp.Iterator[NDArrayAny]:
         '''Generator of arrays across an axis
 
         Args:
@@ -740,7 +784,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         else:
             raise AxisInvalid(f'no support for axis {axis}')
 
-    def _axis_array_items(self, axis: int) -> tp.Iterator[tp.Tuple[tp.Hashable, np.ndarray]]:
+    def _axis_array_items(self, axis: int) -> tp.Iterator[tp.Tuple[tp.Hashable, NDArrayAny]]:
         keys = self._index if axis == 1 else self._columns
         yield from zip(keys, self._axis_array(axis))
 
@@ -771,7 +815,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         assert constructor is not None
 
         for axis_values in self._axis_array(axis):
-            yield constructor(axis_values)
+            yield constructor(axis_values) # type: ignore
 
     def _axis_tuple_items(self, *,
             axis: int,
@@ -788,7 +832,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         for label, axis_values in self._axis_array_items(axis):
             yield Series(axis_values, index=index, name=label, own_index=True)
 
-    def _axis_series_items(self, axis: int) -> tp.Iterator[tp.Tuple[tp.Hashable, np.ndarray]]:
+    def _axis_series_items(self, axis: int) -> tp.Iterator[tp.Tuple[tp.Hashable, NDArrayAny]]:
         keys = self._index if axis == 1 else self._columns
         yield from zip(keys, self._axis_series(axis=axis))
 
@@ -851,13 +895,13 @@ class Quilt(ContainerBase, StoreClientMixin):
     def _extract_array(self,
             row_key: GetItemKeyType = None,
             column_key: GetItemKeyType = None,
-            ) -> np.ndarray:
+            ) -> NDArrayAny:
         '''
         Extract a consolidated array based on iloc selection.
         '''
         assert self._axis_hierarchy is not None #mypy
 
-        extractor = get_extractor(
+        extractor: tp.Callable[..., NDArrayAny] = get_extractor(
                 self._deepcopy_from_bus,
                 is_array=True,
                 memo_active=False,
@@ -877,7 +921,7 @@ class Quilt(ContainerBase, StoreClientMixin):
                     axis=self._axis,
                     )
 
-        parts: tp.List[np.ndarray] = []
+        parts: tp.List[NDArrayAny] = []
         bus_keys: tp.Iterable[tp.Hashable]
 
         if self._axis == 0:
@@ -898,9 +942,9 @@ class Quilt(ContainerBase, StoreClientMixin):
         if isinstance(axis_map_sub, tuple): # type: ignore
             bus_keys = (axis_map_sub[0],) #type: ignore
         else:
-            bus_keys = axis_map_sub._levels.index
+            bus_keys = axis_map_sub.unique(depth_level=0, order_by_occurrence=True)
 
-        for key_count, key in enumerate(bus_keys):
+        for key in bus_keys:
             sel_component = sel[self._axis_hierarchy._loc_to_iloc(HLoc[key])]
 
             if self._axis == 0:
@@ -917,6 +961,10 @@ class Quilt(ContainerBase, StoreClientMixin):
 
             parts.append(component)
 
+        if sel_reduces and opposite_reduces: # we have an element
+            return parts.pop()
+
+        # we call extractor() when we might be referencing data to control if we give a slice or a deepcopy
         if len(parts) == 1:
             return extractor(parts.pop())
 
@@ -957,7 +1005,7 @@ class Quilt(ContainerBase, StoreClientMixin):
             else:
                 frames = (extractor(f) for _, f in self._bus.items())
 
-            return Frame.from_concat( #type: ignore
+            return Frame.from_concat(
                     frames,
                     axis=self._axis,
                     )
@@ -983,8 +1031,9 @@ class Quilt(ContainerBase, StoreClientMixin):
             frame_labels = (axis_map_sub[0],) #type: ignore
         else:
             # get the outer level, or just the unique frame labels needed
-            frame_labels = axis_map_sub._levels.index
+            frame_labels = axis_map_sub.unique(depth_level=0, order_by_occurrence=True)
 
+        component: tp.Any
         for key_count, key in enumerate(frame_labels):
             # get Boolean segment for this Frame
             sel_component = sel[self._axis_hierarchy._loc_to_iloc(HLoc[key])]
@@ -1021,7 +1070,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         # NOTE: Series/Frame from_concate will attempt to re-use ndarrays, and thus using extractor above is appropriate
         if component_is_series:
             return Series.from_concat(parts)
-        return Frame.from_concat(parts, axis=self._axis) #type: ignore
+        return Frame.from_concat(parts, axis=self._axis)
 
     #---------------------------------------------------------------------------
     @doc_inject(selector='sample')
@@ -1108,12 +1157,12 @@ class Quilt(ContainerBase, StoreClientMixin):
     # interfaces
 
     @property
-    def loc(self) -> InterfaceGetItem['Frame']:
-        return InterfaceGetItem(self._extract_loc) #type: ignore
+    def loc(self) -> InterfaceGetItemCompound[Frame | Series]:
+        return InterfaceGetItemCompound(self._extract_loc)
 
     @property
-    def iloc(self) -> InterfaceGetItem['Frame']:
-        return InterfaceGetItem(self._extract_iloc) #type: ignore
+    def iloc(self) -> InterfaceGetItemCompound[Frame | Series]:
+        return InterfaceGetItemCompound(self._extract_iloc)
 
     #---------------------------------------------------------------------------
     # iterators
@@ -1211,7 +1260,7 @@ class Quilt(ContainerBase, StoreClientMixin):
 
     #---------------------------------------------------------------------------
 
-    @property #type: ignore
+    @property
     @doc_inject(selector='window')
     def iter_window(self) -> IterNodeWindow['Quilt']:
         '''
@@ -1231,7 +1280,7 @@ class Quilt(ContainerBase, StoreClientMixin):
                 apply_type=IterNodeApplyType.SERIES_ITEMS,
                 )
 
-    @property #type: ignore
+    @property
     @doc_inject(selector='window')
     def iter_window_items(self) -> IterNodeWindow['Quilt']:
         '''
@@ -1251,7 +1300,7 @@ class Quilt(ContainerBase, StoreClientMixin):
                 apply_type=IterNodeApplyType.SERIES_ITEMS,
                 )
 
-    @property #type: ignore
+    @property
     @doc_inject(selector='window')
     def iter_window_array(self) -> IterNodeWindow['Quilt']:
         '''
@@ -1271,7 +1320,7 @@ class Quilt(ContainerBase, StoreClientMixin):
                 apply_type=IterNodeApplyType.SERIES_ITEMS,
                 )
 
-    @property #type: ignore
+    @property
     @doc_inject(selector='window')
     def iter_window_array_items(self) -> IterNodeWindow['Quilt']:
         '''
@@ -1300,7 +1349,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         Args:
             {count}
         '''
-        return self.iloc[:count]
+        return self.iloc[:count] # type: ignore
 
     @doc_inject(selector='tail', class_name='Quilt')
     def tail(self, count: int = 5) -> 'Frame':
@@ -1309,7 +1358,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         Args:
             {count}
         '''
-        return self.iloc[-count:]
+        return self.iloc[-count:] # type: ignore
 
     #---------------------------------------------------------------------------
     @doc_inject()
@@ -1390,3 +1439,32 @@ class Quilt(ContainerBase, StoreClientMixin):
         if self._assign_axis:
             self._update_axis_labels()
         return self._extract(NULL_SLICE, NULL_SLICE) #type: ignore
+
+    def _to_signature_bytes(self,
+            include_name: bool = True,
+            include_class: bool = True,
+            encoding: str = 'utf-8',
+            ) -> bytes:
+
+        if self._assign_axis:
+            self._update_axis_labels()
+
+        return b''.join(chain(
+                iter_component_signature_bytes(self,
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),
+                (self._axis_hierarchy._to_signature_bytes( #type: ignore
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),
+                self._axis_opposite._to_signature_bytes( #type: ignore
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),
+                self._bus._to_signature_bytes(
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),)
+                ))
+
