@@ -1,21 +1,21 @@
+from __future__ import annotations
 
 import typing as tp
-
+import zipfile
 
 from static_frame.core.doc_str import doc_inject
-from static_frame.core.store import StoreConfigMapInitializer
+from static_frame.core.store_config import StoreConfigMap
+from static_frame.core.store_config import StoreConfigMapInitializer
 from static_frame.core.store_hdf5 import StoreHDF5
 from static_frame.core.store_sqlite import StoreSQLite
 from static_frame.core.store_xlsx import StoreXLSX
 from static_frame.core.store_zip import StoreZipCSV
+from static_frame.core.store_zip import StoreZipNPY
+from static_frame.core.store_zip import StoreZipNPZ
 from static_frame.core.store_zip import StoreZipParquet
 from static_frame.core.store_zip import StoreZipPickle
 from static_frame.core.store_zip import StoreZipTSV
-from static_frame.core.store_zip import StoreZipNPZ
 from static_frame.core.util import PathSpecifier
-from static_frame.core.store import StoreConfigMap
-
-
 
 # NOTE: wanted this to inherit from tp.Generic[T], such that values returned from constructors would be known, but this breaks in 3.6 with: metaclass conflict: the metaclass of a derived class must be a (non-strict) subclass of the metaclasses of all its bases
 
@@ -40,7 +40,7 @@ class StoreClientMixin:
         if hasattr(self, '_bus'): # this is Quilt
             return self._bus._config # type: ignore
         # Yarn does not have a _config attr
-        return getattr(self, '_config', None) #type: ignore
+        return getattr(self, '_config', None)
 
     #---------------------------------------------------------------------------
     # exporters
@@ -50,6 +50,7 @@ class StoreClientMixin:
             fp: PathSpecifier,
             *,
             config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as a zipped archive of TSV files.
@@ -58,13 +59,14 @@ class StoreClientMixin:
         '''
         store = StoreZipTSV(fp)
         config = self._filter_config(config)
-        store.write(self._items_store(), config=config)
+        store.write(self._items_store(), config=config, compression=compression)
 
     @doc_inject(selector='store_client_exporter')
     def to_zip_csv(self,
             fp: PathSpecifier,
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as a zipped archive of CSV files.
@@ -73,13 +75,14 @@ class StoreClientMixin:
         '''
         store = StoreZipCSV(fp)
         config = self._filter_config(config)
-        store.write(self._items_store(), config=config)
+        store.write(self._items_store(), config=config, compression=compression)
 
     @doc_inject(selector='store_client_exporter')
     def to_zip_pickle(self,
             fp: PathSpecifier,
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as a zipped archive of pickles.
@@ -88,13 +91,14 @@ class StoreClientMixin:
         '''
         store = StoreZipPickle(fp)
         config = self._filter_config(config)
-        store.write(self._items_store(), config=config)
+        store.write(self._items_store(), config=config, compression=compression)
 
     @doc_inject(selector='store_client_exporter')
     def to_zip_npz(self,
             fp: PathSpecifier,
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as a zipped archive of NPZ files.
@@ -103,13 +107,30 @@ class StoreClientMixin:
         '''
         store = StoreZipNPZ(fp)
         config = self._filter_config(config)
-        store.write(self._items_store(), config=config)
+        store.write(self._items_store(), config=config, compression=compression)
+
+    @doc_inject(selector='store_client_exporter')
+    def to_zip_npy(self,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
+            ) -> None:
+        '''
+        Write the complete :obj:`Bus` as a zipped archive of NPY files.
+
+        {args}
+        '''
+        store = StoreZipNPY(fp)
+        config = self._filter_config(config)
+        store.write(self._items_store(), config=config, compression=compression)
 
     @doc_inject(selector='store_client_exporter')
     def to_zip_parquet(self,
             fp: PathSpecifier,
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as a zipped archive of parquet files.
@@ -118,13 +139,13 @@ class StoreClientMixin:
         '''
         store = StoreZipParquet(fp)
         config = self._filter_config(config)
-        store.write(self._items_store(), config=config)
+        store.write(self._items_store(), config=config, compression=compression)
 
     @doc_inject(selector='store_client_exporter')
     def to_xlsx(self,
             fp: PathSpecifier,
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as a XLSX workbook.
@@ -139,7 +160,7 @@ class StoreClientMixin:
     def to_sqlite(self,
             fp: PathSpecifier,
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as an SQLite database file.
@@ -154,7 +175,7 @@ class StoreClientMixin:
     def to_hdf5(self,
             fp: PathSpecifier,
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
             ) -> None:
         '''
         Write the complete :obj:`Bus` as an HDF5 table.
