@@ -1,28 +1,31 @@
+from __future__ import annotations
 
 import typing as tp
 
 import numpy as np
 
 from static_frame.core.interface_meta import InterfaceMeta
+from static_frame.core.util import COMPLEX_TYPES
+from static_frame.core.util import DT64_MONTH
+from static_frame.core.util import DT64_YEAR
 from static_frame.core.util import DTYPE_BOOL
 from static_frame.core.util import DTYPE_COMPLEX_KIND
-from static_frame.core.util import DTYPE_INT_KINDS
+from static_frame.core.util import DTYPE_FLOAT_KIND
 from static_frame.core.util import DTYPE_INEXACT_KINDS
+from static_frame.core.util import DTYPE_INT_KINDS
 from static_frame.core.util import DTYPE_NAT_KINDS
 from static_frame.core.util import DTYPE_OBJECT
+from static_frame.core.util import DTYPE_OBJECT_KIND
 from static_frame.core.util import DTYPE_STR_KINDS
 from static_frame.core.util import EMPTY_SET
 from static_frame.core.util import FLOAT_TYPES
-from static_frame.core.util import COMPLEX_TYPES
-from static_frame.core.util import EMPTY_TUPLE
-from static_frame.core.util import DTYPE_OBJECT_KIND
-from static_frame.core.util import DTYPE_FLOAT_KIND
 from static_frame.core.util import NAT
 from static_frame.core.util import NAT_STR
-from static_frame.core.util import DT64_YEAR
-from static_frame.core.util import DT64_MONTH
+from static_frame.core.util import frozenset_filter
 
-# from static_frame.core.util import InexactTypes
+if tp.TYPE_CHECKING:
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    # DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 class StoreFilter(metaclass=InterfaceMeta):
     '''
@@ -88,11 +91,11 @@ class StoreFilter(metaclass=InterfaceMeta):
             from_posinf: tp.Optional[str] = 'inf',
             from_neginf: tp.Optional[str] = '-inf',
             # str to type
-            to_nan: tp.FrozenSet[str] = frozenset(('', 'nan', 'NaN', 'NAN', 'NULL', '#N/A')),
-            to_nat: tp.FrozenSet[str] = frozenset(EMPTY_TUPLE), # do not assume there are NaTs.
-            to_none: tp.FrozenSet[str] = frozenset(('None',)),
-            to_posinf: tp.FrozenSet[str] = frozenset(('inf',)),
-            to_neginf: tp.FrozenSet[str] = frozenset(('-inf',)),
+            to_nan: tp.Collection[str] = frozenset(('', 'nan', 'NaN', 'NAN', 'NULL', '#N/A')),
+            to_nat: tp.Collection[str] = frozenset(()), # do not assume there are NaTs.
+            to_none: tp.Collection[str] = frozenset(('None',)),
+            to_posinf: tp.Collection[str] = frozenset(('inf',)),
+            to_neginf: tp.Collection[str] = frozenset(('-inf',)),
             # from float to str
             value_format_float_positional: tp.Optional[str] = None,
             value_format_float_scientific: tp.Optional[str] = None,
@@ -106,11 +109,11 @@ class StoreFilter(metaclass=InterfaceMeta):
         self.from_posinf = from_posinf
         self.from_neginf = from_neginf
 
-        self.to_nan = to_nan
-        self.to_nat = to_nat
-        self.to_none = to_none
-        self.to_posinf = to_posinf
-        self.to_neginf = to_neginf
+        self.to_nan = frozenset_filter(to_nan)
+        self.to_nat = frozenset_filter(to_nat)
+        self.to_none = frozenset_filter(to_none)
+        self.to_posinf = frozenset_filter(to_posinf)
+        self.to_neginf = frozenset_filter(to_neginf)
 
         self.value_format_float_positional = value_format_float_positional
         self.value_format_float_scientific = value_format_float_scientific
@@ -136,7 +139,7 @@ class StoreFilter(metaclass=InterfaceMeta):
         self._EQUAL_FUNC_TO_FROM = (
                 # NOTE: this using the same heuristic as util.isna_array, which may not be the best choice for non-standard objects
                 (lambda x: np.not_equal(x, x), self.from_nan),
-                (lambda x: np.equal(x, None), self.from_none),
+                (lambda x: np.equal(x, None), self.from_none), # type: ignore
                 (lambda x: np.equal(x, np.inf), self.from_posinf),
                 (lambda x: np.equal(x, -np.inf), self.from_neginf)
                 )
@@ -161,8 +164,7 @@ class StoreFilter(metaclass=InterfaceMeta):
                 )
 
     # --------------------------------------------------------------------------
-    # converting from types (in memory) to datastore
-
+    # converting from types (in memory) to data store
 
     def _format_inexact_element(self,
             value: tp.Any,
@@ -201,11 +203,10 @@ class StoreFilter(metaclass=InterfaceMeta):
             return self.value_format_complex_positional.format(value)
         return value
 
-
     def _format_inexact_array(self,
-            array: np.ndarray,
-            array_object: tp.Optional[np.ndarray],
-            ) -> np.ndarray:
+            array: NDArrayAny,
+            array_object: tp.Optional[NDArrayAny],
+            ) -> NDArrayAny:
         '''
         Args:
             array_object: if we have already created an object array, use it as destination, mutating values in-place. ``array`` and ``array_object`` can be the same array.
@@ -225,8 +226,8 @@ class StoreFilter(metaclass=InterfaceMeta):
         return array_object
 
     def from_type_filter_array(self,
-            array: np.ndarray
-            ) -> np.ndarray:
+            array: NDArrayAny
+            ) -> NDArrayAny:
         '''Given an array, replace types with strings
         '''
         kind = array.dtype.kind
@@ -265,7 +266,7 @@ class StoreFilter(metaclass=InterfaceMeta):
         if kind in DTYPE_NAT_KINDS:
             post = None
             if array.dtype == DT64_YEAR or array.dtype == DT64_MONTH:
-                post = array.astype(str) # nat will go to "NaT"
+                post = array.astype(str) # nat will go to 'NaT'
 
             if post is not None and post.dtype.kind in DTYPE_STR_KINDS:
                 is_nat = post == NAT_STR
@@ -320,8 +321,8 @@ class StoreFilter(metaclass=InterfaceMeta):
     # converting from strings (in data store) to types
 
     def to_type_filter_array(self,
-            array: np.ndarray
-            ) -> np.ndarray:
+            array: NDArrayAny
+            ) -> NDArrayAny:
         '''Given an array, replace strings with types.
         '''
         kind = array.dtype.kind
@@ -349,7 +350,6 @@ class StoreFilter(metaclass=InterfaceMeta):
 
         return array
 
-
     def to_type_filter_element(self,
             value: tp.Any
             ) -> tp.Any:
@@ -362,10 +362,9 @@ class StoreFilter(metaclass=InterfaceMeta):
                     return value_replace
         return value
 
-    def to_type_filter_iterable(self, iterable: tp.Iterable[tp.Any]) -> tp.Iterator[tp.Any]:
-        for value in iterable:
-            yield self.to_type_filter_element(value)
-
+    # def to_type_filter_iterable(self, iterable: tp.Iterable[tp.Any]) -> tp.Iterator[tp.Any]:
+    #     for value in iterable:
+    #         yield self.to_type_filter_element(value)
 
 
 STORE_FILTER_DEFAULT = StoreFilter()
