@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 import typing as tp
-import warnings
 
 import numpy as np
 
@@ -8,10 +9,14 @@ from static_frame.core.frame import Frame
 from static_frame.core.store import Store
 from static_frame.core.store import store_coherent_non_write
 from static_frame.core.store import store_coherent_write
-from static_frame.core.store import StoreConfigMap
-from static_frame.core.store import StoreConfigMapInitializer
+from static_frame.core.store_config import StoreConfigMap
+from static_frame.core.store_config import StoreConfigMapInitializer
 from static_frame.core.type_blocks import TypeBlocks
 from static_frame.core.util import DTYPE_STR_KINDS
+from static_frame.core.util import WarningsSilent
+
+if tp.TYPE_CHECKING:
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 
 class StoreHDF5(Store):
@@ -28,11 +33,13 @@ class StoreHDF5(Store):
 
         config_map = StoreConfigMap.from_initializer(config)
 
-        import tables
+        with WarningsSilent():
+            import tables
 
-        with tables.open_file(self._fp, mode='w') as file, warnings.catch_warnings():
+            # silence: DeprecationWarning: `np.typeDict` is a deprecated alias for `np.sctypeDict`.
+
+        with tables.open_file(self._fp, mode='w') as file, WarningsSilent():
             # silence NaturalNameWarning: object name is not a valid Python identifier:
-            warnings.simplefilter("ignore")
 
             for label, frame in items:
                 c = config_map[label]
@@ -94,7 +101,7 @@ class StoreHDF5(Store):
                 table = file.get_node(f'/{label_encoded}')
                 colnames = table.cols._v_colnames
 
-                def blocks() -> tp.Iterator[np.ndarray]:
+                def blocks() -> tp.Iterator[NDArrayAny]:
                     for col_idx, colname in enumerate(colnames):
                         # can also do: table.read(field=colname)
                         array = table.col(colname)
