@@ -1,6 +1,7 @@
+from __future__ import annotations
 
-import typing as tp
 import datetime
+import typing as tp
 from functools import partial
 
 import numpy as np
@@ -14,12 +15,11 @@ from static_frame.core.index_hierarchy import IndexHierarchy
 from static_frame.core.store import Store
 from static_frame.core.store import store_coherent_non_write
 from static_frame.core.store import store_coherent_write
-from static_frame.core.store import StoreConfig
-from static_frame.core.store import StoreConfigMap
-from static_frame.core.store import StoreConfigMapInitializer
+from static_frame.core.store_config import StoreConfig
+from static_frame.core.store_config import StoreConfigMap
+from static_frame.core.store_config import StoreConfigMapInitializer
 from static_frame.core.store_filter import STORE_FILTER_DEFAULT
 from static_frame.core.store_filter import StoreFilter
-from static_frame.core.util import AnyCallable
 from static_frame.core.util import BOOL_TYPES
 from static_frame.core.util import COMPLEX_TYPES
 from static_frame.core.util import DTYPE_BOOL
@@ -28,14 +28,16 @@ from static_frame.core.util import DTYPE_INT_KINDS
 from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import DTYPE_STR_KINDS
 from static_frame.core.util import NUMERIC_TYPES
-from static_frame.core.util import array1d_to_last_contiguous_to_edge
 from static_frame.core.util import STORE_LABEL_DEFAULT
+from static_frame.core.util import AnyCallable
+from static_frame.core.util import array1d_to_last_contiguous_to_edge
 
 if tp.TYPE_CHECKING:
-    from xlsxwriter.worksheet import Worksheet  # pylint: disable=W0611 #pragma: no cover
+    from xlsxwriter.format import Format  # pylint: disable=W0611 #pragma: no cover
     from xlsxwriter.workbook import Workbook  # pylint: disable=W0611 #pragma: no cover
-    from xlsxwriter.format import Format  # pylint: disable=W0611 #pragma: no cover
-    from xlsxwriter.format import Format  # pylint: disable=W0611 #pragma: no cover
+    from xlsxwriter.worksheet import Worksheet  # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
+
     # from openpyxl.cell.read_only import ReadOnlyCell # pylint: disable=W0611 #pragma: no cover
     # from openpyxl.cell.read_only import EmptyCell # pylint: disable=W0611 #pragma: no cover
 
@@ -81,14 +83,14 @@ class StoreXLSX(Store):
 
     @staticmethod
     def _dtype_to_writer_attr(
-            dtype: np.dtype,
+            dtype: DtypeAny,
             ) -> tp.Tuple[str, bool]:
         '''
         Return a pair of writer function, Boolean, where Boolean denotes if replacements need be applied.
         '''
         kind = dtype.kind
 
-        # NOTE: xlsxwriter cannot handle datetime64, raises TypeError("Unknown or unsupported datetime type")
+        # NOTE: xlsxwriter cannot handle datetime64, raises TypeError('Unknown or unsupported datetime type')
         # if kind in DTYPE_NAT_KINDS and dtype != DT64_MONTH and dtype != DT64_YEAR:
         #     return 'write_datetime', True
 
@@ -104,7 +106,7 @@ class StoreXLSX(Store):
 
     @classmethod
     def _get_writer(cls,
-            dtype: np.dtype,
+            dtype: DtypeAny,
             ws: 'Worksheet'
             ) -> AnyCallable: # find better type
         '''
@@ -130,7 +132,7 @@ class StoreXLSX(Store):
             if isinstance(value, COMPLEX_TYPES):
                 return ws.write_string(row, col, str(value), format_cell)
 
-            if writer_attr == 'write': #type: ignore [unreachable]
+            if writer_attr == 'write':
                 # determine type for each value
                 if isinstance(value, BOOL_TYPES):
                     return ws.write_boolean(row, col, value, format_cell)
@@ -268,7 +270,8 @@ class StoreXLSX(Store):
                 col = index_depth_effective # start after index
                 for label, width in frame._columns.label_widths_at_depth(depth):
                     # TODO: use store_filter
-                    ws.merge_range(row, col, row, col + width - 1, label, format_columns)
+                    if width > 1:
+                        ws.merge_range(row, col, row, col + width - 1, label, format_columns)
                     col += width
 
         if include_index and merge_hierarchical_labels and index_depth > 1:
@@ -277,7 +280,8 @@ class StoreXLSX(Store):
                 col = depth
                 for label, width in frame._index.label_widths_at_depth(depth):
                     # TODO: use store_filter
-                    ws.merge_range(row, col, row + width - 1, col, label, format_index)
+                    if width > 1:
+                        ws.merge_range(row, col, row + width - 1, col, label, format_index)
                     row += width
 
     @store_coherent_write
@@ -501,10 +505,11 @@ class StoreXLSX(Store):
             if index_depth <= 1:
                 index_default_constructor = partial(Index, name=index_name)
             else: # > 1
-                index_default_constructor = partial(IndexHierarchy.from_labels,
+                index_default_constructor = partial(IndexHierarchy.from_labels, # type: ignore
                         name=index_name,
                         continuation_token=None, # NOTE: needed
                         )
+
             index, own_index = index_from_optional_constructors(
                     index_values,
                     depth=index_depth,
@@ -528,7 +533,7 @@ class StoreXLSX(Store):
                         )
             elif columns_depth > 1:
                 columns_default_constructor = partial(
-                        container_type._COLUMNS_HIERARCHY_CONSTRUCTOR.from_labels,
+                        container_type._COLUMNS_HIERARCHY_CONSTRUCTOR.from_labels, # type: ignore
                         name=columns_name,
                         continuation_token=None, # NOTE: needed, not the default
                         )
