@@ -1,28 +1,28 @@
+from __future__ import annotations
+
+import os
+import pickle
 import typing as tp
 import zipfile
-import pickle
-from io import StringIO
 from io import BytesIO
-from concurrent.futures import ProcessPoolExecutor
-# import multiprocessing as mp
-# mp_context = mp.get_context('spawn')
+from io import StringIO
 
+from static_frame.core.archive_npy import ArchiveFrameConverter
+from static_frame.core.archive_npy import ArchiveZipWrapper
+from static_frame.core.container_util import container_to_exporter_attr
+from static_frame.core.exception import ErrorNPYEncode
+from static_frame.core.exception import StoreLabelNonUnique
 from static_frame.core.frame import Frame
 from static_frame.core.store import Store
 from static_frame.core.store import store_coherent_non_write
 from static_frame.core.store import store_coherent_write
-from static_frame.core.store import StoreConfigMap
-from static_frame.core.store import StoreConfig
-from static_frame.core.store import StoreConfigHE
-from static_frame.core.store import StoreConfigMapInitializer
-from static_frame.core.util import AnyCallable
+from static_frame.core.store_config import StoreConfig
+from static_frame.core.store_config import StoreConfigHE
+from static_frame.core.store_config import StoreConfigMap
+from static_frame.core.store_config import StoreConfigMapInitializer
 from static_frame.core.util import NOT_IN_CACHE_SENTINEL
-from static_frame.core.container_util import container_to_exporter_attr
-
-
-# class FrameExporter(tp.Protocol):
-#     def __call__(self, frame: Frame, *args: tp.Any, **kwargs: tp.Any) -> tp.Any:
-#         raise NotImplementedError
+from static_frame.core.util import AnyCallable
+from static_frame.core.util import get_concurrent_executor
 
 FrameExporter = AnyCallable # Protocol not supported yet...
 FrameConstructor = tp.Callable[[tp.Any], Frame]
@@ -81,10 +81,10 @@ class _StoreZip(Store):
 
     @staticmethod
     def _set_container_type(frame: Frame, container_type: tp.Type[Frame]) -> Frame:
-        """
+        '''
         Helper method to coerce a frame to the expected type, or return it as is
         if the type is already correct
-        """
+        '''
         if frame.__class__ is not container_type:
             return frame._to_frame(container_type)
         return frame
@@ -112,10 +112,10 @@ class _StoreZip(Store):
             constructor: FrameConstructor,
             container_type: tp.Type[Frame],
             ) -> tp.Iterator[Frame]:
-        """
+        '''
         Simplified logic path for reading many frames in a single thread, using
         the weak_cache when possible.
-        """
+        '''
         with zipfile.ZipFile(self._fp) as zf:
             for label in labels:
                 # Since the value can be deallocated between lookup & extraction,
@@ -123,7 +123,7 @@ class _StoreZip(Store):
                 # don't have a race condition
                 cache_lookup = self._weak_cache.get(label, NOT_IN_CACHE_SENTINEL)
                 if cache_lookup is not NOT_IN_CACHE_SENTINEL:
-                    yield self._set_container_type(cache_lookup, container_type)
+                    yield self._set_container_type(cache_lookup, container_type) # type: ignore
                     continue
 
                 c: StoreConfig = config_map[label]
@@ -170,7 +170,7 @@ class _StoreZip(Store):
                 count_labels += 1
                 cache_lookup = self._weak_cache.get(label, NOT_IN_CACHE_SENTINEL)
                 if cache_lookup is not NOT_IN_CACHE_SENTINEL:
-                    results[label] = self._set_container_type(cache_lookup, container_type)
+                    results[label] = self._set_container_type(cache_lookup, container_type) # type: ignore
                     count_cache += 1
                 else:
                     results[label] = None
@@ -201,7 +201,6 @@ class _StoreZip(Store):
                         continue
 
                     c: StoreConfig = config_map[label]
-
                     label_encoded: str = config_map.default.label_encode(label)
                     src: bytes = zf.read(label_encoded + self._EXT_CONTAINED)
 
@@ -210,11 +209,16 @@ class _StoreZip(Store):
                             name=label,
                             config=c.to_store_config_he(),
                             constructor=constructor,
-                    )
+                            )
 
         chunksize = config_map.default.read_chunksize
+        pool_executor = get_concurrent_executor(
+                use_threads=False,
+                max_workers=config_map.default.read_max_workers,
+                mp_context=config_map.default.mp_context,
+                )
 
-        with ProcessPoolExecutor(max_workers=config_map.default.read_max_workers) as executor:
+        with pool_executor() as executor:
             frame_gen = executor.map(self._payload_to_frame, gen(), chunksize=chunksize)
 
             for label, cached_frame in results_items():
@@ -222,7 +226,6 @@ class _StoreZip(Store):
                     yield cached_frame
                 else:
                     frame = next(frame_gen)
-
                     # Newly read frame, add it to our weak_cache
                     self._weak_cache[label] = frame
                     yield frame
@@ -237,7 +240,8 @@ class _StoreZip(Store):
     def write(self,
             items: tp.Iterable[tp.Tuple[tp.Hashable, Frame]],
             *,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
             ) -> None:
         config_map = StoreConfigMap.from_initializer(config)
         multiprocess = (config_map.default.write_max_workers is not None and
@@ -253,19 +257,40 @@ class _StoreZip(Store):
                         )
 
         if multiprocess:
+            pool_executor = get_concurrent_executor(
+                    use_threads=False,
+                    max_workers=config_map.default.write_max_workers,
+                    mp_context=config_map.default.mp_context,
+                    )
             def label_and_bytes() -> tp.Iterator[LabelAndBytes]:
-                with ProcessPoolExecutor(max_workers=config_map.default.write_max_workers) as executor:
+                with pool_executor() as executor:
                     yield from executor.map(self._payload_to_bytes,
                             gen(),
                             chunksize=config_map.default.write_chunksize)
         else:
             label_and_bytes = lambda: (self._payload_to_bytes(x) for x in gen())
 
-        with zipfile.ZipFile(self._fp, 'w', zipfile.ZIP_DEFLATED) as zf:
-            for label, frame_bytes in label_and_bytes():
-                label_encoded = config_map.default.label_encode(label)
-                # this will write it without a container
-                zf.writestr(label_encoded + self._EXT_CONTAINED, frame_bytes)
+        labels_encoded = set() # track uniqueness post encoding
+
+        try:
+            with zipfile.ZipFile(self._fp,
+                    mode='w',
+                    compression=compression,
+                    allowZip64=True,
+                    ) as zf:
+                for label, frame_bytes in label_and_bytes():
+                    label_encoded = config_map.default.label_encode(label)
+
+                    if label_encoded in labels_encoded:
+                        raise StoreLabelNonUnique(label_encoded)
+                    labels_encoded.add(label_encoded)
+
+                    zf.writestr(label_encoded + self._EXT_CONTAINED, frame_bytes)
+        except ErrorNPYEncode:
+            # NOTE: catch NPY failures and remove self._fp to not leave a malformed zip
+            if os.path.exists(self._fp):
+                os.remove(self._fp)
+            raise
 
 
 class _StoreZipDelimited(_StoreZip):
@@ -338,7 +363,7 @@ class StoreZipPickle(_StoreZip):
     '''A zip of pickles, permitting incremental loading of Frames.
     '''
     _EXT_CONTAINED = '.pickle'
-    _EXPORTER = pickle.dumps
+    _EXPORTER = pickle.dumps # NOTE: might be able to use to_pickle
 
     @classmethod
     def _container_type_to_constructor(cls, container_type: tp.Type[Frame]) -> FrameConstructor:
@@ -456,3 +481,95 @@ class StoreZipParquet(_StoreZip):
                 include_columns_name=c.include_columns_name,
                 )
         return payload.name, dst.getvalue()
+
+#-------------------------------------------------------------------------------
+class StoreZipNPY(Store):
+    '''A zip of NPY files. This does not presently support multi-processing.
+    '''
+    _EXT: tp.FrozenSet[str] = frozenset(('.zip',))
+    _DELIMITER = '/'
+
+    @store_coherent_write
+    def write(self,
+            items: tp.Iterable[tp.Tuple[tp.Hashable, Frame]],
+            *,
+            config: StoreConfigMapInitializer = None,
+            compression: int = zipfile.ZIP_DEFLATED,
+            ) -> None:
+        config_map = StoreConfigMap.from_initializer(config)
+
+        try:
+            with zipfile.ZipFile(self._fp,
+                    mode='w',
+                    compression=compression,
+                    allowZip64=True,
+                    ) as zf:
+                archive = ArchiveZipWrapper(zf,
+                        writeable=True,
+                        memory_map=False,
+                        delimiter=self._DELIMITER,
+                        )
+                for label, frame in items:
+                    c: StoreConfig = config_map[label]
+                    archive.prefix = config_map.default.label_encode(label) # mutate
+                    ArchiveFrameConverter.frame_encode(
+                            archive=archive,
+                            frame=frame,
+                            include_index=c.include_index,
+                            include_columns=c.include_columns,
+                            consolidate_blocks=c.consolidate_blocks,
+                            )
+        except ErrorNPYEncode:
+            # NOTE: catch NPY failures and remove self._fp to not leave a malformed zip
+            if os.path.exists(self._fp):
+                os.remove(self._fp)
+            raise
+
+    @store_coherent_non_write
+    def labels(self, *,
+            config: StoreConfigMapInitializer = None,
+            strip_ext: bool = True, # not used
+            ) -> tp.Iterator[tp.Hashable]:
+
+        config_map = StoreConfigMap.from_initializer(config)
+
+        with zipfile.ZipFile(self._fp) as zf:
+            archive = ArchiveZipWrapper(zf,
+                    writeable=False,
+                    memory_map=False,
+                    delimiter=self._DELIMITER,
+                    )
+            # NOTE: this labels() delibers directories of NPY, not individual NPY
+            yield from (config_map.default.label_decode(name)
+                    for name in archive.labels())
+
+    @store_coherent_non_write
+    def read_many(self,
+            labels: tp.Iterable[tp.Hashable],
+            *,
+            config: StoreConfigMapInitializer = None,
+            container_type: tp.Type[Frame] = Frame,
+            ) -> tp.Iterator[Frame]:
+
+        config_map = StoreConfigMap.from_initializer(config)
+
+        with zipfile.ZipFile(self._fp) as zf:
+            archive = ArchiveZipWrapper(zf,
+                    writeable=False,
+                    memory_map=False,
+                    delimiter=self._DELIMITER,
+                    )
+            for label in labels:
+                cache_lookup = self._weak_cache.get(label, NOT_IN_CACHE_SENTINEL)
+                if cache_lookup is not NOT_IN_CACHE_SENTINEL:
+                    yield _StoreZip._set_container_type(cache_lookup, container_type) # type: ignore
+                    continue
+
+                archive.prefix = config_map.default.label_encode(label) # mutate
+                frame = ArchiveFrameConverter.frame_decode(
+                            archive=archive,
+                            constructor=container_type,
+                            )
+                # Newly read frame, add it to our weak_cache
+                self._weak_cache[label] = frame
+                yield frame
