@@ -1,47 +1,54 @@
+from __future__ import annotations
+
 import typing as tp
 from collections.abc import Set
+from itertools import chain
 
 import numpy as np
 
-from static_frame.core.frame import Frame
+from static_frame.core.axis_map import buses_to_hierarchy
 from static_frame.core.bus import Bus
 from static_frame.core.container import ContainerBase
+from static_frame.core.container_util import index_from_optional_constructor
+from static_frame.core.container_util import index_many_concat
+from static_frame.core.container_util import iter_component_signature_bytes
+from static_frame.core.container_util import rehierarch_from_index_hierarchy
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
 from static_frame.core.display import DisplayHeader
 from static_frame.core.display_config import DisplayConfig
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import ErrorInitYarn
+from static_frame.core.exception import RelabelInvalid
+from static_frame.core.frame import Frame
+from static_frame.core.index import Index
+from static_frame.core.index_auto import IndexAutoFactory
+from static_frame.core.index_auto import IndexAutoFactoryType
+from static_frame.core.index_auto import RelabelInput
 from static_frame.core.index_base import IndexBase
 from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.node_iter import IterNodeApplyType
 from static_frame.core.node_iter import IterNodeNoArg
 from static_frame.core.node_iter import IterNodeType
-from static_frame.core.node_iter import IterNodeApplyType
+from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.node_selector import InterfaceSelectTrio
-from static_frame.core.util import IndexInitializer
-from static_frame.core.container_util import rehierarch_from_index_hierarchy
-
 from static_frame.core.series import Series
 from static_frame.core.store_client_mixin import StoreClientMixin
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import NameType
+from static_frame.core.style_config import StyleConfig
 from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import NAME_DEFAULT
-
-from static_frame.core.style_config import StyleConfig
-from static_frame.core.axis_map import buses_to_hierarchy
-from static_frame.core.index_auto import IndexAutoFactoryType
-from static_frame.core.index_auto import IndexAutoFactory
-from static_frame.core.node_selector import InterfaceGetItem
-from static_frame.core.container_util import index_many_set
-from static_frame.core.index import Index
+from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import IndexConstructor
-from static_frame.core.container_util import index_from_optional_constructor
-
-from static_frame.core.index_auto import RelabelInput
-from static_frame.core.exception import RelabelInvalid
+from static_frame.core.util import IndexConstructors
+from static_frame.core.util import IndexInitializer
+from static_frame.core.util import IntegerLocType
+from static_frame.core.util import NameType
 from static_frame.core.util import is_callable_or_mapping
 
+if tp.TYPE_CHECKING:
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
+
 
 class Yarn(ContainerBase, StoreClientMixin):
     '''
@@ -112,7 +119,7 @@ class Yarn(ContainerBase, StoreClientMixin):
             name:
             deepcopy_from_bus:
         '''
-        bus_components = []
+        bus_components: tp.List[Bus] = []
         index_components: tp.Optional[tp.List[IndexBase]] = None if index is not None else []
         for element in containers:
             if isinstance(element, Yarn):
@@ -128,7 +135,7 @@ class Yarn(ContainerBase, StoreClientMixin):
         array.flags.writeable = False
 
         if index_components is not None:
-            index = index_many_set(index_components, Index, union=True)
+            index = index_many_concat(index_components, Index)
 
         series = Series(array, name=name)
         return cls(series,
@@ -140,7 +147,7 @@ class Yarn(ContainerBase, StoreClientMixin):
     def __init__(self,
             series: tp.Union[Series, tp.Iterable[Bus]],
             *,
-            index: tp.Optional[tp.Union[IndexBase, IndexAutoFactoryType]] = None,
+            index: IndexInitializer | IndexAutoFactoryType | None = None,
             index_constructor: tp.Optional[IndexConstructor] = None,
             deepcopy_from_bus: bool = False,
             hierarchy: tp.Optional[IndexHierarchy] = None,
@@ -186,7 +193,7 @@ class Yarn(ContainerBase, StoreClientMixin):
                     explicit_constructor=index_constructor
                     )
         else: # an iterable of labels or an Index
-            self._index = index_from_optional_constructor(index, #type: ignore
+            self._index = index_from_optional_constructor(index,
                     default_constructor=Index,
                     explicit_constructor=index_constructor
                     )
@@ -211,12 +218,12 @@ class Yarn(ContainerBase, StoreClientMixin):
         Returns:
             :obj:`Index`
         '''
-        return reversed(self._index) #type: ignore
+        return reversed(self._index)
 
     #---------------------------------------------------------------------------
     # name interface
 
-    @property #type: ignore
+    @property
     @doc_inject()
     def name(self) -> NameType:
         '''{}'''
@@ -242,11 +249,11 @@ class Yarn(ContainerBase, StoreClientMixin):
 
     @property
     def loc(self) -> InterfaceGetItem['Yarn']:
-        return InterfaceGetItem(self._extract_loc)
+        return InterfaceGetItem(self._extract_loc) # type: ignore
 
     @property
     def iloc(self) -> InterfaceGetItem['Yarn']:
-        return InterfaceGetItem(self._extract_iloc)
+        return InterfaceGetItem(self._extract_iloc) # type: ignore
 
     @property
     def drop(self) -> InterfaceSelectTrio['Yarn']:
@@ -291,7 +298,7 @@ class Yarn(ContainerBase, StoreClientMixin):
     # common attributes from the numpy array
 
     @property
-    def dtype(self) -> np.dtype:
+    def dtype(self) -> DtypeAny:
         '''
         Return the dtype of the realized NumPy array.
 
@@ -397,7 +404,7 @@ class Yarn(ContainerBase, StoreClientMixin):
     _items_store = items
 
     @property
-    def values(self) -> np.ndarray:
+    def values(self) -> NDArrayAny:
         '''A 1D object array of all :obj:`Frame` contained in all contained :obj:`Bus`.
         '''
         array = np.empty(shape=len(self._index), dtype=DTYPE_OBJECT)
@@ -496,7 +503,7 @@ class Yarn(ContainerBase, StoreClientMixin):
     #---------------------------------------------------------------------------
     # extraction
 
-    def _extract_iloc(self, key: GetItemKeyType) -> 'Yarn':
+    def _extract_iloc(self, key: IntegerLocType) -> Yarn | Frame:
         '''
         Returns:
             Yarn or, if an element is selected, a Frame
@@ -507,7 +514,8 @@ class Yarn(ContainerBase, StoreClientMixin):
             return self._series[target_hierarchy[0]][target_hierarchy[1]] #type: ignore
 
         # get the outer-most index of the hierarchical index
-        target_bus_index = target_hierarchy._levels.index
+        target_bus_index = target_hierarchy.unique(depth_level=0, order_by_occurrence=True)
+        target_bus_index = next(iter(target_hierarchy._index_constructors))(target_bus_index)
 
         # create a Boolean array equal to the entire realized length
         valid = np.full(len(self._index), False)
@@ -541,14 +549,14 @@ class Yarn(ContainerBase, StoreClientMixin):
                 own_index=True,
                 )
 
-    def _extract_loc(self, key: GetItemKeyType) -> 'Yarn':
+    def _extract_loc(self, key: GetItemKeyType) -> Yarn | Frame:
         # use the index active for this Yarn
         key_iloc = self._index._loc_to_iloc(key)
         return self._extract_iloc(key_iloc)
 
 
     @doc_inject(selector='selector')
-    def __getitem__(self, key: GetItemKeyType) -> 'Yarn':
+    def __getitem__(self, key: GetItemKeyType) -> Yarn | Frame:
         '''Selector of values by label.
 
         Args:
@@ -562,7 +570,7 @@ class Yarn(ContainerBase, StoreClientMixin):
     def _drop_iloc(self, key: GetItemKeyType) -> 'Yarn':
         invalid = np.full(len(self._index), True)
         invalid[key] = False
-        return self._extract_iloc(invalid)
+        return self._extract_iloc(invalid) # type: ignore
 
     def _drop_loc(self, key: GetItemKeyType) -> 'Yarn':
         return self._drop_iloc(self._index._loc_to_iloc(key))
@@ -605,10 +613,13 @@ class Yarn(ContainerBase, StoreClientMixin):
                 header=DisplayHeader(self.__class__, self._series._name),
                 config=config)
 
+        # NOTE: do not load FrameDeferred, so concatenate contained Series's values directly
         array = np.empty(shape=len(self._index), dtype=DTYPE_OBJECT)
-        # NOTE: do not load FrameDeferred, so concate contained Series's values directly
-        np.concatenate([b._values_mutable for b in self._series.values], out=array)
+        np.concatenate(
+            [b._values_mutable for b in self._series.values],
+            out=array)
         array.flags.writeable = False
+
         series = Series(array, index=self._index, own_index=True)
 
         return series._display(config,
@@ -631,11 +642,10 @@ class Yarn(ContainerBase, StoreClientMixin):
     def dtypes(self) -> Frame:
         '''Returns a Frame of dtypes for all loaded Frames.
         '''
-        f = Frame.from_concat(
+        return Frame.from_concat(
                 frames=(f.dtypes for f in self._series.values),
                 fill_value=None,
                 ).relabel(index=self._index)
-        return tp.cast(Frame, f)
 
     @property
     def shapes(self) -> Series:
@@ -661,7 +671,7 @@ class Yarn(ContainerBase, StoreClientMixin):
         f = Frame.from_concat(
                 (b.status for b in self._series.values),
                 index=IndexAutoFactory)
-        return f.relabel(index=self._index) # type: ignore
+        return f.relabel(index=self._index)
 
 
 
@@ -676,6 +686,36 @@ class Yarn(ContainerBase, StoreClientMixin):
         # NOTE: this should load all deferred Frame
         return Series(self.values, index=self._index, own_index=True)
 
+    def _to_signature_bytes(self,
+            include_name: bool = True,
+            include_class: bool = True,
+            encoding: str = 'utf-8',
+            ) -> bytes:
+
+        v = (f._to_signature_bytes(
+                include_name=include_name,
+                include_class=include_class,
+                encoding=encoding,
+                ) for f in self._axis_element())
+
+        return b''.join(chain(
+                iter_component_signature_bytes(self,
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),
+                (self._index._to_signature_bytes(
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),
+                self._hierarchy._to_signature_bytes(
+                        include_name=include_name,
+                        include_class=include_class,
+                        encoding=encoding),),
+                v))
+
+
+
+
     #---------------------------------------------------------------------------
     # index manipulation
 
@@ -695,7 +735,7 @@ class Yarn(ContainerBase, StoreClientMixin):
             index_init = None
         elif index is None:
             index_init = self._index
-        elif is_callable_or_mapping(index): #type: ignore
+        elif is_callable_or_mapping(index):
             index_init = self._index.relabel(index)
             own_index = True
         elif isinstance(index, Set):
@@ -763,7 +803,9 @@ class Yarn(ContainerBase, StoreClientMixin):
                 )
 
     def rehierarch(self,
-            depth_map: tp.Sequence[int]
+            depth_map: tp.Sequence[int],
+            *,
+            index_constructors: IndexConstructors = None,
             ) -> 'Yarn':
         '''
         Return a new :obj:`Series` with new a hierarchy based on the supplied ``depth_map``.
@@ -774,7 +816,8 @@ class Yarn(ContainerBase, StoreClientMixin):
         index, iloc_map = rehierarch_from_index_hierarchy(
                 labels=self._index, #type: ignore
                 depth_map=depth_map,
+                index_constructors=index_constructors,
                 name=self._index.name,
                 )
 
-        return self._extract_iloc(iloc_map).relabel(index) #type: ignore
+        return self._extract_iloc(iloc_map).relabel(index) # type: ignore
