@@ -1,29 +1,33 @@
+from __future__ import annotations
 
-import unittest
-import os
-import sys
-from itertools import zip_longest
-import typing as tp
-import itertools as it
-import string
 import cmath
-import sqlite3
 import contextlib
+import datetime
+import itertools as it
+import os
+import sqlite3
+import string
+import sys
 import tempfile
 import time
-import datetime
+import typing as tp
+import unittest
+from itertools import zip_longest
 from pathlib import Path
 
 import numpy as np
 import pytest
 
-
 from static_frame import TypeBlocks
-from static_frame.core.container import ContainerOperand
+from static_frame.core.container import ContainerBase
 from static_frame.core.frame import Frame
 from static_frame.core.index_base import IndexBase
+from static_frame.core.index_datetime import IndexDatetime
 from static_frame.core.util import PathSpecifier
 
+if tp.TYPE_CHECKING:
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 # for running with coverage
 # pytest -s --color no --disable-pytest-warnings --cov=static_frame --cov-report html static_frame/test
@@ -36,16 +40,22 @@ skip_win = pytest.mark.skipif(
         reason='Windows default dtypes'
         )
 
-skip_pylt37 = pytest.mark.skipif(
-        sys.version_info < (3, 7),
-        reason='Python earlier than 3.7'
-        )
+# skip_pylt37 = pytest.mark.skipif(
+#         sys.version_info < (3, 7),
+#         reason='Python earlier than 3.7'
+#         )
 
 skip_linux_no_display = pytest.mark.skipif(
         sys.platform == 'linux' and 'DISPLAY' not in os.environ,
         reason='No display available'
         )
 
+skip_mac_pyle38 = pytest.mark.skipif(
+        sys.platform == 'darwin' and sys.version_info[:2] <= (3, 8),
+        reason='MacOS tk.h issue'
+        )
+
+
 #-------------------------------------------------------------------------------
 class Timer():
 
@@ -87,7 +97,7 @@ class TestCase(unittest.TestCase):
     '''
 
     @staticmethod
-    def get_arrays_a() -> tp.Iterator[np.ndarray]:
+    def get_arrays_a() -> tp.Iterator[NDArrayAny]:
         '''
         Return sample array suitable for TypeBlock creation, testing. Unique values required.
         '''
@@ -109,7 +119,7 @@ class TestCase(unittest.TestCase):
 
 
     @staticmethod
-    def get_arrays_b() -> tp.Iterator[np.ndarray]:
+    def get_arrays_b() -> tp.Iterator[NDArrayAny]:
         '''
         Return sample array suitable for TypeBlock creation, testing. Many NaNs.
         '''
@@ -146,16 +156,17 @@ class TestCase(unittest.TestCase):
 
 
     @staticmethod
-    def get_containers() -> tp.Iterator[tp.Type[ContainerOperand]]:
-
-        def yield_sub(cls: tp.Type[ContainerOperand]) -> tp.Iterator[tp.Type[ContainerOperand]]:
+    def get_containers() -> tp.Iterator[tp.Type[ContainerBase]]:
+        '''This function is a dynamic search of containers, to only be used in testing. For a declaritive alternative, use container_util.ContainerMap.
+        '''
+        def yield_sub(cls: tp.Type[ContainerBase]) -> tp.Iterator[tp.Type[ContainerBase]]:
             for cls in cls.__subclasses__():
-                if cls is not IndexBase:
+                if cls is not IndexBase and cls is not IndexDatetime:
                     yield cls
-                if issubclass(cls, ContainerOperand):
+                if issubclass(cls, ContainerBase):
                     yield from yield_sub(cls)
 
-        yield from yield_sub(ContainerOperand)
+        yield from yield_sub(ContainerBase)
 
     @staticmethod
     def get_test_db_a() -> sqlite3.Connection:
@@ -243,8 +254,8 @@ class TestCase(unittest.TestCase):
     #---------------------------------------------------------------------------
 
     def assertEqualWithNaN(self,
-            v1: object,
-            v2: object,
+            v1: tp.Any,
+            v2: tp.Any,
             ) -> None:
         if ((isinstance(v1, float) or isinstance(v1, np.floating))
                 and np.isnan(v1)
@@ -256,21 +267,21 @@ class TestCase(unittest.TestCase):
         if ((isinstance(v1, complex) or isinstance(v1, np.complexfloating))
                 and cmath.isnan(v1)
                 and (isinstance(v2, complex) or isinstance(v1, np.complexfloating))
-                and cmath.isnan(v2)  # type: ignore
+                and cmath.isnan(v2)
                 ):
             return
         return self.assertEqual(v1, v2)
 
 
-    def assertAlmostEqualArray(self, a1: np.ndarray, a2: np.ndarray) -> None:
+    def assertAlmostEqualArray(self, a1: NDArrayAny, a2: NDArrayAny) -> None:
         # NaNs are treated as equal
         np.testing.assert_allclose(a1, a2)
         # np.testing.assert_array_almost_equal(a1, a2, decimal=5)
 
     def assertTypeBlocksArrayEqual(self,
             tb: TypeBlocks,
-            match: tp.Iterable[object],
-            match_dtype: tp.Optional[tp.Union[type, np.dtype, str]] = None) -> None:
+            match: tp.Iterable[tp.Any],
+            match_dtype: tp.Optional[tp.Union[type, DtypeAny, str]] = None) -> None:
         '''
         Args:
             tb: a TypeBlocks instance
@@ -284,14 +295,14 @@ class TestCase(unittest.TestCase):
 
 
     def assertAlmostEqualValues(self,
-            values1: tp.Iterable[object], values2: tp.Iterable[object]) -> None:
+            values1: tp.Iterable[tp.Any], values2: tp.Iterable[tp.Any]) -> None:
 
         for v1, v2 in zip_longest(values1, values2):
             self.assertEqualWithNaN(v1, v2)
 
     def assertAlmostEqualItems(self,
-            pairs1: tp.Iterable[tp.Tuple[tp.Hashable, object]],
-            pairs2: tp.Iterable[tp.Tuple[tp.Hashable, object]]) -> None:
+            pairs1: tp.Iterable[tp.Tuple[tp.Hashable, tp.Any]],
+            pairs2: tp.Iterable[tp.Tuple[tp.Hashable, tp.Any]]) -> None:
 
         for (k1, v1), (k2, v2) in zip_longest(pairs1, pairs2):
             self.assertEqual(k1, k2)
@@ -303,8 +314,8 @@ class TestCase(unittest.TestCase):
 
 
     def assertAlmostEqualFramePairs(self,
-            pairs1: tp.Iterable[tp.Tuple[tp.Hashable, tp.Iterable[object]]],
-            pairs2: tp.Iterable[tp.Tuple[tp.Hashable, tp.Iterable[object]]]) -> None:
+            pairs1: tp.Iterable[tp.Tuple[tp.Hashable, tp.Iterable[tp.Any]]],
+            pairs2: tp.Iterable[tp.Tuple[tp.Hashable, tp.Iterable[tp.Any]]]) -> None:
         '''
         For comparing nested tuples returned by Frame.to_pairs()
         '''
