@@ -1,34 +1,36 @@
+from __future__ import annotations
+
+import os
+import pickle
+import typing as tp
 from datetime import date
 from datetime import datetime
-# from io import StringIO
-import typing as tp
+from hashlib import sha256
+
+import frame_fixtures as ff
 import numpy as np
-import frame_fixtures as ff
 
-from static_frame.core.frame import Frame
 from static_frame.core.bus import Bus
 from static_frame.core.bus import FrameDeferred
-
-from static_frame.core.series import Series
-from static_frame.core.index_hierarchy import IndexHierarchy
-from static_frame.core.store_zip import StoreZipTSV
-
-from static_frame.core.store import StoreConfigMap
-from static_frame.core.store import StoreConfig
 from static_frame.core.display_config import DisplayConfig
-from static_frame.core.hloc import HLoc
-
-from static_frame.test.test_case import TestCase
-from static_frame.test.test_case import temp_file
-from static_frame.test.test_case import skip_win
-
-from static_frame.core.index_auto import IndexAutoFactory
-
 from static_frame.core.exception import ErrorInitBus
-from static_frame.core.exception import StoreFileMutation
 from static_frame.core.exception import ErrorInitIndexNonUnique
-
+from static_frame.core.exception import ErrorNPYEncode
+from static_frame.core.exception import StoreFileMutation
+from static_frame.core.frame import Frame
+from static_frame.core.hloc import HLoc
+from static_frame.core.index_auto import IndexAutoConstructorFactory
+from static_frame.core.index_auto import IndexAutoFactory
 from static_frame.core.index_datetime import IndexDate
+from static_frame.core.index_datetime import IndexYearMonth
+from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.series import Series
+from static_frame.core.store_config import StoreConfig
+from static_frame.core.store_config import StoreConfigMap
+from static_frame.core.store_zip import StoreZipTSV
+from static_frame.test.test_case import TestCase
+from static_frame.test.test_case import skip_win
+from static_frame.test.test_case import temp_file
 
 
 class TestUnit(TestCase):
@@ -77,38 +79,39 @@ class TestUnit(TestCase):
 
             f3 = b2['bar']
             f4 = b2['foo']
-            # import ipdb; ipdb.set_trace()
             zs = StoreZipTSV(fp)
             zs.write(b1.items())
 
             # how to show that this derived getitem has derived type?
             f3 = zs.read('foo', config=config['foo'])
             self.assertEqual(
-                f3.to_pairs(0),
+                f3.to_pairs(0), # type: ignore
                 (('a', (('x', 1), ('y', 2))), ('b', (('x', 3), ('y', 4))))
             )
 
     def test_bus_init_b(self) -> None:
 
         with self.assertRaises(ErrorInitBus):
-            Bus(Series([1, 2, 3]))
+            Bus.from_series(Series([1, 2, 3]))
 
         with self.assertRaises(ErrorInitBus):
-            Bus(Series([3, 4], dtype=object))
+            Bus.from_series(Series([3, 4], dtype=object))
 
         with self.assertRaises(ErrorInitBus):
-            Bus(Series([3, 4], index=('a', 'b'), dtype=object))
+            Bus.from_series(Series([3, 4], index=('a', 'b'), dtype=object))
 
     def test_bus_init_c(self) -> None:
 
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
                 index=('x', 'y'),
-                name='foo')
+                name='foo',
+                dtypes=np.int64)
         f2 = Frame.from_dict(
                 dict(a=(1,2,3), b=(4,5,6)),
                 index=('x', 'y', 'z'),
-                name='bar')
+                name='bar',
+                dtypes=np.int64)
 
         config = StoreConfigMap.from_config(StoreConfig(index_depth=1))
         b1 = Bus.from_frames((f1, f2), config=config)
@@ -126,6 +129,44 @@ class TestUnit(TestCase):
             self.assertEqualFrames(f1, f1_loaded)
             self.assertEqualFrames(f2, f2_loaded)
 
+    def test_bus_init_d(self) -> None:
+        f1 = ff.parse('s(2,2)|c(I,str)|v(int)')
+        f2 = ff.parse('s(2,2)|c(I,str)|v(bool)')
+
+        b1 = Bus((f1, f2), index=('a', 'b'))
+        self.assertEqual(b1.shapes.to_pairs(),
+                (('a', (2, 2)), ('b', (2, 2))),
+                )
+
+        b2 = Bus((f1, f2),
+                index=IndexDate(('2021-01-01', '1542-01-22')),
+                own_index=True,
+                )
+        self.assertEqual(b2.keys().values.tolist(),
+                [date(2021, 1, 1), date(1542, 1, 22)]
+                )
+
+    def test_bus_init_e(self) -> None:
+        with self.assertRaises(ErrorInitBus):
+            b1 = Bus(np.arange(0,2), index=('a', 'b'))
+
+    def test_bus_init_f(self) -> None:
+        with self.assertRaises(ErrorInitBus):
+            _ = Bus(frames=None, index=('a', 'b', 'c'))
+
+    def test_bus_init_g(self) -> None:
+        f1 = ff.parse('s(2,2)|c(I,str)|v(int)')
+        f2 = ff.parse('s(2,2)|c(I,str)|v(bool)')
+
+        with self.assertRaises(ErrorInitBus):
+            _ = Bus((f1, f2), index=('a', 'b'), own_data=True)
+
+    def test_bus_init_h(self) -> None:
+        f1 = ff.parse('s(2,2)|c(I,str)|v(int)')
+
+        with self.assertRaises(ErrorInitBus):
+            b1 = Bus((f for f in (f1,)), index=('a', 'b'))
+
     #---------------------------------------------------------------------------
 
     def test_bus_from_frames_a(self) -> None:
@@ -142,6 +183,7 @@ class TestUnit(TestCase):
         with self.assertRaises(ErrorInitIndexNonUnique):
             _ = Bus.from_frames((f1, f2))
 
+
     #---------------------------------------------------------------------------
 
     def test_bus_from_series_a(self) -> None:
@@ -221,7 +263,7 @@ class TestUnit(TestCase):
                     (('f1', None), ('f2', (3, 2)), ('f3', (2, 2 )))
                     )
 
-    @skip_win # type: ignore
+    @skip_win
     def test_bus_nbytes_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -254,7 +296,7 @@ class TestUnit(TestCase):
 
             self.assertEqual(b2.nbytes, 112)
 
-    @skip_win # type: ignore
+    @skip_win
     def test_bus_dtypes_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -289,7 +331,7 @@ class TestUnit(TestCase):
                     (('b', (('f1', None), ('f2', np.dtype('int64')), ('f3', np.dtype('int64')))), ('c', (('f1', None), ('f2', np.dtype('int64')), ('f3', None))), ('d', (('f1', None), ('f2', None), ('f3', np.dtype('int64')))))
                     )
 
-    @skip_win # type: ignore
+    @skip_win
     def test_bus_status_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -759,7 +801,7 @@ class TestUnit(TestCase):
 
             b2 = Bus.from_sqlite(fp, config=config, index_constructor=IndexDate)
             tuple(b2.items()) # force loading all
-            self.assertEqual(b2.index.dtype.kind, 'M')
+            self.assertEqual(b2.index.dtype.kind, 'M') # type: ignore
 
 
         for frame in (f1, f2):
@@ -837,7 +879,7 @@ class TestUnit(TestCase):
 
             b2 = Bus.from_hdf5(fp, config=config, index_constructor=IndexDate)
             tuple(b2.items()) # force loading all
-            self.assertEqual(b2.index.dtype.kind, 'M')
+            self.assertEqual(b2.index.dtype.kind, 'M') # type: ignore
 
 
         for frame in (f1, f2):
@@ -1047,7 +1089,7 @@ class TestUnit(TestCase):
 
         # simulating a Bus with a FrameDefferred but no Store, just for testing
         s1 = Series((f1, FrameDeferred), index=('p', 'q'))
-        b1 = Bus(s1, config=config)
+        b1 = Bus.from_series(s1, config=config)
         self.assertFalse(b1._loaded_all)
 
         with self.assertRaises(RuntimeError):
@@ -1076,7 +1118,7 @@ class TestUnit(TestCase):
         # do not support IndexHierarchy, as lables are tuples, not strings
         from ast import literal_eval
         config = StoreConfig(label_encoder=str, label_decoder=literal_eval)
-        b1 = Bus(s1)
+        b1 = Bus.from_series(s1)
         b2 = b1[HLoc[:, 1]]
         self.assertEqual(b2.shape, (2,))
         self.assertEqual(b2.index.values.tolist(),
@@ -1174,7 +1216,7 @@ class TestUnit(TestCase):
                 )
 
         b1 = Bus.from_frames((f1, f2, f3), config=config, index_constructor=IndexDate)
-        self.assertEqual(b1.index.dtype.kind, 'M')
+        self.assertEqual(b1.index.dtype.kind, 'M') # type: ignore
 
         with temp_file('.zip') as fp:
             b1.to_zip_parquet(fp, config=config)
@@ -1182,7 +1224,7 @@ class TestUnit(TestCase):
             b2 = Bus.from_zip_parquet(fp,
                     config=config,
                     index_constructor=IndexDate)
-            self.assertEqual(b2.index.dtype.kind, 'M')
+            self.assertEqual(b2.index.dtype.kind, 'M') # type: ignore
 
             key = dt64('2020-01-01')
             self.assertEqualFrames(b1[key], b2[key], compare_dtype=False)
@@ -1213,7 +1255,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1240,7 +1282,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1259,11 +1301,11 @@ class TestUnit(TestCase):
             self.assertEqual(b3._loaded.tolist(),
                     [False, False, False, False, False, False, False, False, False, True]
                     )
-            self.assertEqual(b3.iloc[0].sum().sum(), 145)
+            self.assertEqual(b3.iloc[0].sum().sum(), 145) # type: ignore
             self.assertEqual(b3._loaded.tolist(),
                     [True, False, False, False, False, False, False, False, False, False]
                     )
-            self.assertEqual(b3.iloc[4].sum().sum(), 185)
+            self.assertEqual(b3.iloc[4].sum().sum(), 185) # type: ignore
             self.assertEqual(b3._loaded.tolist(),
                     [False, False, False, False, True, False, False, False, False, False]
                     )
@@ -1274,7 +1316,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1320,7 +1362,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1372,7 +1414,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1407,7 +1449,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1434,7 +1476,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1461,7 +1503,7 @@ class TestUnit(TestCase):
         s1 = Series((f1, f2, f3), index=('a', 'b', 'c'))
         with self.assertRaises(ErrorInitBus):
             # max_persist cannot be less than the number of already loaded Frames
-            Bus(s1, max_persist=2)
+            Bus.from_series(s1, max_persist=2)
 
     def test_bus_max_persist_i(self) -> None:
         f1 = ff.parse('s(4,2)').rename('f1')
@@ -1513,7 +1555,7 @@ class TestUnit(TestCase):
             a1 = b2.values
             self.assertNotEqual(id(a1), id(b2._values_mutable))
             self.assertEqual(b2.status['loaded'].sum(), 2)
-            self.assertTrue(all([f.__class__ is Frame for f in a1]))
+            self.assertTrue(all(f.__class__ is Frame for f in a1))
 
     #---------------------------------------------------------------------------
 
@@ -1544,11 +1586,11 @@ class TestUnit(TestCase):
             self.assertTrue(isinstance(b3, Bus))
 
             f4 = b3['f2']
-            self.assertEqual(f4.to_pairs(0),
+            self.assertEqual(f4.to_pairs(0), # type: ignore
                     (('c', (('x', 1), ('y', 2), ('z', 3))), ('b', (('x', 4), ('y', 5), ('z', 6)))))
 
             f5 = b3['f1']
-            self.assertEqual(f5.to_pairs(),
+            self.assertEqual(f5.to_pairs(), # type: ignore
                     (('a', (('x', 1), ('y', 2))), ('b', (('x', 3), ('y', 4)))))
 
     #---------------------------------------------------------------------------
@@ -1597,7 +1639,7 @@ class TestUnit(TestCase):
         b1 = Bus.from_frames((f3, f2, f1))
         b2 = b1.iter_element().apply(lambda f: f.sum().sum())
 
-        self.assertEqual(b2.to_pairs(),
+        self.assertEqual(b2.to_pairs(), # type: ignore
                 (('f3', 14), ('f2', 21), ('f1', 30007)))
 
         self.assertEqual(id(b1.index), id(b2.index))
@@ -1645,6 +1687,20 @@ class TestUnit(TestCase):
         self.assertEqual(b1.index.values.tolist(),
                 ['a', 'b'])
 
+    def test_bus_from_dict_b(self) -> None:
+        f1 = Frame.from_dict(
+                dict(a=(1,2), b=(3,4)),
+                index=('x', 'y'),
+                name='f1')
+        f2 = Frame.from_dict(
+                dict(c=(1,2,3), b=(4,5,6)),
+                index=('x', 'y', 'z'),
+                name='f2')
+
+        b1 = Bus.from_dict({'2021-01':f1, '2021-06':f2}, index_constructor=IndexYearMonth)
+        self.assertIs(b1.index.__class__, IndexYearMonth)
+
+
     #---------------------------------------------------------------------------
 
     def test_bus_iter_element_a(self) -> None:
@@ -1785,7 +1841,7 @@ class TestUnit(TestCase):
                 yield 'abcde'[i], Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1951,7 +2007,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -1986,7 +2042,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -2016,7 +2072,7 @@ class TestUnit(TestCase):
                 yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
 
         s = Series.from_items(items(), dtype=object)
-        b1 = Bus(s)
+        b1 = Bus.from_series(s)
 
         config = StoreConfig(
                 index_depth=1,
@@ -2202,10 +2258,150 @@ class TestUnit(TestCase):
             # set max_persist to size to test when fully loaded with max_persist
             b2 = Bus.from_zip_npz(fp, config=config, max_persist=3)
             b3 = b2['f2':]
-            self.assertEqual(b3['f5'].to_pairs(),
+            self.assertEqual(b3['f5'].to_pairs(), # type: ignore
                 ((0, ((0, 1930.4), (1, -1760.34), (2, 1857.34), (3, 1699.34))), (1, ((0, -610.8), (1, 3243.94), (2, -823.14), (3, 114.58))), (2, ((0, 694.3), (1, -72.96), (2, 1826.02), (3, 604.1))), (3, ((0, 1080.4), (1, 2580.34), (2, 700.42), (3, 3338.48))))
                 )
 
+    def test_bus_npz_b(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3').astype(object)
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        config = StoreConfig()
+
+        with temp_file('.zip') as fp:
+            with self.assertRaises(ErrorNPYEncode):
+                b1.to_zip_npz(fp)
+            self.assertFalse(os.path.exists(fp))
+
+    def test_bus_npz_c(self) -> None:
+        frame = Frame(
+            data=np.random.normal(size=(2, 2)),
+            columns=IndexAutoFactory,
+            index=IndexAutoFactory,
+            name=np.datetime64('2000-01-01'),
+            )
+
+        b1 = Bus.from_frames(
+            frames=(frame,),
+            index_constructor=IndexDate,
+            )
+
+        config = StoreConfig(
+            label_encoder=str,
+            label_decoder=np.datetime64,
+            )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_npz(fp, config=config)
+            b2 = Bus.from_zip_npz(fp, config=config, index_constructor=IndexAutoConstructorFactory)
+            self.assertEqual(frame.name, b2.iloc[0].name)
+            self.assertEqual(frame.shape, b2.iloc[0].shape)
+
+
+    #---------------------------------------------------------------------------
+    def test_bus_npy_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+
+        config = StoreConfig()
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_npy(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_npy(fp, config=config, max_persist=3)
+            b3 = b2['f2':]
+            self.assertEqual(b3['f5'].to_pairs(), # type: ignore
+                ((0, ((0, 1930.4), (1, -1760.34), (2, 1857.34), (3, 1699.34))), (1, ((0, -610.8), (1, 3243.94), (2, -823.14), (3, 114.58))), (2, ((0, 694.3), (1, -72.96), (2, 1826.02), (3, 604.1))), (3, ((0, 1080.4), (1, 2580.34), (2, 700.42), (3, 3338.48))))
+                )
+
+    def test_bus_npy_b(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3').astype(object)
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        config = StoreConfig()
+
+        with temp_file('.zip') as fp:
+            with self.assertRaises(ErrorNPYEncode):
+                b1.to_zip_npy(fp)
+            self.assertFalse(os.path.exists(fp))
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_to_signature_bytes_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        bytes1 = b1._to_signature_bytes(include_name=False)
+        self.assertEqual(sha256(bytes1).hexdigest(),
+                '29a271e0d800ecaa673c7deded9dd7e8166cc746963c1717298e6af9e4189f23')
+
+        b2 = Bus.from_frames((f1, f2))
+        bytes2 = b2._to_signature_bytes(include_name=False)
+        self.assertNotEqual(sha256(bytes1).hexdigest(), sha256(bytes2).hexdigest())
+
+    def test_bus_to_signature_bytes_b(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,2)').rename('f4')
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        bytes1 = b1._to_signature_bytes(include_name=False)
+
+        b2 = Bus.from_frames((f1, f2, f4))
+        bytes2 = b2._to_signature_bytes(include_name=False)
+        self.assertNotEqual(sha256(bytes1).hexdigest(), sha256(bytes2).hexdigest())
+
+
+    def test_bus_via_hashlib_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        d = b1.via_hashlib(include_name=False).sha256().hexdigest()
+        self.assertEqual(d,
+                '29a271e0d800ecaa673c7deded9dd7e8166cc746963c1717298e6af9e4189f23')
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_store_pickle_roundtrip(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(2,2)').rename('f2')
+
+        b1 = Bus.from_frames((f1, f2))
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_npz(fp)
+            b2 = Bus.from_zip_npz(fp, max_persist=1)
+
+            assert not b2._store._weak_cache # type: ignore
+
+            f1_r = b2.iloc[0]
+            f2_r = b2.iloc[1]
+
+            assert b2.iloc[1] is f2_r
+
+            assert f1_r in b2._store._weak_cache.values() # type: ignore
+            assert b2.iloc[0] is f1_r
+
+            b3 = pickle.loads(pickle.dumps(b2))
+
+            assert not b3._store._weak_cache
+            assert b3.iloc[0].equals(f1_r)
+
 
 if __name__ == '__main__':
     import unittest
