@@ -1,29 +1,10 @@
+from __future__ import annotations
+
 import datetime
+import typing as tp
 
 import numpy as np
 
-from static_frame.core.container_util import bloc_key_normalize
-from static_frame.core.container_util import get_col_dtype_factory
-from static_frame.core.container_util import index_from_optional_constructor
-from static_frame.core.container_util import index_many_concat
-from static_frame.core.container_util import index_many_set
-from static_frame.core.container_util import is_static
-from static_frame.core.container_util import key_to_ascending_key
-from static_frame.core.container_util import matmul
-from static_frame.core.container_util import pandas_to_numpy
-from static_frame.core.container_util import pandas_version_under_1
-from static_frame.core.container_util import apex_to_name
-from static_frame.core.container_util import apply_binary_operator_blocks_columnar
-from static_frame.core.container_util import container_to_exporter_attr
-from static_frame.core.container_util import get_block_match
-
-
-from static_frame.core.frame import FrameHE
-
-from static_frame.test.test_case import TestCase
-from static_frame.core.exception import AxisInvalid
-
-
 from static_frame import Frame
 from static_frame import Index
 from static_frame import IndexDate
@@ -33,6 +14,30 @@ from static_frame import IndexHierarchy
 from static_frame import IndexHierarchyGO
 from static_frame import IndexSecond
 from static_frame import Series
+from static_frame.core.container_util import ContainerMap
+from static_frame.core.container_util import apex_to_name
+from static_frame.core.container_util import apply_binary_operator_blocks_columnar
+from static_frame.core.container_util import bloc_key_normalize
+from static_frame.core.container_util import container_to_exporter_attr
+from static_frame.core.container_util import get_block_match
+from static_frame.core.container_util import get_col_dtype_factory
+from static_frame.core.container_util import get_col_fill_value_factory
+from static_frame.core.container_util import get_col_format_factory
+from static_frame.core.container_util import group_from_container
+from static_frame.core.container_util import index_from_optional_constructor
+from static_frame.core.container_util import index_many_concat
+from static_frame.core.container_util import index_many_to_one
+from static_frame.core.container_util import is_static
+from static_frame.core.container_util import key_to_ascending_key
+from static_frame.core.container_util import matmul
+from static_frame.core.container_util import pandas_to_numpy
+from static_frame.core.container_util import pandas_version_under_1
+from static_frame.core.exception import AxisInvalid
+from static_frame.core.fill_value_auto import FillValueAuto
+from static_frame.core.frame import FrameHE
+from static_frame.core.util import ManyToOneType
+from static_frame.test.test_case import TestCase
+
 
 class TestUnit(TestCase):
 
@@ -147,7 +152,7 @@ class TestUnit(TestCase):
 
         # get an auto incremented integer columns
         self.assertEqual(
-            matmul(f1, np.arange(10).reshape(2, 5)).to_pairs(0),
+            matmul(f1, np.arange(10).reshape(2, 5)).to_pairs(),
             ((0, (('x', 15), ('y', 20), ('z', 25))), (1, (('x', 19), ('y', 26), ('z', 33))), (2, (('x', 23), ('y', 32), ('z', 41))), (3, (('x', 27), ('y', 38), ('z', 49))), (4, (('x', 31), ('y', 44), ('z', 57))))
             )
 
@@ -181,7 +186,7 @@ class TestUnit(TestCase):
             )
 
         # produces a Series indexed 0, 1
-        self.assertEqual(matmul(s1, f1.values).to_pairs(),
+        self.assertEqual(matmul(s1, f1.values).to_pairs(), # type: ignore
             ((0, 17), (1, 35)))
 
     def test_matmul_e(self) -> None:
@@ -200,7 +205,7 @@ class TestUnit(TestCase):
         f1 = Frame.from_items((('a', (1, 2, 3)), ('b', (3, 4, 5))),
                 index=('x', 'y', 'z'))
 
-        self.assertEqual(matmul([3, 4, 5], f1.values).tolist(),
+        self.assertEqual(matmul([3, 4, 5], f1.values).tolist(), # type: ignore
                 [26, 50])
 
         self.assertEqual(matmul([3, 4, 5], f1).to_pairs(),
@@ -211,7 +216,7 @@ class TestUnit(TestCase):
         # lhs: array 1D, rhs: array 1D, Series
 
         s1 = Series((3, 4, 2), index=('x', 'y', 'z'))
-        self.assertEqual(matmul([10, 11, 12], s1.values), 98)
+        self.assertEqual(matmul([10, 11, 12], s1.values), 98) # type: ignore
         self.assertEqual(matmul([10, 11, 12], s1), 98)
 
         with self.assertRaises(RuntimeError):
@@ -224,11 +229,11 @@ class TestUnit(TestCase):
         f2 = Frame.from_dict(dict(p=(1, 2), q=(3, 4), r=(5, 6)), index=tuple('ab'))
 
 
-        self.assertEqual(matmul(f1.values, f2).to_pairs(0),
+        self.assertEqual(matmul(f1.values, f2).to_pairs(),
                 (('p', ((0, 11), (1, 14), (2, 17), (3, 20))), ('q', ((0, 23), (1, 30), (2, 37), (3, 44))), ('r', ((0, 35), (1, 46), (2, 57), (3, 68))))
                 )
 
-        self.assertEqual(matmul(f1, f2.values).to_pairs(0),
+        self.assertEqual(matmul(f1, f2.values).to_pairs(),
                 ((0, (('w', 11), ('x', 14), ('y', 17), ('z', 20))), (1, (('w', 23), ('x', 30), ('y', 37), ('z', 44))), (2, (('w', 35), ('x', 46), ('y', 57), ('z', 68))))
                 )
 
@@ -246,8 +251,8 @@ class TestUnit(TestCase):
         for pair in ((f1, f1.T), (f1, f1.loc['y']), (f1['a'], f1), (f1.loc['y'], f1.loc['z'])):
             for x, y in it.combinations((f_container, f_values, f_container, f_values), 2):
                 post = matmul(x(pair[0]), y(pair[1])) # type: ignore
-                if isinstance(post, (Series, Frame)):
-                    self.assertTrue(post.values.tolist(), (pair[0].values @ pair[1].values).tolist())
+                if isinstance(post, (Series, Frame)): # type: ignore
+                    self.assertTrue(post.values.tolist(), (pair[0].values @ pair[1].values).tolist()) # type: ignore
                 elif isinstance(post, np.ndarray):
                     self.assertTrue(post.tolist(), (pair[0].values @ pair[1].values).tolist())
 
@@ -267,7 +272,8 @@ class TestUnit(TestCase):
         self.assertEqual(key_to_ascending_key([9, 5, 1], 3), [1, 5, 9])
         self.assertEqual(key_to_ascending_key(np.array([9, 5, 1]), 3).tolist(), [1, 5, 9]) # type: ignore
 
-        self.assertEqual(key_to_ascending_key(slice(3, 0, -1), 3), slice(1, 4, 1))
+        self.assertEqual(key_to_ascending_key(slice(3, 0, -1), 3), slice(1, 3, None))
+
         self.assertEqual(key_to_ascending_key(100, 3), 100)
 
         self.assertEqual(key_to_ascending_key([], 3), [])
@@ -285,6 +291,8 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             key_to_ascending_key(dict(a=3), size=3)
 
+    #---------------------------------------------------------------------------
+
     def test_pandas_to_numpy_a(self) -> None:
         import pandas as pd
         pdvu1 = pandas_version_under_1()
@@ -424,6 +432,31 @@ class TestUnit(TestCase):
                 [datetime.date(2020, 1, 1), datetime.date(2020, 1, 2), datetime.date(2020, 2, 1), datetime.date(2020, 2, 2)]
                 )
 
+    def test_index_many_concat_f(self) -> None:
+
+        idx1 = IndexHierarchy.from_product(('a', 'b'), (1, 2))
+        idx2 = IndexHierarchy.from_product(('c', 'd'), (1, 2))
+
+        post = index_many_concat((idx1, idx2), cls_default=Index)
+        post = tp.cast(IndexHierarchy, post)
+
+        self.assertEqual([d.kind for d in post.dtypes.values], ['U', 'i'])
+        self.assertEqual(post.to_frame().to_pairs(),
+                ((0, ((0, 'a'), (1, 'a'), (2, 'b'), (3, 'b'), (4, 'c'), (5, 'c'), (6, 'd'), (7, 'd'))), (1, ((0, 1), (1, 2), (2, 1), (3, 2), (4, 1), (5, 2), (6, 1), (7, 2))))
+                )
+
+    def test_index_many_concat_g(self) -> None:
+
+        idx1 = IndexHierarchy.from_product(('a', 'b'), (1, 2))
+        idx2 = Index(('a', 'b'))
+
+        # both raise for un-aligned depths, regardless of order
+        with self.assertRaises(RuntimeError):
+            post = index_many_concat((idx1, idx2), cls_default=Index)
+
+        with self.assertRaises(RuntimeError):
+            post = index_many_concat((idx2, idx1), cls_default=Index)
+
     #---------------------------------------------------------------------------
 
     def test_index_many_set_a(self) -> None:
@@ -433,18 +466,12 @@ class TestUnit(TestCase):
         idx2 = IndexDate(('2020-01-02', '2020-01-03'))
 
 
-        post1 = index_many_set((idx0,  idx1), Index, union=True)
+        post1 = index_many_to_one((idx0,  idx1), Index, many_to_one_type=ManyToOneType.UNION)
         assert isinstance(post1, Index)
 
         self.assertEqual(post1.name, 'foo')
         self.assertEqual(post1.__class__, Index)
 
-        # self.assertEqual(set(post1.values),
-        #         {'1997-01-02',
-        #         '1997-01-01',
-        #         np.datetime64('2020-01-01'),
-        #         np.datetime64('2020-01-02')})
-
         # the result of this operation is an unstable ordering
         values = set(post1.values)
         self.assertTrue('1997-01-01' in values)
@@ -452,7 +479,7 @@ class TestUnit(TestCase):
         self.assertTrue(datetime.date(2020, 1, 1) in values)
         self.assertTrue(datetime.date(2020, 1, 2) in values)
 
-        post2 = index_many_set((idx1,  idx2), Index, union=True)
+        post2 = index_many_to_one((idx1,  idx2), Index, many_to_one_type=ManyToOneType.UNION)
         assert isinstance(post2, Index)
 
         self.assertEqual(post2.name, None)
@@ -462,7 +489,7 @@ class TestUnit(TestCase):
                 datetime.date(2020, 1, 2),
                 datetime.date(2020, 1, 3)])
 
-        post3 = index_many_set((idx1,  idx2), Index, union=False)
+        post3 = index_many_to_one((idx1,  idx2), Index, many_to_one_type=ManyToOneType.INTERSECT)
         assert isinstance(post3, Index)
 
         self.assertEqual(post3.name, None)
@@ -476,55 +503,127 @@ class TestUnit(TestCase):
         idx1 = IndexDate(('2020-01-01', '2020-01-02'), name='foo')
         idx2 = IndexDate(('2020-02-01', '2020-02-02'))
 
-        post1 = index_many_set((idx0,  idx1), IndexGO, union=True)
+        post1 = index_many_to_one((idx0,  idx1), IndexGO, many_to_one_type=ManyToOneType.UNION)
         self.assertEqual(post1.__class__, IndexGO)
 
-        post2 = index_many_set((idx1,  idx2), IndexGO, union=False)
+        post2 = index_many_to_one((idx1,  idx2), IndexGO, many_to_one_type=ManyToOneType.INTERSECT)
         self.assertEqual(post2.__class__, IndexDateGO)
 
     def test_index_many_set_c(self) -> None:
         idx1 = IndexDate(('2020-02-01', '2020-02-02'))
 
-        post1 = index_many_set((idx1,), Index, union=True)
+        post1 = index_many_to_one((idx1,), Index, many_to_one_type=ManyToOneType.UNION)
         self.assertEqual(post1.__class__, IndexDate)
         self.assertTrue(idx1.equals(post1))
 
         # empty iterable returns an empty index
-        post2 = index_many_set((), Index, union=True)
-        self.assertEqual(len(post2), 0) #type: ignore
+        post2 = index_many_to_one((), Index, many_to_one_type=ManyToOneType.UNION)
+        self.assertEqual(len(post2), 0)
 
     def test_index_many_set_d(self) -> None:
         idx1 = Index(range(3), loc_is_iloc=True)
         idx2 = Index(range(3), loc_is_iloc=True)
-        idx3 = index_many_set((idx1, idx2), Index, union=True)
+        idx3 = index_many_to_one((idx1, idx2), Index, many_to_one_type=ManyToOneType.UNION)
         self.assertTrue(idx3._map is None) #type: ignore
-        self.assertEqual(idx3.values.tolist(), [0, 1, 2]) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1, 2])
 
     def test_index_many_set_e(self) -> None:
         idx1 = Index(range(2), loc_is_iloc=True)
         idx2 = Index(range(4), loc_is_iloc=True)
-        idx3 = index_many_set((idx1, idx2), Index, union=True)
+        idx3 = index_many_to_one((idx1, idx2), Index, many_to_one_type=ManyToOneType.UNION)
         self.assertTrue(idx3._map is None) #type: ignore
-        self.assertEqual(idx3.values.tolist(), [0, 1, 2, 3]) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1, 2, 3])
 
     def test_index_many_set_f(self) -> None:
         idx1 = Index(range(2), loc_is_iloc=True)
         idx2 = Index(range(4), loc_is_iloc=True)
-        idx3 = index_many_set((idx1, idx2), Index, union=False)
+        idx3 = index_many_to_one((idx1, idx2), Index, many_to_one_type=ManyToOneType.INTERSECT)
         self.assertTrue(idx3._map is None) #type: ignore
-        self.assertEqual(idx3.values.tolist(), [0, 1]) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1])
 
     def test_index_many_set_g(self) -> None:
         idx1 = Index(range(2), loc_is_iloc=True)
         idx2 = Index([3, 2, 1, 0])
-        idx3 = index_many_set((idx1, idx2), Index, union=False)
+        idx3 = index_many_to_one((idx1, idx2), Index, many_to_one_type=ManyToOneType.INTERSECT)
         self.assertTrue(idx3._map is not None) #type: ignore
-        self.assertEqual(idx3.values.tolist(), [0, 1]) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1])
 
     def test_index_many_set_h(self) -> None:
-        post1 = index_many_set((), Index, union=True, explicit_constructor=IndexDate)
+        post1 = index_many_to_one((), Index, many_to_one_type=ManyToOneType.UNION, explicit_constructor=IndexDate)
         self.assertIs(post1.__class__, IndexDate)
 
+    def test_index_many_set_i(self) -> None:
+
+        idx1 = IndexHierarchy.from_product(('a', 'b'), (1, 2))
+        idx2 = IndexHierarchy.from_product(('a', 'b'), (1, 2))
+
+        post = index_many_to_one((idx1, idx2), Index, many_to_one_type=ManyToOneType.UNION)
+        post = tp.cast(IndexHierarchy, post)
+
+        self.assertEqual([d.kind for d in post.dtypes.values], ['U', 'i'])
+        self.assertEqual(post.values.tolist(),
+            [['a', 1], ['a', 2], ['b', 1], ['b', 2]])
+
+    def test_index_many_set_j(self) -> None:
+
+        idx1 = IndexHierarchy.from_product(('a', 'b'), (1, 2))
+        idx2 = IndexHierarchy.from_product(('a', 'c'), (1, 2))
+
+        post = index_many_to_one((idx1, idx2), Index, many_to_one_type=ManyToOneType.UNION)
+        post = tp.cast(IndexHierarchy, post)
+
+        self.assertEqual([d.kind for d in post.dtypes.values], ['U', 'i'])
+        self.assertEqual(post.values.tolist(),
+            [['a', 1], ['a', 2], ['b', 1], ['b', 2], ['c', 1], ['c', 2]])
+
+    def test_index_many_set_k(self) -> None:
+
+        idx1 = IndexHierarchy.from_product(('a', 'b'), (1, 2))
+        idx2 = IndexHierarchy.from_product(('a', 'c'), (1, 2))
+
+        post = index_many_to_one((idx1, idx2), Index, many_to_one_type=ManyToOneType.INTERSECT)
+        post = tp.cast(IndexHierarchy, post)
+
+        self.assertEqual([d.kind for d in post.dtypes.values], ['U', 'i'])
+        self.assertEqual(post.values.tolist(),
+            [['a', 1], ['a', 2]],)
+
+    def test_index_many_set_l(self) -> None:
+
+        idx0 = Index(('1997-01-01', '1997-01-02'), name='foo')
+        idx1 = IndexDate(('2020-01-01', '2020-01-02'), name='foo')
+        # idx2 = IndexDate(('2020-01-02', '2020-01-03'))
+
+        post1 = index_many_to_one((idx0,  idx1), Index, many_to_one_type=ManyToOneType.UNION)
+        assert isinstance(post1, Index)
+
+        self.assertEqual(post1.name, 'foo')
+        self.assertEqual(post1.__class__, Index)
+
+    def test_index_many_set_m(self) -> None:
+
+        idx0 = Index((2, 5, 6, 8), name='foo')
+        idx1 = Index((2, 8), name='foo')
+
+        idx2 = index_many_to_one((idx0,  idx1), Index, many_to_one_type=ManyToOneType.DIFFERENCE)
+
+        self.assertEqual(list(idx2), [5, 6])
+        self.assertEqual(idx2.name, 'foo')
+
+    def test_index_many_set_n(self) -> None:
+
+        idx0 = IndexDate(('1997-01-01', '1997-01-02', '2020-01-02'), name='foo')
+        idx1 = IndexDate(('1997-01-01', '2020-01-02'), name='foo')
+
+        post1 = index_many_to_one((idx0,  idx1), Index, many_to_one_type=ManyToOneType.DIFFERENCE)
+
+        self.assertEqual(post1.name, 'foo')
+        self.assertEqual(post1.__class__, IndexDate)
+        self.assertEqual(list(post1), [np.datetime64('1997-01-02')])
+
+
+
+
     #---------------------------------------------------------------------------
 
     def test_get_col_dtype_factory_a(self) -> None:
@@ -544,8 +643,84 @@ class TestUnit(TestCase):
         self.assertEqual(func3(0), None)
         self.assertEqual(func3(1), np.dtype(bool))
 
-        with self.assertRaises(RuntimeError):
-            _ = get_col_dtype_factory(dict(bar=np.dtype(bool)), None)
+    def test_get_col_dtype_factory_b(self) -> None:
+        func = get_col_dtype_factory({1: np.dtype(bool)}, None)
+        self.assertEqual(func(0), None)
+        self.assertEqual(func(1), np.dtype(bool))
+
+
+
+    #---------------------------------------------------------------------------
+
+    def test_get_col_fill_value_a(self) -> None:
+        func1 = get_col_fill_value_factory({'a':-1, 'b':2}, columns=('b', 'a'))
+        self.assertEqual(func1(0, np.dtype(float)), 2)
+        self.assertEqual(func1(1, np.dtype(float)), -1)
+
+    def test_get_col_fill_value_b(self) -> None:
+        func1 = get_col_fill_value_factory(('x', 1), columns=('b', 'a'))
+        self.assertEqual(func1(0, np.dtype(float)), ('x', 1))
+        self.assertEqual(func1(1, np.dtype(float)), ('x', 1))
+
+    def test_get_col_fill_value_c(self) -> None:
+        func1 = get_col_fill_value_factory(('x', 1), columns=('b', 'a'))
+        self.assertEqual(func1(0, np.dtype(float)), ('x', 1))
+        self.assertEqual(func1(1, np.dtype(float)), ('x', 1))
+
+    def test_get_col_fill_value_d(self) -> None:
+        func1 = get_col_fill_value_factory('x', columns=('b', 'a'))
+        self.assertEqual(func1(0, np.dtype(float)), 'x')
+        self.assertEqual(func1(1, np.dtype(float)), 'x')
+
+    def test_get_col_fill_value_e(self) -> None:
+        func1 = get_col_fill_value_factory(
+                (c for c in 'xy'),
+                columns=('b', 'a'),
+                )
+        self.assertEqual(func1(0, np.dtype(float)), 'x')
+        self.assertEqual(func1(1, np.dtype(float)), 'y')
+
+    def test_get_col_fill_value_f(self) -> None:
+        func1 = get_col_fill_value_factory(FillValueAuto, columns=None)
+        self.assertEqual(func1(0, np.dtype(object)), None)
+        self.assertEqual(func1(1, np.dtype(str)), '')
+
+    def test_get_col_fill_value_g(self) -> None:
+        func1 = get_col_fill_value_factory(FillValueAuto(O='', U='na'), columns=None)
+        self.assertEqual(func1(0, np.dtype(object)), '')
+        self.assertEqual(func1(1, np.dtype(str)), 'na')
+
+    #---------------------------------------------------------------------------
+
+    def test_get_col_format_a(self) -> None:
+        func = get_col_format_factory('{}', ('a', 'b', 'c'))
+        self.assertEqual(func(0), '{}')
+        self.assertEqual(func(1), '{}')
+        self.assertEqual(func(2), '{}')
+
+    def test_get_col_format_b(self) -> None:
+        func = get_col_format_factory(('a', 'b', 'c'), ('a', 'b', 'c'))
+        self.assertEqual(func(0), 'a')
+        self.assertEqual(func(1), 'b')
+        self.assertEqual(func(2), 'c')
+
+    def test_get_col_format_c(self) -> None:
+        func = get_col_format_factory(('a', 'b', 'c'), range(3))
+        self.assertEqual(func(0), 'a')
+        self.assertEqual(func(1), 'b')
+        self.assertEqual(func(2), 'c')
+
+    def test_get_col_format_d1(self) -> None:
+        func = get_col_format_factory({'b':'x{}', 'c':'y{}'}, ('a', 'b', 'c'))
+        self.assertEqual(func(0), '{}')
+        self.assertEqual(func(1), 'x{}')
+        self.assertEqual(func(2), 'y{}')
+
+    def test_get_col_format_e(self) -> None:
+        func = get_col_format_factory((f'{{:{i}}}' for i in range(3)), ('a', 'b', 'c'))
+        self.assertEqual(func(0), '{:0}')
+        self.assertEqual(func(1), '{:1}')
+        self.assertEqual(func(2), '{:2}')
 
     #---------------------------------------------------------------------------
 
@@ -697,6 +872,58 @@ class TestUnit(TestCase):
         self.assertEqual([a.shape for a in stack],
                 [(2, 1)])
 
+    #---------------------------------------------------------------------------
+    def test_group_from_container_a(self) -> None:
+        idx = Index(('a', 'b', 'c'))
+        with self.assertRaises(ValueError):
+            group_from_container(idx, np.arange(12).reshape(3, 2, 2), None, 0)
+
+    def test_group_from_container_b(self) -> None:
+        idx = Index(('a', 'b', 'c'))
+        with self.assertRaises(ValueError):
+            group_from_container(idx, 'a', None, 0)
+
+    def test_group_from_container_c(self) -> None:
+        idx = Index(('a', 'b', 'c'))
+        with self.assertRaises(RuntimeError):
+            group_from_container(idx, (2, 2), None, 0)
+
+    def test_group_from_container_d(self) -> None:
+        idx = Index(('a', 'b', 'c'))
+        gs = np.arange(6).reshape(3, 2)
+        group_from_container(idx, gs, None, 0)
+        # if wrong axis does not align
+        with self.assertRaises(RuntimeError):
+            group_from_container(idx, gs, None, 1)
+
+    def test_group_from_container_e(self) -> None:
+        idx = Index(('a', 'b', 'c'))
+        s = Frame.from_element(0, index=('a', 'c'), columns=('x',))
+        post = group_from_container(idx, s, None, 0)
+        self.assertEqual(post.tolist(), [[0], [None], [0]])
+
+
+    def test_get_containers(self) -> None:
+        keys_gc = set(cls.__name__ for cls in TestCase.get_containers())
+        keys_cm = set(ContainerMap.keys())
+
+        # these two different utilities have slightly different constituents as they are used for different purposes
+        self.assertEqual(keys_gc - keys_cm, {'ContainerOperand', 'ContainerOperandSequence'})
+        self.assertEqual(keys_cm - keys_gc, {'FillValueAuto', 'ILoc', 'HLoc', 'MemoryDisplay'})
+
+        self.assertEqual(keys_cm & keys_gc, {'FrameHE', 'IndexSecondGO', 'IndexSecond', 'IndexDateGO', 'Bus', 'IndexMinute', 'Index', 'Frame', 'IndexDate', 'IndexYearMonth', 'IndexYearGO', 'IndexMicrosecondGO', 'Yarn', 'IndexNanosecond', 'IndexYearMonthGO', 'IndexNanosecondGO', 'IndexHourGO', 'Batch', 'Quilt', 'IndexMinuteGO', 'FrameGO', 'IndexHour', 'Series', 'IndexGO', 'IndexHierarchy', 'IndexMillisecondGO', 'TypeBlocks', 'IndexYear', 'SeriesHE', 'IndexMicrosecond', 'IndexMillisecond', 'IndexHierarchyGO'})
+
+
+
+    def test_get_container_map_a(self) -> None:
+        if hasattr(ContainerMap, '_map'):
+            delattr(ContainerMap, '_map')
+        keys = set(ContainerMap.keys())
+
+        delattr(ContainerMap, '_map')
+        for k in keys:
+            ContainerMap.get(k)
+
 
 if __name__ == '__main__':
     import unittest
