@@ -1,37 +1,36 @@
-from itertools import combinations
-import unittest
-import string
-import typing as tp
+from __future__ import annotations
+
 import hashlib
 import json
+import string
+import typing as tp
+import unittest
+from itertools import combinations
 
-import numpy as np
 import frame_fixtures as ff
-
-from static_frame.test.test_case import TestCase
-from static_frame.test.test_case import skip_win
+import numpy as np
 
 import static_frame as sf
 # assuming located in the same directory
-from static_frame import Index
-from static_frame import IndexHierarchy
-from static_frame import Series
-from static_frame import Frame
-from static_frame import FrameGO
 from static_frame import Display
+from static_frame import DisplayActive
 from static_frame import DisplayConfig
 from static_frame import DisplayConfigs
 from static_frame import DisplayFormats
-from static_frame import DisplayActive
-from static_frame.core.display_color import HexColor
-
-from static_frame.core.display_config import DisplayFormatLaTeX
-from static_frame.core.display import DisplayTypeCategoryFactory
-from static_frame.core.display import terminal_ansi
-from static_frame.core.display import DisplayCell
+from static_frame import Frame
+from static_frame import FrameGO
+from static_frame import Index
+from static_frame import IndexHierarchy
+from static_frame import Series
 from static_frame.core.display import FORMAT_EMPTY
+from static_frame.core.display import DisplayCell
+from static_frame.core.display import DisplayTypeCategoryFactory
 from static_frame.core.display import DisplayTypeInt
-
+from static_frame.core.display import terminal_ansi
+from static_frame.core.display_color import HexColor
+from static_frame.core.display_config import DisplayFormatLaTeX
+from static_frame.test.test_case import TestCase
+from static_frame.test.test_case import skip_win
 from static_frame.test.test_case import temp_file
 
 nan = np.nan
@@ -81,7 +80,7 @@ class TestUnit(TestCase):
 
     #---------------------------------------------------------------------------
 
-    @skip_win  #type: ignore
+    @skip_win
     def test_display_active_a(self) -> None:
 
         fp1 = DisplayActive._default_fp()
@@ -105,7 +104,7 @@ class TestUnit(TestCase):
                 index.display(config=config_right).to_rows(),
                 ['<Index>', '      a', '      b', '      c', '  <<U1>'])
 
-    @skip_win  #type: ignore
+    @skip_win
     def test_display_cell_align_left_b(self) -> None:
         config_right = sf.DisplayConfig.from_default(cell_align_left=False, type_color=False)
         config_left = sf.DisplayConfig.from_default(cell_align_left=True, type_color=False)
@@ -147,7 +146,7 @@ class TestUnit(TestCase):
                 '      y       2      95     c  False  False',
                 '  <<U1> <int64> <int64> <<U1> <bool> <bool>'])
 
-    @skip_win  # type: ignore
+    @skip_win
     def test_display_type_show_a(self) -> None:
         config_type = sf.DisplayConfig.from_default(
                 type_show=True,
@@ -196,7 +195,7 @@ class TestUnit(TestCase):
                 'y       2       95      c     False  False',
                 '<<U1>   <int64> <int64> <<U1> <bool> <bool>'])
 
-    @skip_win  # type: ignore
+    @skip_win
     def test_display_type_show_b(self) -> None:
         config_type = sf.DisplayConfig.from_default(
                 type_show=True,
@@ -234,7 +233,7 @@ class TestUnit(TestCase):
                 ['p   q  r s     t', '2   2  a False False', '30  34 b True  False', '2   95 c False False', '300 -4 x False False']
                 )
 
-    @skip_win #type: ignore
+    @skip_win
     def test_display_cell_fill_width_a(self) -> None:
 
         config_width_12 = sf.DisplayConfig.from_default(cell_max_width=12, cell_max_width_leftmost=12, type_color=False)
@@ -305,7 +304,6 @@ class TestUnit(TestCase):
         index = list(''.join(x) for x in combinations(string.ascii_lowercase, 2))
         s = Series(range(len(index)), index=index, dtype=np.int64)
 
-        # import ipdb; ipdb.set_trace()
         self.assertEqual(s.display(config_rows_12).to_rows(),
                 [
                 '<Series>',
@@ -382,7 +380,7 @@ class TestUnit(TestCase):
                 '<int64> <int64>']
                 )
 
-    @skip_win  # type: ignore
+    @skip_win
     def test_display_display_columns_a(self) -> None:
 
         config_columns_8 = sf.DisplayConfig.from_default(display_columns=8, type_color=False)
@@ -416,7 +414,7 @@ class TestUnit(TestCase):
                 '3         0       1       ... 323     324',
                 '<int64>   <int64> <int64> ... <int64> <int64>'])
 
-    @skip_win  # type: ignore
+    @skip_win
     def test_display_display_columns_b(self) -> None:
 
         config_columns_4 = sf.DisplayConfig.from_default(display_columns=4, type_color=False)
@@ -541,7 +539,7 @@ class TestUnit(TestCase):
         self.assertEqual(d1.transform().to_rows(),
                 ['header 1 2 3 4 <int64>', 'header 5 6 7 8 <int64>'])
 
-    @skip_win  # type: ignore
+    @skip_win
     def test_display_html_pre_a(self) -> None:
         f = Frame.from_dict(dict(a=(1, 2),
                 b=(1.2, 3.4),
@@ -560,7 +558,7 @@ class TestUnit(TestCase):
 
         self.assertEqualLines(html, str(expected))
 
-    @skip_win  # type: ignore
+    @skip_win
     def test_display_html_table_a(self) -> None:
         f = sf.Frame.from_dict(
             dict(a=(1,2,3,4), b=(True, False, True, False), c=list('qrst')))
@@ -766,7 +764,7 @@ class TestUnit(TestCase):
             '''\\begin{table}[ht]\n\\centering\nx\n\\label{table:foo}\n\\end{table}'''
             )
 
-    @skip_win #type: ignore
+    @skip_win
     def test_display_type_color_markup_a(self) -> None:
 
         config1 = DisplayConfig(display_format=DisplayFormats.TERMINAL)
