@@ -1,18 +1,21 @@
+from __future__ import annotations
+
 import typing as tp
 
-import numpy as np
 import frame_fixtures as ff
+import numpy as np
 
 import static_frame as sf
-from static_frame import IndexHierarchy
-from static_frame import IndexDate
-from static_frame import Series
 from static_frame import Frame
 from static_frame import FrameGO
-from static_frame import TypeBlocks
 from static_frame import HLoc
-from static_frame.test.test_case import TestCase
+from static_frame import IndexDate
+from static_frame import IndexHierarchy
+from static_frame import IndexYear
+from static_frame import Series
+from static_frame import TypeBlocks
 from static_frame.core.exception import AxisInvalid
+from static_frame.test.test_case import TestCase
 
 nan = np.nan
 
@@ -33,7 +36,7 @@ class TestUnit(TestCase):
 
         self.assertEqual((f1.keys() == f1.columns).all(), True)
         self.assertEqual([x for x in f1.columns], ['p', 'q', 'r', 's', 't'])
-        self.assertEqual([x for x in f1], ['p', 'q', 'r', 's', 't'])
+        self.assertEqual([x for x in f1], ['p', 'q', 'r', 's', 't']) # type: ignore
 
     def test_frame_iter_array_a(self) -> None:
 
@@ -46,11 +49,11 @@ class TestUnit(TestCase):
                 index=('x','y'))
 
         self.assertEqual(
-                next(iter(f1.iter_array(axis=0))).tolist(),
+                next(iter(f1.iter_array(axis=0))).tolist(), # type: ignore
                 [1, 30])
 
         self.assertEqual(
-                next(iter(f1.iter_array(axis=1))).tolist(),
+                next(iter(f1.iter_array(axis=1))).tolist(), # type: ignore
                 [1, 2, 'a', False, True])
 
     def test_frame_iter_array_b(self) -> None:
@@ -111,7 +114,7 @@ class TestUnit(TestCase):
                 dtypes=dict(diameter=np.int64))
 
         post = f.iter_array(axis=0).apply(np.sum)
-        self.assertTrue(post.dtype == float)
+        self.assertTrue(post.dtype == float) # type: ignore
 
     def test_frame_iter_array_f(self) -> None:
 
@@ -186,6 +189,91 @@ class TestUnit(TestCase):
         self.assertEqual(post2,
                 [(1, 30), (2, 50), ('a', 'b'), (False, True), (True, False)])
 
+    def test_frame_iter_tuple_f(self) -> None:
+        f = ff.parse('s(3,2)|v(dtD,dtY)')
+        post = tuple(f.iter_tuple(constructor=tuple, axis=1))
+        self.assertEqual(len(post[0]), 2)
+        self.assertEqual(post[0][1].dtype, f.iloc[0, 1].dtype)
+        self.assertEqual(post[0],
+            (np.datetime64('2210-12-26'), np.datetime64('164167'))
+            )
+
+    def test_frame_iter_tuple_g(self) -> None:
+        # NOTE: this test demonstrate the utility of mapping functions on the only iterable axis type (tuple, ignoring SeriesHE) that is hashable
+        columns = tuple('pqrs')
+        index = tuple('zxwy')
+        records = (('A', 1,  False, False),
+                   ('A', 2,  True, False),
+                   ('B', 1,  False, False),
+                   ('B', 2,  True, True))
+        f = Frame.from_records(records, columns=columns, index=index)
+        post = f[['r', 's']].iter_tuple(axis=1, constructor=tuple).map_any({(False, False): None}) # type: ignore
+        self.assertEqual(post.to_pairs(),
+                (('z', None), ('x', (True, False)), ('w', None), ('y', (True, True)))
+                )
+
+    def test_frame_iter_tuple_h(self) -> None:
+        # NOTE: this test demonstrate the utility of mapping functions on the only iterable axis type (tuple, ignoring SeriesHE) that is hashable
+        columns = tuple('pqrs')
+        index = tuple('zxwy')
+        records = (('A', 1,  False, False),
+                   ('A', 2,  True, False),
+                   ('B', 1,  False, False),
+                   ('B', 2,  True, True))
+        f = Frame.from_records(records, columns=columns, index=index)
+        post = f[['r', 's']].iter_tuple_items(axis=1, constructor=tuple).map_fill ({('w', (False, False)): None}, fill_value=0) # type: ignore
+        self.assertEqual(post.to_pairs(),
+            (('z', 0), ('x', 0), ('w', None), ('y', 0))
+            )
+
+    def test_frame_iter_tuple_i1(self) -> None:
+        from dataclasses import dataclass
+
+        records = (
+                (1,  False, True),
+                (30, True, False))
+
+        f1 = FrameGO.from_records(records,
+                columns=('p', 'q', 'r'),
+                index=('x','y'))
+
+        @dataclass
+        class Record:
+            p: int
+            q: bool
+            r: bool
+
+        post1 = list(f1.iter_tuple(axis=1, constructor=Record))
+        self.assertTrue(all(isinstance(x, Record) for x in post1))
+        self.assertEqual([x.p for x in post1], [1, 30])
+        self.assertEqual([x.r for x in post1], [True, False])
+
+    def test_frame_iter_tuple_i2(self) -> None:
+        from dataclasses import dataclass
+
+        records = (
+                (1,  False, True),
+                (30, True, False))
+
+        f1 = FrameGO.from_records(records,
+                columns=('p', 'q', 'r'),
+                index=('x','y'))
+
+        @dataclass
+        class Record:
+            p: int
+            q: bool
+            r: bool
+
+        ctor = lambda args: Record(**dict(zip(('p', 'q', 'r'), args)))
+        post1 = list(f1.iter_tuple(axis=1, constructor=ctor))
+        self.assertTrue(all(isinstance(x, Record) for x in post1))
+        self.assertEqual([x.p for x in post1], [1, 30])
+        self.assertEqual([x.r for x in post1], [True, False])
+
+
+
+
     #---------------------------------------------------------------------------
 
     def test_frame_iter_series_a(self) -> None:
@@ -224,6 +312,18 @@ class TestUnit(TestCase):
         self.assertEqual(post5.dtype, object)
         self.assertEqual(post5.shape, (10,))
 
+    #---------------------------------------------------------------------------
+    def test_frame_iter_series_items_a(self) -> None:
+        f1 = ff.parse('f(Fg)|s(2,8)|i(I,str)|c(Ig,str)|v(int)')
+        post1 = tuple(f1.iter_series_items(axis=0))
+        self.assertEqual([(k, v.values.tolist()) for (k, v) in post1],
+                [('zZbu', [-88017, 92867]), ('ztsv', [162197, -41157]), ('zUvW', [-3648, 91301]), ('zkuW', [129017, 35021]), ('zmVj', [58768, 146284]), ('z2Oo', [84967, 13448]), ('z5l6', [146284, 170440]), ('zCE3', [137759, -62964])])
+
+        post2 = tuple(f1.iter_series_items(axis=1))
+        self.assertEqual([(k, v.values.tolist()) for (k, v) in post2],
+                [('zZbu', [-88017, 162197, -3648, 129017, 58768, 84967, 146284, 137759]), ('ztsv', [92867, -41157, 91301, 35021, 146284, 13448, 170440, -62964])]
+                )
+
     #---------------------------------------------------------------------------
 
     def test_frame_iter_tuple_items_a(self) -> None:
@@ -352,6 +452,36 @@ class TestUnit(TestCase):
         self.assertEqual([d.kind for d in f2.dtypes.values],
                 ['i', 'i', 'i'])
 
+    def test_frame_iter_element_f(self) -> None:
+        f1 = Frame.from_records(np.arange(9).reshape(3, 3))
+
+        mapping = {x: x*3 for x in range(3)}
+        f2 = f1.iter_element(axis=1).map_fill(mapping, fill_value=-1)
+        self.assertEqual(f2.to_pairs(),
+            ((0, ((0, 0), (1, -1), (2, -1))), (1, ((0, 3), (1, -1), (2, -1))), (2, ((0, 6), (1, -1), (2, -1))))
+            )
+
+    def test_frame_iter_element_g1(self) -> None:
+        f1 = Frame(np.arange(9).reshape(3, 3)).rename(index='a', columns='b')
+        f2 = f1.iter_element().apply(str)
+        self.assertEqual(f1.columns.name, f2.columns.name)
+        self.assertEqual(f1.index.name, f2.index.name)
+
+    def test_frame_iter_element_g2(self) -> None:
+        f1 = Frame(np.arange(9).reshape(3, 3), columns=('2021', '1943', '1523')).rename(index='a', columns='b')
+        f2 = f1.iter_element().apply(str, columns_constructor=IndexYear)
+        self.assertIs(f2.columns.__class__, IndexYear)
+        self.assertIs(f2.columns.name, 'b')
+        self.assertEqual(f1.shape, f2.shape)
+
+    def test_frame_iter_element_g3(self) -> None:
+        f1 = Frame(np.arange(9).reshape(3, 3), index=('2021', '1943', '1523')).rename(index='a', columns='b')
+        f2 = f1.iter_element().apply(str, index_constructor=IndexYear)
+        self.assertIs(f2.index.__class__, IndexYear)
+        self.assertIs(f2.index.name, 'a')
+        self.assertEqual(f1.shape, f2.shape)
+
+
     #---------------------------------------------------------------------------
 
     def test_frame_iter_group_a(self) -> None:
@@ -402,12 +532,12 @@ class TestUnit(TestCase):
                 records, columns=columns, index=index, name='foo')
 
         with self.assertRaises(TypeError):
-            next(iter(f.iter_group(foo='x')))
+            next(iter(f.iter_group(foo='x'))) # type: ignore
 
         with self.assertRaises(TypeError):
             next(iter(f.iter_group(3, 5)))
 
-        self.assertEqual(next(iter(f.iter_group('q'))).to_pairs(0),
+        self.assertEqual(next(iter(f.iter_group('q'))).to_pairs(0), # type: ignore
                 (('p', (('z', 'A'), ('w', 'B'))), ('q', (('z', 1), ('w', 1))), ('r', (('z', 'a'), ('w', 'c'))), ('s', (('z', False), ('w', False))), ('t', (('z', False), ('w', False))))
                 )
 
@@ -446,6 +576,58 @@ class TestUnit(TestCase):
             ((np.datetime64('1998-10-12'), 1),
             (np.datetime64('1998-10-13'), 2)))
 
+    def test_frame_iter_group_h(self) -> None:
+        f1 = sf.Frame(np.arange(25).reshape(5, 5), index=tuple("ABCDE"))
+
+        with self.assertRaises(KeyError):
+            f1.iter_group(-99).apply(lambda x: x)
+
+        with self.assertRaises(KeyError):
+            s2 = f1.iter_group('z').apply(lambda x: x)
+
+
+
+    #---------------------------------------------------------------------------
+    def test_frame_iter_group_array_a(self) -> None:
+        f1 = ff.parse('s(7,3)|v(int)').assign[1].apply(
+                lambda s: s % 3)
+
+        post = tuple(f1.iter_group_array(1))
+        self.assertEqual([p.shape for p in post],
+                [(3, 3), (4, 3)]
+                )
+        self.assertEqual([p.__class__ for p in post],
+                [np.ndarray, np.ndarray]
+                )
+
+    def test_frame_iter_group_array_b(self) -> None:
+        f1 = ff.parse('s(7,3)|v(int)').assign[1].apply(
+                lambda s: s % 3)
+
+        post = tuple(f1.iter_group_array(1, drop=True))
+        self.assertEqual([p.shape for p in post],
+                [(3, 2), (4, 2)]
+                )
+        self.assertEqual([p.__class__ for p in post],
+                [np.ndarray, np.ndarray]
+                )
+
+    #---------------------------------------------------------------------------
+    def test_frame_iter_group_array_items_a(self) -> None:
+        f1 = ff.parse('s(7,3)|v(int)').assign[1].apply(
+                lambda s: s % 3)
+
+        post = tuple(f1.iter_group_array_items(1))
+        self.assertEqual([p[1].shape for p in post],
+                [(3, 3), (4, 3)]
+                )
+        self.assertEqual([p[1].__class__ for p in post],
+                [np.ndarray, np.ndarray]
+                )
+        self.assertEqual([p[0] for p in post],
+                [0, 2]
+                )
+
     #---------------------------------------------------------------------------
 
     def test_frame_iter_group_items_a(self) -> None:
@@ -646,7 +828,7 @@ class TestUnit(TestCase):
 
     #---------------------------------------------------------------------------
 
-    def test_frame_iter_group_index_a(self) -> None:
+    def test_frame_iter_group_labels_a(self) -> None:
 
         records = (
                 (2, 2, 'a', False, False),
@@ -661,8 +843,7 @@ class TestUnit(TestCase):
             f1.iter_group_labels(3, 4)
 
         with self.assertRaises(TypeError):
-            f1.iter_group_labels(foo=4)
-
+            f1.iter_group_labels(foo=4) # type: ignore
 
         post = tuple(f1.iter_group_labels(0, axis=0))
 
@@ -672,7 +853,7 @@ class TestUnit(TestCase):
                 (('x', 4), ('y', 64), ('z', 97))
                 )
 
-    def test_frame_iter_group_index_b(self) -> None:
+    def test_frame_iter_group_labels_b(self) -> None:
 
         records = (
                 (2, 2, 'a', 'q', False, False),
@@ -685,7 +866,6 @@ class TestUnit(TestCase):
 
         # with axis 1, we are grouping based on columns while maintain the index
         post_tuple = tuple(f1.iter_group_labels(1, axis=1))
-
         self.assertEqual(len(post_tuple), 2)
 
         post = f1[HLoc[f1.columns[0]]]
@@ -705,7 +885,7 @@ class TestUnit(TestCase):
                 (('a', 34), ('b', 131))
                 )
 
-    def test_frame_iter_group_index_c(self) -> None:
+    def test_frame_iter_group_labels_c(self) -> None:
         columns = tuple('pqrst')
         index = tuple('zxwy')
         records = (('A', 1, 'a', False, False),
@@ -726,6 +906,303 @@ class TestUnit(TestCase):
                 (('A', 'A:6'), ('B', 'B:6'))
         )
 
+    def test_frame_iter_group_labels_d(self) -> None:
+        columns = tuple('pqrst')
+        index = tuple('zxwy')
+        records = (('A', 1, 'a', False, 4),
+                   ('A', 2, 'b', True, 3),
+                   ('B', 1, 'c', False, 2),
+                   ('B', 2, 'd', True, 1))
+
+        f = Frame.from_records(
+                records, columns=columns, index=index,name='foo')
+        f = f.set_index_hierarchy(('p', 'q', 'r'), drop=True)
+
+        post = tuple(f.iter_group_labels_items((1, 2), axis=0))
+        self.assertEqual([p[0] for p in post],
+                [(1, 'a'), (1, 'c'), (2, 'b'), (2, 'd')]
+                )
+
+        self.assertEqual([p[1].values.tolist() for p in post],
+                [[[False, 4]], [[False, 2]], [[True, 3]], [[True, 1]]]
+                )
+
+    def test_frame_iter_group_labels_e(self) -> None:
+        index = tuple('pq')
+        columns = IndexHierarchy._from_type_blocks(TypeBlocks.from_blocks((
+                np.array(('A', 'A', 'B', 'B', 'B')),
+                np.array((4, 2, 1, 0, 4)),
+                np.array(('b', 'a', 'c', 'a', 'b')),
+                )))
+
+        records = (
+                   (True, False, 1, 2, 1),
+                   (False, True, 30, 8,7),
+                   )
+
+        f = Frame.from_records(
+                records, columns=columns, index=index)
+        post = tuple(f.iter_group_labels_items((2, 1), axis=1))
+        self.assertEqual([p[0] for p in post],
+                [('a', 0), ('a', 2), ('b', 4), ('c', 1)]
+                )
+        self.assertEqual([p[1].values.tolist() for p in post],
+                [[[2], [8]], [[False], [True]], [[True, 1], [False, 7]], [[1], [30]]]
+                )
+
+    def test_frame_iter_group_labels_f(self) -> None:
+        columns = tuple('pqrst')
+        index = tuple('zxwy')
+        records = (('A', 1, 'a', False, 4),
+                   (None, 2, 'b', True, 3),
+                   ('B', 1, 'c', False, 2),
+                   ('B', None, 'd', True, 1))
+
+        f = Frame.from_records(
+                records, columns=columns, index=index,name='foo')
+        f = f.set_index_hierarchy(('p', 'q'), drop=True)
+
+        post = tuple(f.iter_group_labels_items(0, axis=0))
+        self.assertEqual([p[0] for p in post],
+                ['A', None, 'B']
+                )
+        self.assertEqual([p[1].values.tolist() for p in post],
+            [[['a', False, 4]], [['b', True, 3]], [['c', False, 2], ['d', True, 1]]]
+            )
+
+    #---------------------------------------------------------------------------
+
+    def test_frame_iter_group_labels_array_a(self) -> None:
+        columns = tuple('pqrs')
+        index = tuple('zxwy')
+        records = (('A', 1,  False, False),
+                   ('A', 2,  True, False),
+                   ('B', 1,  False, False),
+                   ('B', 2,  True, True))
+
+        f = Frame.from_records(records, columns=columns, index=index)
+        f = f.set_index_hierarchy(('p', 'q'), drop=True)
+
+        post1 = tuple(f.iter_group_labels_array(0))
+        self.assertEqual(len(post1), 2)
+        self.assertEqual([a.__class__ for a in post1], [np.ndarray, np.ndarray])
+        self.assertEqual([a.shape for a in post1], [(2, 2), (2, 2)])
+
+    #---------------------------------------------------------------------------
+
+    def test_frame_iter_group_labels_array_items_a(self) -> None:
+        columns = tuple('pqrs')
+        index = tuple('zxwy')
+        records = (('A', 1,  False, False),
+                   ('A', 2,  True, False),
+                   ('B', 1,  False, False),
+                   ('B', 2,  True, True))
+
+        f = Frame.from_records(records, columns=columns, index=index)
+        f = f.set_index_hierarchy(('p', 'q'), drop=True)
+
+        post1 = tuple(f.iter_group_labels_array_items(0))
+        self.assertEqual(len(post1), 2)
+        self.assertEqual([a[1].__class__ for a in post1], [np.ndarray, np.ndarray])
+        self.assertEqual([a[1].shape for a in post1], [(2, 2), (2, 2)])
+        self.assertEqual([a[0] for a in post1], ['A', 'B'])
+
+    def test_frame_iter_group_labels_array_items_b(self) -> None:
+        f = sf.Frame(np.arange(8).reshape(2, 4),
+                columns=sf.IndexHierarchy.from_labels(
+                        ((1, 'a'), (2, 'b'), (1, 'b'), (2, 'a'))
+                ))
+        post1 = tuple(f.iter_group_labels_array_items(1, axis=1))
+        self.assertEqual(len(post1), 2)
+        self.assertEqual([a[1].__class__ for a in post1], [np.ndarray, np.ndarray])
+        self.assertEqual([a[1].shape for a in post1], [(2, 2), (2, 2)])
+        self.assertEqual([a[0] for a in post1], ['a', 'b'])
+
+    #---------------------------------------------------------------------------
+
+    def test_frame_iter_group_other_a(self) -> None:
+        columns = tuple('pqr')
+        index = tuple('zxwy')
+        records = (('A', 1, False),
+                   ('A', 2, True),
+                   ('B', 1, False),
+                   ('B', 2, True))
+        f = Frame.from_records(records, columns=columns, index=index)
+        # asxis 0 means for other grouping means that they key is a column key
+        post1, post2 = list(f.iter_group_other(axis=0, other=(0, 0, 0, 1)))
+        self.assertEqual(post1.to_pairs(),
+                (('p', (('z', 'A'), ('x', 'A'), ('w', 'B'))), ('q', (('z', 1), ('x', 2), ('w', 1))), ('r', (('z', False), ('x', True), ('w', False))))
+                )
+        self.assertEqual(post2.to_pairs(),
+                (('p', (('y', 'B'),)), ('q', (('y', 2),)), ('r', (('y', True),)))
+                )
+
+        post3, post4 = list(f.iter_group_other(axis=1, other=(1, 0, 1)))
+        self.assertEqual(post3.to_pairs(),
+                (('q', (('z', 1), ('x', 2), ('w', 1), ('y', 2))),)
+                )
+        self.assertEqual(post4.to_pairs(),
+                (('p', (('z', 'A'), ('x', 'A'), ('w', 'B'), ('y', 'B'))), ('r', (('z', False), ('x', True), ('w', False), ('y', True))))
+                )
+
+    def test_frame_iter_group_other_b(self) -> None:
+        columns = tuple('pqr')
+        index = tuple('zxwy')
+        records = (('A', 1, False),
+                   ('A', 2, True),
+                   ('B', 1, False),
+                   ('B', 2, True))
+        f = Frame.from_records(records, columns=columns, index=index)
+        # axis 0 means for other grouping means that they key is a column key
+        post1, post2 = list(f.iter_group_other(axis=0, other=f[['q', 'r']]))
+        self.assertEqual(post1.to_pairs(),
+                (('p', (('z', 'A'), ('w', 'B'))), ('q', (('z', 1), ('w', 1))), ('r', (('z', False), ('w', False))))
+                )
+        self.assertEqual(post2.to_pairs(),
+                (('p', (('x', 'A'), ('y', 'B'))), ('q', (('x', 2), ('y', 2))), ('r', (('x', True), ('y', True))))
+                )
+
+    def test_frame_iter_group_other_c(self) -> None:
+        columns = tuple('pqr')
+        index = tuple('zxwy')
+        records = ((1, 1, False),
+                   (1, 1, False),
+                   (5, 1, False),
+                   (8, 2, True))
+        f = Frame.from_records(records, columns=columns, index=index)
+        # axis 0 means for other grouping means that they key is a column key
+        (g1, post1), (g2, post2) = list(f.iter_group_other_items(axis=1,
+                other=f.iloc[:2, :2],
+                fill_value = None
+                ))
+        self.assertEqual(g1, (1, 1))
+        self.assertEqual(post1.to_pairs(),
+                (('p', (('z', 1), ('x', 1), ('w', 5), ('y', 8))), ('q', (('z', 1), ('x', 1), ('w', 1), ('y', 2))))
+                )
+        self.assertEqual(g2, (None, None))
+        self.assertEqual(post2.to_pairs(),
+                (('r', (('z', False), ('x', False), ('w', False), ('y', True))),)
+                )
+
+    def test_frame_iter_group_other_d(self) -> None:
+        columns = tuple('pqr')
+        index = tuple('zxwy')
+        records = ((1, 1, False),
+                   (1, 1, False),
+                   (5, 1, False),
+                   (8, 2, True))
+        f = Frame.from_records(records, columns=columns, index=index)
+        post1, post2 = list(f.iter_group_other(axis=0,
+                other=np.array([[1, 1], [0, 0], [1, 1], [0, 0]]),
+                fill_value = None
+                ))
+        self.assertEqual(post1.to_pairs(),
+                (('p', (('x', 1), ('y', 8))), ('q', (('x', 1), ('y', 2))), ('r', (('x', False), ('y', True))))
+                )
+        self.assertEqual(post2.to_pairs(),
+                (('p', (('z', 1), ('w', 5))), ('q', (('z', 1), ('w', 1))), ('r', (('z', False), ('w', False))))
+                )
+
+    #---------------------------------------------------------------------------
+    def test_frame_iter_group_other_items_a(self) -> None:
+        columns = tuple('pqr')
+        index = tuple('zxwy')
+        records = (('A', 1, False),
+                   ('A', 2, True),
+                   ('B', 1, False),
+                   ('B', 2, True))
+        f = Frame.from_records(records, columns=columns, index=index)
+        # asxis 0 means for other grouping means that they key is a column key
+        (g1, post1), (g2, post2) = list(f.iter_group_other_items(
+                axis=0, other=(0, 0, 0, 1))
+                )
+
+        self.assertEqual(post1.to_pairs(),
+                (('p', (('z', 'A'), ('x', 'A'), ('w', 'B'))), ('q', (('z', 1), ('x', 2), ('w', 1))), ('r', (('z', False), ('x', True), ('w', False))))
+                )
+        self.assertEqual(g1, 0)
+        self.assertEqual(post2.to_pairs(),
+                (('p', (('y', 'B'),)), ('q', (('y', 2),)), ('r', (('y', True),)))
+                )
+        self.assertEqual(g2, 1)
+
+        (g3, post3), (g4, post4) = list(f.iter_group_other_items(
+                axis=1, other=(1, 0, 1))
+                )
+        self.assertEqual(post3.to_pairs(),
+                (('q', (('z', 1), ('x', 2), ('w', 1), ('y', 2))),)
+                )
+        self.assertEqual(g3, 0)
+
+        self.assertEqual(post4.to_pairs(),
+                (('p', (('z', 'A'), ('x', 'A'), ('w', 'B'), ('y', 'B'))), ('r', (('z', False), ('x', True), ('w', False), ('y', True))))
+                )
+        self.assertEqual(g4, 1)
+
+    #---------------------------------------------------------------------------
+    def test_frame_iter_group_other_array_a(self) -> None:
+        columns = tuple('pq')
+        index = tuple('zxwy')
+        records = ((1, 10),
+                   (2, 15),
+                   (1, 20),
+                   (2, 25))
+        f = Frame.from_records(records, columns=columns, index=index)
+        # asxis 0 means for other grouping means that they key is a column key
+        post1, post2 = list(f.iter_group_other_array(
+                axis=0, other=(1, 0, 0, 1))
+                )
+        self.assertEqual(post1.tolist(),
+                [[2, 15], [1, 20]]
+                )
+        self.assertEqual(post2.tolist(),
+                [[1, 10], [2, 25]]
+                )
+
+        post3, post4 = list(f.iter_group_other_array(
+                axis=1, other=(0, 1))
+                )
+        self.assertEqual(post3.tolist(),
+                [[1], [2], [1], [2]]
+                )
+        self.assertEqual(post4.tolist(),
+                [[10], [15], [20], [25]]
+                )
+
+    #---------------------------------------------------------------------------
+    def test_frame_iter_group_other_array_items_a(self) -> None:
+        columns = tuple('pq')
+        index = tuple('zxwy')
+        records = ((1, 10),
+                   (2, 15),
+                   (1, 20),
+                   (2, 25))
+        f = Frame.from_records(records, columns=columns, index=index)
+        # asxis 0 means for other grouping means that they key is a column key
+        (g1, post1), (g2, post2) = list(f.iter_group_other_array_items(
+                axis=0, other=(1, 0, 0, 1))
+                )
+        self.assertEqual(g1, 0)
+        self.assertEqual(post1.tolist(),
+                [[2, 15], [1, 20]]
+                )
+        self.assertEqual(g2, 1)
+        self.assertEqual(post2.tolist(),
+                [[1, 10], [2, 25]]
+                )
+
+        (g3, post3), (g4, post4) = list(f.iter_group_other_array_items(
+                axis=1, other=(0, 1))
+                )
+        self.assertEqual(g3, 0)
+        self.assertEqual(post3.tolist(),
+                [[1], [2], [1], [2]]
+                )
+        self.assertEqual(g4, 1)
+        self.assertEqual(post4.tolist(),
+                [[10], [15], [20], [25]]
+                )
+
     #---------------------------------------------------------------------------
 
     def test_frame_reversed(self) -> None:
@@ -848,19 +1325,19 @@ class TestUnit(TestCase):
 
         f2 = f1[['p', 'q']]
 
-        s1 = f2.iter_array(axis=0).apply(np.sum)
+        s1 = f2.iter_array(axis=0).apply(np.sum) # type: ignore
         self.assertEqual(list(s1.items()), [('p', 150), ('q', 204)])
 
-        s2 = f2.iter_array(axis=1).apply(np.sum)
+        s2 = f2.iter_array(axis=1).apply(np.sum) # type: ignore
         self.assertEqual(list(s2.items()),
                 [('w', 3), ('x', 64), ('y', 149), ('z', 138)])
 
         def sum_if(idx: tp.Hashable, vals: tp.Iterable[int]) -> tp.Optional[int]:
             if idx in ('x', 'z'):
-                return tp.cast(int, np.sum(vals))
+                return np.sum(vals) # type: ignore
             return None
 
-        s3 = f2.iter_array_items(axis=1).apply(sum_if)
+        s3 = f2.iter_array_items(axis=1).apply(sum_if) # type: ignore
         self.assertEqual(list(s3.items()),
                 [('w', None), ('x', 64), ('y', None), ('z', 138)])
 
@@ -913,15 +1390,14 @@ class TestUnit(TestCase):
         self.assertEqual(post[0][1].to_pairs(0),
                 (('p', (('w', 2), ('x', 30), ('y', 2), ('z', 30))), ('q', (('w', 2), ('x', 34), ('y', 95), ('z', 73)))))
 
-        self.assertEqual(post[1][0], False)
+        self.assertEqual(post[1][0], 'a')
         self.assertEqual(post[1][1].to_pairs(0),
-                (('s', (('w', False), ('x', True), ('y', False), ('z', True))), ('t', (('w', False), ('x', False), ('y', False), ('z', True)))))
-
-        self.assertEqual(post[2][0], 'a')
-
-        self.assertEqual(post[2][1].to_pairs(0),
                 (('r', (('w', 'a'), ('x', 'b'), ('y', 'c'), ('z', 'd'))),))
 
+        self.assertEqual(post[2][0], False)
+        self.assertEqual(post[2][1].to_pairs(0),
+                (('s', (('w', False), ('x', True), ('y', False), ('z', True))), ('t', (('w', False), ('x', False), ('y', False), ('z', True)))))
+
     def test_frame_group_c(self) -> None:
         f = ff.parse('s(10,3)|v(int,str,bool)').assign[0].apply(lambda s: s % 4)
         post1 = tuple(f.iter_group(0, axis=0, drop=True))
@@ -978,7 +1454,6 @@ class TestUnit(TestCase):
         self.assertEqual(post[1][1].to_pairs(0),
                 (('p', (('x', 30), ('z', 30))), ('q', (('x', 34), ('z', 73))), ('r', (('x', 'b'), ('z', 'd'))), ('s', (('x', True), ('z', True))), ('t', (('x', False), ('z', True)))))
 
-
         s1 = f1.iter_group('p', axis=0).apply(lambda f: f['q'].values.sum())
         self.assertEqual(list(s1.items()), [(2, 97), (30, 107)])
 
