@@ -1,36 +1,60 @@
-import unittest
-import pickle
-import datetime
+from __future__ import annotations
+
 import copy
+import datetime
+import pickle
 import typing as tp
+import unittest
+from hashlib import sha256
 from io import StringIO
 
 import numpy as np
 from arraykit import mloc
 
-from static_frame import Index
-from static_frame import IndexGO
-from static_frame import IndexDate
-from static_frame import IndexDateGO
-from static_frame import IndexHierarchy
-from static_frame import Series
-from static_frame import IndexYear
+from static_frame import DisplayConfig
 from static_frame import Frame
 from static_frame import ILoc
+from static_frame import Index
 from static_frame import IndexAutoFactory
-from static_frame.test.test_case import TestCase
-from static_frame.core.index import _index_initializer_needs_init
+from static_frame import IndexDate
+from static_frame import IndexDateGO
+from static_frame import IndexGO
+from static_frame import IndexHierarchy
+from static_frame import IndexYear
+from static_frame import Series
 from static_frame.core.exception import ErrorInitIndex
-from static_frame.core.exception import LocInvalid
 from static_frame.core.exception import ErrorInitIndexNonUnique
-
+from static_frame.core.exception import LocInvalid
+from static_frame.core.index import _index_initializer_needs_init
+from static_frame.core.util import NULL_SLICE
 from static_frame.core.util import PositionsAllocator
 from static_frame.core.util import arrays_equal
-from static_frame.core.util import NULL_SLICE
+from static_frame.test.test_case import TestCase
 
+if tp.TYPE_CHECKING:
+    NDArrayAny = np.ndarray[tp.Any, tp.Any] # pylint: disable=W0611 #pragma: no cover
+    DtypeAny = np.dtype[tp.Any] # pylint: disable=W0611 #pragma: no cover
 
 class TestUnit(TestCase):
 
+    def test_iloc_repr(self) -> None:
+        self.assertEqual(repr(ILoc[1]), '<ILoc[1]>')
+        self.assertEqual(repr(ILoc[1,]), '<ILoc[1]>')
+        self.assertEqual(repr(ILoc[1, 2]), '<ILoc[1,2]>')
+        self.assertEqual(repr(ILoc[:, 1]), '<ILoc[:,1]>')
+        self.assertEqual(repr(ILoc[:, 1, 2]), '<ILoc[:,1,2]>')
+        self.assertEqual(repr(ILoc[1:2, :]), '<ILoc[1:2,:]>')
+        self.assertEqual(repr(ILoc[1:2, :2]), '<ILoc[1:2,:2]>')
+        self.assertEqual(repr(ILoc[1:2, :2, :3]), '<ILoc[1:2,:2,:3]>')
+        self.assertEqual(repr(ILoc[::1]), '<ILoc[:]>')
+        self.assertEqual(repr(ILoc[1:, 1:2, 1:2:3, :2, :2:3, ::3]), '<ILoc[1:,1:2,1:2:3,:2,:2:3,::3]>')
+        # self.assertEqual(repr(ILoc[()]), '<ILoc[()]>')
+        self.assertEqual(repr(ILoc[(1,),]), '<ILoc[(1,)]>')
+        self.assertEqual(repr(ILoc[(),]), '<ILoc[()]>')
+        self.assertEqual(repr(ILoc[:]), '<ILoc[:]>')
+        self.assertEqual(repr(ILoc[:, :]), '<ILoc[:,:]>')
+        self.assertEqual(repr(ILoc[:, :, 4]), '<ILoc[:,:,4]>')
+
     def test_positions_allocator_a(self) -> None:
 
         a1 = PositionsAllocator.get(3)
@@ -130,7 +154,24 @@ class TestUnit(TestCase):
 
     def test_index_init_l(self) -> None:
         with self.assertRaises(ErrorInitIndex):
-            idx = Index('bar')
+            _ = Index('bar')
+
+    def test_index_init_m(self) -> None:
+        with self.assertRaises(ErrorInitIndex):
+            _ = Index(np.array(('2021-02', '2022-04'), dtype=np.datetime64))
+
+    def test_index_init_n(self) -> None:
+        a1 = np.array((np.nan, np.nan, 3), dtype=np.float64)
+        a2 = a1.astype(np.float16)
+        a2.flags.writeable = False
+        idx = Index(a2)
+        self.assertEqual(idx.loc_to_iloc(3), 2)
+
+    def test_index_init_o(self) -> None:
+        a1 = np.array((10, 40, 20))
+        idx = Index(a1) # permit a mutable array
+        self.assertEqual(idx.values.tolist(), [10, 40, 20])
+
 
     #---------------------------------------------------------------------------
 
@@ -139,7 +180,7 @@ class TestUnit(TestCase):
         idx = Index(('a', 'b', 'c', 'd'))
 
         self.assertEqual(
-                tp.cast(np.ndarray, idx._loc_to_iloc(np.array([True, False, True, False]))).tolist(),
+                idx._loc_to_iloc(np.array([True, False, True, False])).tolist(), # type: ignore
                 [0, 2])
 
         self.assertEqual(idx._loc_to_iloc(slice('c',)), slice(None, 3, None))
@@ -149,7 +190,7 @@ class TestUnit(TestCase):
     def test_index_loc_to_iloc_b(self) -> None:
         idx = Index(('a', 'b', 'c', 'd'))
         post = idx._loc_to_iloc(Series(['b', 'c']))
-        self.assertEqual(post, [1, 2])
+        self.assertEqual(post.tolist(), [1, 2]) #type: ignore
 
     def test_index_loc_to_iloc_c(self) -> None:
         idx = Index(('a', 'b', 'c', 'd'))
@@ -171,7 +212,7 @@ class TestUnit(TestCase):
         with self.assertRaises(LocInvalid):
             _ = idx1.loc_to_iloc(slice('x', 'y'))
 
-        self.assertEqual(idx1.loc_to_iloc(['d', 'a']), [3, 0])
+        self.assertEqual(idx1.loc_to_iloc(['d', 'a']).tolist(), [3, 0]) #type: ignore
         with self.assertRaises(KeyError):
             _ = idx1.loc_to_iloc(['d', 'x'])
 
@@ -188,13 +229,15 @@ class TestUnit(TestCase):
             _ = idx2.loc_to_iloc(5)
 
         self.assertEqual(idx2.loc_to_iloc(slice(1, 3)), slice(1, 4))
+
         with self.assertRaises(LocInvalid):
             _ = idx2.loc_to_iloc(slice('x', 'y'))
+
         with self.assertRaises(LocInvalid):
             # loc slices are always interpreted as inclusive, so going beyond the inclusive boundary is an error
             _ = idx2.loc_to_iloc(slice(0, 4))
 
-        self.assertEqual(idx2.loc_to_iloc([3, 0]), [3, 0])
+        self.assertEqual(idx2.loc_to_iloc([3, 0]).tolist(), [3, 0]) #type: ignore
         with self.assertRaises(KeyError):
             _ = idx2.loc_to_iloc([3, 20])
 
@@ -218,10 +261,10 @@ class TestUnit(TestCase):
                 )
         # NOTE: this fails as we only see a list of dt64s and cannot match them in the AutoMap dictionary unless we were to directly examine and conert each element
         with self.assertRaises(KeyError):
-            _ = idx1.loc_to_iloc([dt64(d) for d in reversed(idx1)])
+            _ = idx1.loc_to_iloc([dt64(d) for d in reversed(idx1)]) # type: ignore
 
-        post = idx1.loc_to_iloc(np.array([dt64(d) for d in reversed(idx1)]))
-        self.assertEqual(post, [2, 1, 0])
+        post = idx1.loc_to_iloc(np.array([dt64(d) for d in reversed(idx1)])) # type: ignore
+        self.assertEqual(post.tolist(), [2, 1, 0]) #type: ignore
 
     def test_index_loc_to_iloc_g(self) -> None:
         dt = datetime.date
@@ -253,7 +296,7 @@ class TestUnit(TestCase):
         self.assertTrue(idx1._map is None)
 
         idx2 = idx1[:]
-        self.assertTrue(idx2._map is None) #type: ignore
+        self.assertTrue(idx2._map is None)
 
     def test_index_loc_to_iloc_j(self) -> None:
         idx1 = IndexAutoFactory.from_optional_constructor(10,
@@ -261,16 +304,38 @@ class TestUnit(TestCase):
         post = idx1.loc_to_iloc(NULL_SLICE)
         self.assertEqual(post, NULL_SLICE)
 
+    def test_index_loc_to_iloc_k(self) -> None:
+        idx1 = Index(range(4), loc_is_iloc=True)
+        self.assertTrue(idx1._map is None)
+        # for now, lists of Bools only work on indicies without maps
+        post = idx1.loc[[True, False, True, False]]
+        self.assertEqual(post.values.tolist(), [0, 2]) #type: ignore
+
+    def test_index_loc_to_iloc_l(self) -> None:
+        idx1 = Index(range(4), loc_is_iloc=True)
+        self.assertTrue(idx1._map is None)
+
+        post1 = idx1[Series((3, 1), index=('a', 'b'))]
+        post2 = idx1.loc_to_iloc(Series((3, 1), index=('a', 'b')))
+        self.assertEqual(post1.tolist(), post2.tolist()) #type: ignore
+
+    def test_index_loc_to_iloc_m(self) -> None:
+        idx1 = IndexGO(range(4), loc_is_iloc=True)
+        idx1.append(4)
+        self.assertTrue(idx1._map is None)
+        post1 = idx1.loc_to_iloc([3, 0])
+        self.assertEqual(post1.tolist(), [3, 0]) #type: ignore
+
     #---------------------------------------------------------------------------
 
     def test_index_mloc_a(self) -> None:
         idx = Index(('a', 'b', 'c', 'd'))
-        self.assertTrue(idx.mloc == idx[:2].mloc) #type: ignore
+        self.assertTrue(idx.mloc == idx[:2].mloc)
 
     def test_index_mloc_b(self) -> None:
         idx = IndexGO(('a', 'b', 'c', 'd'))
         idx.append('e')
-        self.assertTrue(idx.mloc == idx[:2].mloc) #type: ignore
+        self.assertTrue(idx.mloc == idx[:2].mloc)
 
     def test_index_dtype_a(self) -> None:
         idx = IndexGO(('a', 'b', 'c', 'd'))
@@ -354,13 +419,13 @@ class TestUnit(TestCase):
 
         self.assertEqual(idx.values.tolist(), ['a', 'b', 'c', 'd'])
 
-        self.assertEqual(idx[2:].values.tolist(), ['c', 'd']) #type: ignore
+        self.assertEqual(idx[2:].values.tolist(), ['c', 'd'])
 
         self.assertEqual(idx.loc['b':].values.tolist(), ['b', 'c', 'd'])  # type: ignore  # https://github.com/python/typeshed/pull/3024
 
         self.assertEqual(idx.loc['b':'d'].values.tolist(), ['b', 'c', 'd'])  # type: ignore  # https://github.com/python/typeshed/pull/3024
 
-        self.assertEqual(idx._loc_to_iloc(['b', 'b', 'c']), [1, 1, 2])
+        self.assertEqual(idx._loc_to_iloc(['b', 'b', 'c']).tolist(), [1, 1, 2]) #type: ignore
 
         self.assertEqual(idx.loc['c'], 'c')
 
@@ -534,16 +599,16 @@ class TestUnit(TestCase):
         self.assertEqual(len(index), 7)
 
         index3 = index[2:]
-        index3.append('i') #type: ignore
+        index3.append('i')
 
-        self.assertEqual(index3.values.tolist(), ['c', 'd', 'e', 'f', 'h', 'i']) #type: ignore
+        self.assertEqual(index3.values.tolist(), ['c', 'd', 'e', 'f', 'h', 'i'])
         self.assertEqual(index.values.tolist(), ['a', 'b', 'c', 'd', 'e', 'f', 'h'])
 
     def test_index_go_b(self) -> None:
 
         index = IndexGO(('a', 'b', 'c'))
         index.append('d')
-        self.assertEqual(len(index.__slots__), 8)
+        self.assertEqual(len(index.__slots__), 3)
         self.assertFalse(index.STATIC)
         self.assertEqual(index._IMMUTABLE_CONSTRUCTOR, Index)
         self.assertEqual(Index._MUTABLE_CONSTRUCTOR, IndexGO)
@@ -591,8 +656,8 @@ class TestUnit(TestCase):
         idx1 = IndexAutoFactory.from_optional_constructor(3,
                 default_constructor=IndexGO)
         idx1.append(3) # type: ignore
-        post = idx1._loc_to_iloc(np.array([True, False, True, False]), 2) #type: ignore
-        self.assertEqual(post.tolist(), [2, 4]) #type: ignore
+        post = idx1._loc_to_iloc(np.array([True, False, True, False]))
+        self.assertEqual(post.tolist(), [True, False, True, False]) #type: ignore
 
     #---------------------------------------------------------------------------
 
@@ -675,7 +740,6 @@ class TestUnit(TestCase):
             pbytes = pickle.dumps(index)
             index_new = pickle.loads(pbytes)
             for v in index: # iter labels
-                # import ipdb; ipdb.set_trace()
                 # this compares Index objects
                 self.assertFalse(index_new._labels.flags.writeable)
                 self.assertEqual(index_new.loc[v], index.loc[v])
@@ -707,7 +771,7 @@ class TestUnit(TestCase):
         idx = IndexGO(('a', 'b', 'c', 'd'))
         idx.append('e')
         post = idx._extract_iloc(slice(None))
-        self.assertEqual(post.values.tolist(), #type: ignore
+        self.assertEqual(post.values.tolist(),
                 ['a', 'b', 'c', 'd', 'e'])
 
     def test_index_drop_iloc_a(self) -> None:
@@ -795,7 +859,6 @@ class TestUnit(TestCase):
                 ['d', 'a', 'b', 'c'])
 
     #---------------------------------------------------------------------------
-
     def test_index_fillna_a(self) -> None:
 
         idx1 = Index(('a', 'b', 'c', None))
@@ -826,6 +889,14 @@ class TestUnit(TestCase):
                 [3.0, 2.0, 'foo'],
                 )
 
+    #---------------------------------------------------------------------------
+    def test_index_fillfalsy_a(self) -> None:
+
+        idx1 = Index(('a', 'b', 'c', ''))
+        idx2 = idx1.fillfalsy('d')
+        self.assertEqual(idx2.values.tolist(),
+                ['a', 'b', 'c', 'd'])
+
     #---------------------------------------------------------------------------
 
     def test_index_attributes_a(self) -> None:
@@ -1295,17 +1366,11 @@ class TestUnit(TestCase):
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
         self.assertEqual(idx1.tail(2).values.tolist(), ['d' ,'e'])
 
-    #---------------------------------------------------------------------------
-
     def test_index_via_str_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
         a1 = idx1.via_str.upper()
 
-        self.assertEqual(a1.tolist(),
-                ['A', 'B', 'C', 'D', 'E']
-                )
-
     def test_index_via_str_b(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
@@ -1340,6 +1405,15 @@ class TestUnit(TestCase):
                 [2, 4, 6, 2, 4]
                 )
 
+
+    def test_index_via_values_a(self) -> None:
+
+        idx1 = IndexGO((10, 20, 30))
+        idx1.append(40)
+        idx2 = idx1.via_values.apply(lambda x: (x * .5).astype(int))
+        self.assertEqual(idx2.__class__, IndexGO)
+        self.assertEqual(idx2.values.tolist(), [5, 10, 15, 20])
+
     #---------------------------------------------------------------------------
 
     def test_index_equals_a(self) -> None:
@@ -1417,7 +1491,7 @@ class TestUnit(TestCase):
         a = IndexDate([dt64('2021-01-01'), dt64('1954-01-01')])
         b = IndexYear([dt64('2021'), dt64('1954')])
 
-        self.assertFalse(arrays_equal(a, b, skipna=True))
+        self.assertFalse(arrays_equal(a.values, b.values, skipna=True))
 
     def test_index_equals_h(self) -> None:
         a = IndexGO([1, 2, 3])
@@ -1477,10 +1551,11 @@ class TestUnit(TestCase):
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
         self.assertEqual(idx1.loc_searchsorted('c'), 'c')
         self.assertEqual(idx1.loc_searchsorted('c', side_left=False), 'd')
-        self.assertEqual(idx1.loc_searchsorted(('a', 'c'), side_left=False).tolist(), ['b', 'd'])
+        self.assertEqual(idx1.loc_searchsorted(('a', 'c'), side_left=False).tolist(), # type: ignore
+                ['b', 'd'])
 
         self.assertEqual(idx1.loc_searchsorted(
-                ('a', 'e'), side_left=False, fill_value=None).tolist(),
+                ('a', 'e'), side_left=False, fill_value=None).tolist(), # type: ignore
                 ['b', None])
 
     def test_index_iloc_searchsorted_c(self) -> None:
@@ -1518,6 +1593,228 @@ class TestUnit(TestCase):
             (idx2.values_at_depth(0) == np.array(['2012', '2012', '2012'], dtype='datetime64[Y]')).all()
             )
 
+    #---------------------------------------------------------------------------
+
+    def test_index_iloc_map_a(self) -> None:
+
+        ih1 = Index(tuple('QafDeHbdc'))
+        ih2 = Index(tuple('ABcdHebDEsaQf'))
+
+        post = ih1._index_iloc_map(ih2)
+        expected= ih1.iter_label().apply(ih2._loc_to_iloc)
+        self.assertEqual(post.tolist(), expected.tolist())
+        self.assertEqual([11, 10, 12, 7, 5, 4, 6, 3, 2], post.tolist())
+
+    def test_index_iloc_map_b(self) -> None:
+
+        ih1 = Index(tuple('abcdefg'))
+        ih2 = Index(tuple('bcdefg'))
+
+        with self.assertRaises(KeyError):
+            ih1._index_iloc_map(ih2)
+
+        with self.assertRaises(KeyError):
+            ih1.iter_label().apply(ih2._loc_to_iloc)
+
+    #---------------------------------------------------------------------------
+    def test_index_extract_iloc_by_int(self) -> None:
+        idx = IndexGO(('a', 'b'))
+        idx.append('c')
+        post = idx._extract_iloc_by_int(2)
+        self.assertEqual(post, 'c')
+
+    #---------------------------------------------------------------------------
+    def test_index_dropna_a(self) -> None:
+        idx1 = Index((3.5, np.nan, 1.5))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [3.5, 1.5])
+
+    def test_index_dropna_b(self) -> None:
+        idx1 = Index((3.5, None, 1.5))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [3.5, 1.5])
+
+    def test_index_dropna_c(self) -> None:
+        idx1 = IndexDate(('2020-03', None, '1981-05-30'))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [datetime.date(2020, 3, 1), datetime.date(1981, 5, 30)])
+
+    def test_index_dropna_d(self) -> None:
+        idx1 = Index((None, np.nan))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [])
+
+    def test_index_dropna_e(self) -> None:
+        idx1 = Index((3, 4))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [3, 4])
+        self.assertIs(idx2, idx1)
+
+    def test_index_dropna_f(self) -> None:
+        idx1 = IndexGO((3, 4))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [3, 4])
+        self.assertIsNot(idx2, idx1)
+
+    def test_index_dropna_g(self) -> None:
+        idx1 = IndexDate(('2020-03', '1981-05-30'))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [datetime.date(2020, 3, 1), datetime.date(1981, 5, 30)])
+        self.assertIs(idx1, idx2)
+
+    def test_index_dropna_h(self) -> None:
+        idx1 = IndexDateGO(('2020-03', '1981-05-30'))
+        idx2 = idx1.dropna()
+        self.assertEqual(idx2.values.tolist(), [datetime.date(2020, 3, 1), datetime.date(1981, 5, 30)])
+        self.assertIsNot(idx1, idx2)
+
+    #---------------------------------------------------------------------------
+    def test_index_dropfalsy_a(self) -> None:
+        idx1 = Index((3.5, np.nan, 1.5))
+        idx2 = idx1.dropfalsy()
+        self.assertEqual(idx2.values.tolist(), [3.5, 1.5])
+
+    def test_index_dropfalsy_b(self) -> None:
+        idx1 = Index((0, '', None, 2))
+        idx2 = idx1.dropfalsy()
+        self.assertEqual(idx2.values.tolist(), [2])
+
+    #---------------------------------------------------------------------------
+    def test_index_display_a(self) -> None:
+        idx = IndexGO(('a', 'b', 'c', 'd'))
+        idx.append('e')
+        post = idx.display(DisplayConfig(type_show=False, type_color=False))
+        self.assertEqual(str(post), 'a\nb\nc\nd\ne')
+
+    #---------------------------------------------------------------------------
+    def test_index_hash_bytes_a(self) -> None:
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='')
+        bytes1 = idx1._to_signature_bytes()
+        self.assertEqual(sha256(bytes1).hexdigest(),
+            'c767ec91c4609de269307eb178d169503f5ae91f2e690cfc11a83c78b6687b1e')
+
+        idx2 = Index(('a', 'b', 'c', 'd'), name='')
+        bytes2 = idx2._to_signature_bytes()
+        self.assertEqual(sha256(bytes2).hexdigest(),
+            '108f99787a5b8c8acc45ebfcc934ad1a1eaedda394679c192d3b4be385590d93')
+
+    def test_index_hash_bytes_b(self) -> None:
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='')
+        bytes1 = idx1._to_signature_bytes(include_class=False)
+
+        idx2 = Index(('a', 'b', 'c', 'd'), name='')
+        bytes2 = idx2._to_signature_bytes(include_class=False)
+
+        self.assertEqual(
+                sha256(bytes1).hexdigest(),
+                sha256(bytes2).hexdigest(),
+                )
+
+    def test_index_hash_bytes_c(self) -> None:
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name=None)
+        with self.assertRaises(TypeError):
+            _ = idx1._to_signature_bytes()
+
+        bytes1 = idx1._to_signature_bytes(include_name=False)
+        self.assertEqual(sha256(bytes1).hexdigest(),
+            'c767ec91c4609de269307eb178d169503f5ae91f2e690cfc11a83c78b6687b1e')
+
+    def test_index_hash_bytes_d(self) -> None:
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='')
+        bytes1 = idx1._to_signature_bytes(include_class=False)
+
+        idx2 = Index(('a', 'b', 'c', 'd', 'e'), name='')
+        bytes2 = idx2._to_signature_bytes(include_class=False)
+
+        self.assertNotEqual(
+                sha256(bytes1).hexdigest(),
+                sha256(bytes2).hexdigest(),
+                )
+
+    def test_index_via_hashlib_a(self) -> None:
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='')
+
+        self.assertEqual(idx1.via_hashlib().sha256().hexdigest(),
+            'c767ec91c4609de269307eb178d169503f5ae91f2e690cfc11a83c78b6687b1e')
+
+    def test_index_get_argsort_cache_a(self) -> None:
+        idx1 = Index(('a', 'b', 'c', 'd'), name='')
+        idx2 = IndexGO(('a', 'b', 'c', 'd'), name='')
+
+        unique1, indexers1 = idx1._get_argsort_cache()
+        unique2, indexers2 = idx2._get_argsort_cache()
+
+        # Force re-cache
+        idx2.append("e")
+        unique3, indexers3 = idx2._get_argsort_cache()
+
+        assert (unique1 == unique2).all()
+        assert (indexers1 == indexers2).all()
+
+        assert unique1.size != unique3.size
+        assert indexers1.size != indexers3.size
+
+        assert unique1.size == indexers1.size
+        assert unique3.size == indexers3.size
+
+    def test_index_get_argsort_cache_b(self) -> None:
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='')
+
+        unique1, indexers1 = idx1._get_argsort_cache()
+        assert (unique1 == idx1.values).all()
+
+        idx2 = idx1.__deepcopy__({})
+
+        # Force re-cache
+        idx1.append("e")
+
+        idx3 = idx1.__deepcopy__({})
+
+        unique2, indexers2 = idx1._get_argsort_cache()
+        unique3, indexers3 = idx2._get_argsort_cache()
+        unique4, indexers4 = idx3._get_argsort_cache()
+
+        assert (unique2 == idx1.values).all()
+        assert (unique1 == idx2.values).all()
+        assert (unique2 == idx3.values).all()
+
+        assert (unique1 == unique3).all()
+        assert (unique2 == unique4).all()
+
+        assert (indexers1 == indexers3).all()
+        assert (indexers2 == indexers4).all()
+
+        assert len(unique1) == len(unique3) == len(indexers1) == len(indexers3) == 4
+        assert len(unique2) == len(unique4) == len(indexers2) == len(indexers4) == 5
+
+    def test_index_get_argsort_cache_c(self) -> None:
+        idx1 = Index(('a', 'b', 'c', 'd'), name='')
+        assert idx1._argsort_cache is None
+
+        idx2 = idx1.__deepcopy__({})
+        assert idx2._argsort_cache is None
+
+        unique1, indexers1 = idx1._get_argsort_cache()
+        assert idx1._argsort_cache is not None
+        idx3 = idx1.__deepcopy__({}) # type: ignore
+        assert idx3._argsort_cache is not None
+
+        unique2, indexers2 = idx2._get_argsort_cache()
+        unique3, indexers3 = idx3._get_argsort_cache()
+
+        assert len(unique1) == len(unique2) == len(indexers1) == len(indexers2) == len(unique3) == len(indexers3) == 4
+
+        assert unique1 is not unique2
+        assert unique1 is not unique3
+        assert unique2 is not unique3
+        assert indexers1 is not indexers2
+        assert indexers1 is not indexers3
+        assert indexers2 is not indexers3
+        assert (unique1 == unique2).all()
+        assert (unique2 == unique3).all()
+        assert (indexers1 == indexers2).all()
+        assert (indexers2 == indexers3).all()
+
 
 if __name__ == '__main__':
     unittest.main()
