@@ -1,25 +1,28 @@
+from __future__ import annotations
+
 import typing as tp
+from hashlib import sha256
 
 import frame_fixtures as ff
 import numpy as np
 
-from static_frame.test.test_case import TestCase
-
-from static_frame.core.quilt import Quilt
-from static_frame.core.hloc import HLoc
+from static_frame.core.axis_map import bus_to_hierarchy
+from static_frame.core.batch import Batch
+from static_frame.core.bus import Bus
 from static_frame.core.display_config import DisplayConfig
+from static_frame.core.exception import AxisInvalid
+from static_frame.core.exception import ErrorInitQuilt
+from static_frame.core.frame import Frame
+from static_frame.core.hloc import HLoc
 from static_frame.core.index import ILoc
 from static_frame.core.index import Index
-from static_frame.core.frame import Frame
-from static_frame.core.bus import Bus
+from static_frame.core.index_datetime import IndexDate
+from static_frame.core.index_datetime import IndexSecond
+from static_frame.core.quilt import Quilt
+from static_frame.core.store_config import StoreConfig
 from static_frame.core.yarn import Yarn
-from static_frame.core.batch import Batch
-from static_frame.core.store import StoreConfig
-
+from static_frame.test.test_case import TestCase
 from static_frame.test.test_case import temp_file
-from static_frame.core.exception import ErrorInitQuilt
-from static_frame.core.exception import AxisInvalid
-from static_frame.core.axis_map import bus_to_hierarchy
 
 
 class TestUnit(TestCase):
@@ -34,6 +37,7 @@ class TestUnit(TestCase):
                 name='f1')
         f2 = Frame.from_dict(
                 dict(a=(1,2,3), b=(4,5,6)),
+
                 index=('x', 'y', 'z'),
                 name='f2')
         f3 = Frame.from_dict(
@@ -230,10 +234,10 @@ class TestUnit(TestCase):
                 ((('f1', 'a'), (('x', 1), ('y', 2))), (('f1', 'c'), (('x', 3), ('y', 4))), (('f2', 'a'), (('x', 2), ('y', 3))), (('f2', 'b'), (('x', 4), ('y', 6))), (('f3', 'c'), (('x', 10), ('y', 20))), (('f3', 'b'), (('x', 50), ('y', 60)))))
 
     def test_quilt_from_frames_b(self) -> None:
-        index_a = Index(list(map(str, range(100))), name="a")
-        index_b = Index(list(map(str, range(100, 200))), name="b")
-        columns_a = Index(tuple("ABCDEFGHIJKLMNOPQRSTUVWXYZ"), name="a")
-        columns_b = Index(tuple("abcdefghijklmnopqrstuvwxyz"), name="b")
+        index_a = Index(list(map(str, range(100))), name='a')
+        index_b = Index(list(map(str, range(100, 200))), name='b')
+        columns_a = Index(tuple('ABCDEFGHIJKLMNOPQRSTUVWXYZ'), name='a')
+        columns_b = Index(tuple('abcdefghijklmnopqrstuvwxyz'), name='b')
 
         valid_construction_args = [
             (index_a, columns_a, index_a, columns_a, True, 0),
@@ -264,8 +268,8 @@ class TestUnit(TestCase):
 
         def build_quilt(args: tp.Any) -> Quilt:
             f1_index, f1_columns, f2_index, f2_columns, retain_labels, axis = args
-            f1 = Frame.from_element("f1", index=f1_index, columns=f1_columns, name="f1")
-            f2 = Frame.from_element("f2", index=f2_index, columns=f2_columns, name="f2")
+            f1 = Frame.from_element('f1', index=f1_index, columns=f1_columns, name='f1')
+            f2 = Frame.from_element('f2', index=f2_index, columns=f2_columns, name='f2')
             return Quilt.from_frames((f1, f2), retain_labels=retain_labels, axis=axis)
 
         # This checks that the init was successful, and that the display call also is successful
@@ -314,7 +318,7 @@ class TestUnit(TestCase):
 
         f2 = q1[['zZbu', 'zkuW']]
         self.assertEqual(f2.shape, (100, 2))
-        self.assertEqual(f2.columns.values.tolist(), ['zZbu', 'zkuW'])
+        self.assertEqual(f2.columns.values.tolist(), ['zZbu', 'zkuW']) # type: ignore
 
         f3 = q1.loc['zQuq':, 'zUvW':] #type: ignore
         self.assertEqual(f3.shape, (6, 2))
@@ -1203,6 +1207,7 @@ class TestUnit(TestCase):
 
     def test_quilt_iter_window_b1(self) -> None:
         from string import ascii_lowercase
+
         # indexes are heterogenous but columns are not
         def get_frame(scale: int = 1) -> Frame:
             return Frame(np.arange(12).reshape(4, 3) * scale, columns=('x', 'y', 'z'))
@@ -1237,6 +1242,7 @@ class TestUnit(TestCase):
 
     def test_quilt_iter_window_b2(self) -> None:
         from string import ascii_lowercase
+
         # indexes are heterogenous but columns are not
         def get_frame(scale: int = 1) -> Frame:
             return Frame(np.arange(12).reshape(4, 3) * scale, columns=('x', 'y', 'z'))
@@ -1298,7 +1304,7 @@ class TestUnit(TestCase):
         self.assertTrue(len(q1._bus), 5)
 
         s1 = q1.iter_window_array(size=5, step=4).apply(lambda a: a.sum())
-        self.assertEqual(s1.to_pairs(),
+        self.assertEqual(s1.to_pairs(), # type: ignore
                 (('zmVj', 377853), ('zr4u', 597832), ('zGDJ', 46996), ('zO5l', 391169)))
 
     def test_quilt_iter_window_array_b2(self) -> None:
@@ -1309,7 +1315,7 @@ class TestUnit(TestCase):
         self.assertTrue(len(q1._bus), 5)
 
         s1 = q1.iter_window_array(size=5, step=4).apply(lambda a: a.sum())
-        self.assertEqual(s1.to_pairs(),
+        self.assertEqual(s1.to_pairs(), # type: ignore
                 (('zmVj', 377853), ('zr4u', 597832), ('zGDJ', 46996), ('zO5l', 391169)))
 
     #---------------------------------------------------------------------------
@@ -1323,11 +1329,11 @@ class TestUnit(TestCase):
 
         # axis 0 will produce windows labelled by the Index
         s1 = q1.iter_window_array_items(size=2, step=1).apply(lambda _, a: a.sum())
-        self.assertEqual(round(s1, 2).to_pairs(),
+        self.assertEqual(round(s1, 2).to_pairs(), # type: ignore
                 ((1, 98797.88), (2, 305042.56), (3, 185974.34)))
 
         s2 = q1.iter_window_array_items(size=2, step=1, axis=1).apply(lambda _, a: a.sum())
-        self.assertEqual(round(s2, 2).to_pairs(),
+        self.assertEqual(round(s2, 2).to_pairs(), # type: ignore
                 ((('f1', 1), 105189.58), (('f1', 2), 173802.58), (('f1', 3), 179577.64), (('f2', 0), 7700.64), (('f2', 1), 1.0), (('f2', 2), 2.0), (('f2', 3), 4.0))
                 )
 
@@ -1539,6 +1545,21 @@ class TestUnit(TestCase):
 
     #---------------------------------------------------------------------------
 
+    def test_quilt_to_zip_npy_a(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|c(I,str)').rename('f2')
+        f3 = ff.parse('s(4,4)|v(bool)|c(I,str)').rename('f3')
+        q1 = Quilt.from_frames((f1, f2, f3), retain_labels=True, axis=1)
+
+        with temp_file('.zip') as fp:
+            q1.to_zip_npy(fp)
+            q2 = Quilt.from_zip_npy(fp, retain_labels=True, axis=1)
+
+            self.assertTrue(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
+
+    #---------------------------------------------------------------------------
+
     def test_quilt_equals_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
@@ -1614,6 +1635,103 @@ class TestUnit(TestCase):
 
         self.assertFalse(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
 
+    #---------------------------------------------------------------------------
+    def test_quilt_dt64_index_a(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int)|i(ID,dtD)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|i(ID,dtD)').rename('f2')
+
+        b1 = Bus.from_frames((f1, f2))
+        q1 = Quilt(b1, retain_labels=True)
+        self.assertTrue(q1.shape, (8, 4))
+        self.assertIs(q1.index.index_types[1], IndexDate)
+
+    def test_quilt_dt64_index_b(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int)|i(ID,dtD)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|i(IS,dts)').rename('f2')
+
+        b1 = Bus.from_frames((f1, f2))
+        q1 = Quilt(b1, retain_labels=True)
+        self.assertTrue(q1.shape, (8, 4))
+        self.assertIs(q1.index.index_types[1], IndexSecond)
+
+    #---------------------------------------------------------------------------
+    def test_quilt_to_signature_bytes_a(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int64)|i(ID,dtD)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|i(ID,dtD)').rename('f2')
+
+        b1 = Bus.from_frames((f1, f2))
+        q1 = Quilt(b1, retain_labels=True)
+        bytes1 = q1._to_signature_bytes(include_name=False)
+        self.assertEqual(sha256(bytes1).hexdigest(),
+            '96a4372e4a908a660f149152f5f7d2e099c9c51a1f12384325f7e98faa504006')
+
+    def test_quilt_to_signature_bytes_b(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int64)|i(ID,dtD)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|i(ID,dtD)').rename('f3')
+
+        b1 = Bus.from_frames((f1, f2))
+        q1 = Quilt(b1, retain_labels=True)
+        bytes1 = q1._to_signature_bytes(include_name=False)
+        self.assertNotEqual(sha256(bytes1).hexdigest(),
+            '96a4372e4a908a660f149152f5f7d2e099c9c51a1f12384325f7e98faa504006')
+
+    def test_quilt_via_hashlib_a(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int64)|i(ID,dtD)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|i(ID,dtD)').rename('f3')
+
+        b1 = Bus.from_frames((f1, f2))
+        q1 = Quilt(b1, retain_labels=True)
+
+        hd = q1.via_hashlib(include_name=False).sha256().hexdigest()
+        self.assertEqual(hd,
+            'b9caa5a602b91077d278d35f5558f3bad8d268100ec82842849a9c949de441df')
+
+    #------------------------------------------------------------------
+
+    def test_quilt_bus_a(self) -> None:
+
+        f1 = ff.parse('s(20,4)|v(int)|i(I,str)|c(I,str)')
+        q1 = Quilt.from_frame(f1, chunksize=5, axis=0, retain_labels=True)
+        b1 = q1.bus
+
+        self.assertEqual(b1.index.values.tolist(),
+            ['zZbu', 'z2Oo', 'zOyq', 'zjZQ'])
+
+    #------------------------------------------------------------------
+
+    def test_quilt_from_index_hierarchy_bus_consistent_index_a(self) -> None:
+        frame = ff.parse('f(Fg)|v(int,bool,str)|i((I,I),(str,int))|s(12,2)')
+        frame = frame.iloc[np.array([11, 0, 4, 9, 1, 10, 5, 2, 6, 3, 7, 8])]
+
+        f1 = frame.iloc[:6].rename("f1")
+        f2 = frame.iloc[6:].rename("f2")
+
+        bus = Bus.from_frames((f1, f2))
+        quilt = Quilt(bus, retain_labels=False, axis=0)
+
+        selected_index = quilt.loc[HLoc["ztsv"]].index
+
+        assert set(selected_index.values_at_depth(0)) == {"ztsv"}
+
+    def test_quilt_from_index_hierarchy_bus_consistent_index_b(self) -> None:
+        frame = ff.parse('f(Fg)|v(int,bool,str)|i((I,I),(str,int))|s(12,2)')
+        frame = frame.iloc[np.array([11, 0, 4, 9, 1, 10, 5, 2, 6, 3, 7, 8])]
+
+        f1 = frame.iloc[:6].rename("f1").T
+        f2 = frame.iloc[6:].rename("f2").T
+
+        bus = Bus.from_frames((f1, f2))
+        quilt = Quilt(bus, retain_labels=False, axis=1)
+
+        selected_index = quilt[HLoc["ztsv"]].columns # type: ignore
+
+        assert set(selected_index.values_at_depth(0)) == {"ztsv"}
+
 
 if __name__ == '__main__':
     import unittest
