@@ -1,10 +1,11 @@
+from __future__ import annotations
+
 import frame_fixtures as ff
 
-
+from static_frame import ILoc
 from static_frame import Series
 from static_frame import SeriesHE
 from static_frame.test.test_case import TestCase
-from static_frame import ILoc
 
 
 class TestUnit(TestCase):
@@ -65,7 +66,7 @@ class TestUnit(TestCase):
         s1 = SeriesHE.from_dict(dict(a=10, b=42))
         s2 = SeriesHE.from_dict(dict(x='foo', y='bar'), name=s1)
 
-        self.assertEqual(s2.name['b'], 42)
+        self.assertEqual(s2.name['b'], 42) # type: ignore
 
         s3 = Series.from_dict({s1: 40, s2: 1000})
 
