@@ -1,18 +1,19 @@
-from io import StringIO
+from __future__ import annotations
+
 import datetime
+from io import StringIO
 
 import numpy as np
 
+from static_frame.core.frame import Frame
 from static_frame.core.store_filter import STORE_FILTER_DEFAULT
 from static_frame.core.store_filter import STORE_FILTER_DISABLE
 from static_frame.core.store_filter import StoreFilter
-from static_frame.test.test_case import TestCase
-from static_frame.core.util import DTYPE_OBJECT_KIND
 from static_frame.core.util import DTYPE_COMPLEX_KIND
 from static_frame.core.util import DTYPE_FLOAT_KIND
+from static_frame.core.util import DTYPE_OBJECT_KIND
 from static_frame.core.util import NAT
-
-from static_frame.core.frame import Frame
+from static_frame.test.test_case import TestCase
 
 
 class TestUnit(TestCase):
@@ -225,7 +226,14 @@ class TestUnit(TestCase):
     def test_store_filter_format_inexact_element_c(self) -> None:
         sf1 = StoreFilter(
                 value_format_float_positional='{:.8f}',
-                value_format_float_scientific='{:.8f}')
+                value_format_float_scientific='{:.8f}',
+                # Use non-frozensets to test that they are converted
+                to_nan=('', 'nan', 'NaN', 'NAN', 'NULL', '#N/A'),
+                to_nat=(),
+                to_none=('None',),
+                to_posinf=('inf',),
+                to_neginf=('-inf',),
+                )
 
         self.assertEqual(
                 sf1._format_inexact_element(0.83255500, DTYPE_FLOAT_KIND),
