@@ -1,24 +1,26 @@
+from __future__ import annotations
+
 import datetime
+from hashlib import sha256
 
 import frame_fixtures as ff
 import numpy as np
 
-from static_frame.test.test_case import TestCase
-from static_frame.core.bus import Bus
-from static_frame.core.index_auto import IndexAutoFactory
-from static_frame.core.display_config import DisplayConfig
-from static_frame.core.yarn import Yarn
-from static_frame.core.frame import Frame
-from static_frame.test.test_case import temp_file
-from static_frame.core.exception import ErrorInitYarn
-from static_frame.core.exception import ErrorInitIndex
-from static_frame.core.exception import ErrorInitSeries
-from static_frame.core.index_datetime import IndexDate
-from static_frame import ILoc
 from static_frame import HLoc
+from static_frame import ILoc
+from static_frame.core.bus import Bus
+from static_frame.core.display_config import DisplayConfig
+from static_frame.core.exception import ErrorInitSeries
+from static_frame.core.exception import ErrorInitYarn
 from static_frame.core.exception import RelabelInvalid
+from static_frame.core.frame import Frame
+from static_frame.core.index_auto import IndexAutoFactory
+from static_frame.core.index_datetime import IndexDate
 from static_frame.core.index_hierarchy import IndexHierarchy
 from static_frame.core.series import Series
+from static_frame.core.yarn import Yarn
+from static_frame.test.test_case import TestCase
+from static_frame.test.test_case import temp_file
 
 
 class TestUnit(TestCase):
@@ -367,8 +369,10 @@ class TestUnit(TestCase):
                 (('f7', (4, 2)), ('f3', (4, 4)))
                 )
 
-        with self.assertRaises(ErrorInitIndex):
-            y2 = y1.loc[['f1', 'f7', 'f3']]
+        y2 = y1.loc[['f1', 'f7', 'f3']]
+        self.assertEqual(y2.shapes.to_pairs(),
+                (('f1', (4, 4)), ('f7', (4, 4)), ('f3', (4, 2)))
+                )
 
     def test_yarn_loc_e(self) -> None:
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -386,9 +390,9 @@ class TestUnit(TestCase):
 
         y1 = Yarn.from_buses((b1, b2, b3), retain_labels=False, name='foo')
         y2 = y1['f2':'f6'] #type: ignore
-        self.assertEqual(y2.shapes.to_pairs(),
+        self.assertEqual(y2.shapes.to_pairs(), #type: ignore
                 (('f2', (4, 4)), ('f3', (4, 4)), ('f4', (4, 4)), ('f5', (4, 4)), ('f6', (2, 4))))
-        self.assertEqual(y2['f5'].to_pairs(),
+        self.assertEqual(y2['f5'].to_pairs(), #type: ignore
                 ((0, ((0, 'zjZQ'), (1, 'zO5l'), (2, 'zEdH'), (3, 'zB7E'))), (1, ((0, 'zaji'), (1, 'zJnC'), (2, 'zDdR'), (3, 'zuVU'))), (2, ((0, 'ztsv'), (1, 'zUvW'), (2, 'zkuW'), (3, 'zmVj'))), (3, ((0, 'z2Oo'), (1, 'z5l6'), (2, 'zCE3'), (3, 'zr4u')))))
 
     def test_yarn_loc_f(self) -> None:
@@ -996,7 +1000,7 @@ class TestUnit(TestCase):
             y1.relabel({3,4,5})
 
         self.assertEqual(
-                y1.relabel(tuple('abcdef'))['d':].status['shape'].to_pairs(),
+                y1.relabel(tuple('abcdef'))['d':].status['shape'].to_pairs(), #type: ignore
                 (('d', (2, 8)), ('e', (4, 4)), ('f', (6, 4)))
                 )
 
@@ -1022,7 +1026,7 @@ class TestUnit(TestCase):
         y1 = Yarn((b1, b2, b3), index=IndexHierarchy.from_product(('a', 'b'), (1, 2, 3)))
 
         self.assertEqual(
-                y1.relabel_flat()[('a', 3):].status['shape'].to_pairs(),
+                y1.relabel_flat()[('a', 3):].status['shape'].to_pairs(), # type: ignore
                 ((('a', 3), (2, 2)), (('b', 1), (2, 8)), (('b', 2), (4, 4)), (('b', 3), (6, 4)))
                 )
 
@@ -1121,6 +1125,89 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             y1.rehierarch((1,0))
 
+    #---------------------------------------------------------------------------
+
+    def test_yarn_to_signature_bytes_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        b2 = Bus.from_frames((f4,))
+        b3 = Bus.from_frames((f5, f6))
+
+        a1 = np.array((1, 2, 3), dtype=np.int64)
+        a1.flags.writeable = False
+        y1 = Yarn((b1, b2, b3), index=IndexHierarchy.from_product(('a', 'b'), a1))
+
+        bytes1 = y1._to_signature_bytes(include_name=False)
+        self.assertEqual(sha256(bytes1).hexdigest(),
+            '0a6978231ec671903449e39ae465747a68a0da8492e9b5b5fcf4dc98afb8143e')
+
+    def test_yarn_to_signature_bytes_b(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        b2 = Bus.from_frames((f4,))
+        b3 = Bus.from_frames((f5, f6))
+
+        a1 = np.array((1, 2, 3), dtype=np.int64)
+        a1.flags.writeable = False
+        y1 = Yarn((b1, b2, b3), index=IndexHierarchy.from_product(('a', 'b'), a1))
+        bytes1 = y1._to_signature_bytes(include_name=False)
+
+        y2 = Yarn((b1, b2, b3))
+        bytes2 = y2._to_signature_bytes(include_name=False)
+
+        self.assertNotEqual(sha256(bytes1).hexdigest(), sha256(bytes2).hexdigest())
+
+    def test_yarn_to_signature_bytes_c(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        b2 = Bus.from_frames((f4,))
+        b3 = Bus.from_frames((f5, f6))
+        b4 = Bus.from_frames((f5,))
+
+        y1 = Yarn((b1, b2, b3))
+        bytes1 = y1._to_signature_bytes(include_name=False)
+
+        y2 = Yarn((b1, b2, b4))
+        bytes2 = y2._to_signature_bytes(include_name=False)
+
+        self.assertNotEqual(sha256(bytes1).hexdigest(), sha256(bytes2).hexdigest())
+
+    def test_yarn_via_hashlib_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3))
+        b2 = Bus.from_frames((f4,))
+        b3 = Bus.from_frames((f5, f6))
+
+        y1 = Yarn((b1, b2, b3))
+        digest = y1.via_hashlib(include_name=False).sha256().hexdigest()
+        self.assertEqual(digest, '694e92cccad9bc2ae2f6bf9e0cb212bfd4b0677f79c155c89ece4dcaf7311324')
+
+
+
 
 if __name__ == '__main__':
     import unittest
